/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
#include <Base/Common/Singleton.hh>

using namespace BIAS;
using namespace std;

class A {
public:
  double data;

  /// declare the singleton friend, because its needs to access
  /// the constructor
  friend class Singleton<A>;
private:
  /// make sure that class A cannot be used as non-singleton by
  /// declaring constructors and copy operator private
  A() : data(0) {}
  A(const A &a) : data(a.data) {}
  A& operator=(const A& a) { data = a.data; return *this; }
};

typedef Singleton<A> ASingleton;

void function()
{
  // set the singleton with 5
  ASingleton::GetInstance()->data = 5;
}

/** @author woelk 10/2007 (c) www.vision-n.de */
int main(int argc, char *argv[])
{
  cout << "initial data: " << ASingleton::GetInstance()->data << endl;

  function();

  cout << "after function: " << ASingleton::GetInstance()->data << endl;
  
  cout << "accessing Singleton<A> is identical to ASingleton: "
       <<Singleton<A>::GetInstance()->data << endl;
  
  // does not compile, because constructor of A is private
  //A a; 

  return 0;
}
