/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/TimeStamp.hh>

using namespace BIAS;

bool TimeStamp::operator>(const TimeStamp &ts) const
{
  return msec_ > ts.GetMSec();
}

bool TimeStamp::operator>(const long long &ts) const
{
  return msec_ > ts;
}

bool TimeStamp::operator<(const TimeStamp &ts) const
{
  return msec_ < ts.GetMSec();
}

bool TimeStamp::operator<(const long long &ts) const
{
  return msec_ < ts;
}

bool TimeStamp::operator>=(const TimeStamp &ts) const
{
  return msec_ >= ts.GetMSec();
}

bool TimeStamp::operator>=(const long long &ts) const
{
  return msec_ >= ts;
}

bool TimeStamp::operator<=(const TimeStamp &ts) const
{
  return msec_ <= ts.GetMSec();
}

bool TimeStamp::operator<=(const long long &ts) const
{
  return msec_ <= ts;
}

bool TimeStamp::operator==(const TimeStamp &ts) const
{
  return msec_ == ts.GetMSec();
}

bool TimeStamp::operator==(const long long &ts) const
{
  return msec_ == ts;
}

bool TimeStamp::operator!=(const TimeStamp &ts) const 
{
  return msec_ != ts.GetMSec();
}

bool TimeStamp::operator!=(const long long &ts) const
{
  return msec_ != ts;
}

TimeStamp& TimeStamp::operator=(const TimeStamp &ts) 
{
  msec_ = ts.GetMSec();
  return *this;
}

TimeStamp& TimeStamp::operator=(const long long &ts) 
{
  msec_ = ts; 
  return *this;
}

TimeStamp TimeStamp::operator+(const TimeStamp &ts) const
{
  return TimeStamp(msec_ + ts.GetMSec());
}

TimeStamp TimeStamp::operator+(const long long &ts) const
{
  return TimeStamp(msec_ + ts);
}

TimeStamp TimeStamp::operator-(const TimeStamp &ts) const
{
  return TimeStamp(msec_ - ts.GetMSec());
}

TimeStamp TimeStamp::operator-(const long long &ts) const
{
  return TimeStamp(msec_ - ts);
}

TimeStamp TimeStamp::operator*(const TimeStamp &ts) const
{
  return TimeStamp(msec_ * ts.GetMSec());
}

TimeStamp TimeStamp::operator*(const long long &ts) const
{
  return TimeStamp(msec_ * ts);
}

 TimeStamp TimeStamp::operator/(const TimeStamp &ts) const
{
  return TimeStamp(msec_ / ts.GetMSec());
}

TimeStamp TimeStamp::operator/(const long long &ts) const
{
  return TimeStamp(msec_ / ts);
}

TimeStamp TimeStamp::operator%(const TimeStamp &ts) const
{
  return TimeStamp(msec_ % ts.GetMSec());
}

TimeStamp TimeStamp::operator%(const long long &ts) const
{
  return TimeStamp(msec_ % ts);
}

std::ostream& BIAS::operator<<(std::ostream& os, const TimeStamp &ts)
{
  os << "Timestamp: " << ts.GetMSec() << "msec";
  return os;
}
