/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _W32Compat_hh_
#define _W32Compat_hh_

#include <bias_config.h>

/// @author Felix Woelk
/// @author Jan Woetzel


// Windows:
#ifdef WIN32

// avoid deprecated warnings in VS 2005
#ifndef _CRT_SECURE_NO_DEPRECATE
#  define _CRT_SECURE_NO_DEPRECATE
#endif 
#if !defined(_SCL_SECURE_NO_DEPRECATE) && !defined(_SECURE_SCL_DEPRECATE)
#  define _SCL_SECURE_NO_DEPRECATE
#endif
//#  define _CRT_NONSTDC_NO_DEPRECATE
//#  define _AFX_SECURE_NO_DEPRECATE 
//#  define _ATL_SECURE_NO_DEPRECATE


// avoid definition of min and max macro in windows.h/windef.h JW 12/2004)
#  define NOMINMAX 

// switch for math.h to define M_PI, M_PI_2 in WIN32
#  define _USE_MATH_DEFINES


///
/// implement Unix quasi-standard functions for code compatibility
///

// implement non-standard (double)rint(double x)
#  define rint(x) (floor(x+0.5))
#  define rintf(x) (floor(x+0.5))
#  define cbrt(x) (pow(x,1.0/3.0))
#  define trunc(x) (x<0?floor(x+1):floor(x))
#  define exp2(x) (exp(x * log(2.0)))
#  define log2(x) (log(x)/log(2.0))




/* Return nonzero value if X is a NaN.  We could use `fpclassify' but
   we already have this functions `__isnan' and it is faster.  */
#  define isnan(x) (x==x ? false : true)

/* Return nonzero value is X is positive or negative infinity.  */
#  define isinf(x) (sizeof(x)==sizeof(float) ? (x>FLT_MAX||x<-FLT_MAX)?true:false :  \
	(sizeof(x)==sizeof(double)) ? (x>DBL_MAX||x<-DBL_MAX)?true:false  : \
	x>LDBL_MAX || x<-LDBL_MAX?true:false)

// User need to include windows.h !! It is not included here to avoid clashes
# define usleep(usec_) Sleep((unsigned long)(usec_ /1000.0))
# define  sleep(sec_)  Sleep((unsigned long)(sec_  *1000.0))


 /* timeval.h    1.0 01/12/19
  * Defines gettimeofday, timeval, etc. for Win32
  * copied from  Wu Yongwei */
#define EPOCHFILETIME (116444736000000000LL)

struct timezone {
  int tz_minuteswest; /* minutes W of Greenwich */
  int tz_dsttime;     /* type of dst correction */
 };

BIASCommon_EXPORT int gettimeofday(struct timeval *tv, struct timezone *tz);


/// @brief wrapper for mkdir with Unix mode arg ignored
/// Requires on WIN32:
/// \verbatim
/// #include <direct.h>
/// \endverbatim
/// @author Jan Woetzel
BIASCommon_EXPORT int mkdir(const char *dirname, int mode_IsIgnored );

/// @author woelk 04/2007
BIASCommon_EXPORT double copysign(double x, double y);

#endif // WIN32

#endif //_W32Compat_hh_
