
#include <Base/Math/Random.hh>

#include <Base/Geometry/EParametrization.hh>
#include <Base/Geometry/HomgPoint3D.hh>
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Geometry/RMatrixBase.hh>
#include <Base/Geometry/PMatrixBase.hh>
#include <Base/Geometry/KMatrix.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  Random r;
  HomgPoint3D p3d;
  p3d[0] = r.GetUniformDistributed(-5.0, 5.0);
  p3d[1] = r.GetUniformDistributed(-5.0, 5.0);
  p3d[2] = r.GetUniformDistributed(10, 15.0);
  p3d[3] = 1.0;

  KMatrix K(MatrixIdentity);
  Vector3<double> C1(0,0,0), C2(1,0,0);
  RMatrixBase R1(MatrixIdentity), R2(MatrixIdentity), R;
  R2.SetXYZ(0.01, -0.01, 0.01);
  R = R1.Transpose() * R2;
  PMatrixBase P1(K, R1, C1), P2(K, R2, C2);
  HomgPoint2D p1=P1*p3d, p2=P2*p3d, epp=P1*HomgPoint3D(C2);
  cout << "p1: "<<p1<<"\tp2: "<<p2<<"\nepipole: "<<epp<<endl;
  Quaternion<double> q;
  R.GetQuaternion(q);
  EParametrization ep, epi;
  ep.SetEpipole(epp);
  ep.SetOrientation(q);
  epi = ep;
  epi.Invert();

  Matrix3x3<double> E = ep.GetEssentialMatrix(), Ei = epi.GetEssentialMatrix();

  cout << p1.ScalarProduct(E*p2) << "\t"
    //<< p2.ScalarProduct(E*p1) << "\n"
    //<< p1.ScalarProduct(Ei*p2) << "\t"
       << p2.ScalarProduct(Ei*p1) << "\n";
  
}
