/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _HomgPoint2DCov_hh_
#define _HomgPoint2DCov_hh_

#include <Base/Math/Matrix2x2.hh>
#include <Base/Math/Matrix3x3.hh>
#include <Base/Common/CompareFloatingPoint.hh>

namespace BIAS {

  class HomgPoint2D;

  /** @class HomgPoint2DCov
      @brief class representing the covariance matrix of a homogenous point 2D 
      @author woelk 03/2006 */
  class BIASGeometryBase_EXPORT HomgPoint2DCov
    : public Matrix3x3<double>
  {
  public:
    HomgPoint2DCov();

    HomgPoint2DCov(const MatrixInitType& i);

    explicit HomgPoint2DCov(const Matrix2x2<double>& m);

    explicit HomgPoint2DCov(const Matrix3x3<double>& m);

    explicit HomgPoint2DCov(const Matrix<double>& m);

    explicit HomgPoint2DCov(const TNT::Matrix<double>& m);

    virtual ~HomgPoint2DCov();
    
    /** @brief homogenizes p and *this
        @author woelk 03/2006 */
    void Homogenize(HomgPoint2D& p);

    /** @brief normalizes p and *this
        @author woelk 03/2006 */
    void Normalize(HomgPoint2D& p);

    Matrix2x2<double> GetEuclidean() const;

    void SetEuclidean(const Matrix2x2<double>& m);

    inline bool IsHomogenized() const {
      return (Equal((*this)[0][2], 0.0) && Equal((*this)[1][2], 0.0) &&
              Equal((*this)[2][0], 0.0) && Equal((*this)[2][1], 0.0) &&
              Equal((*this)[2][2], 0.0));
    }

    inline bool IsZero() const {
      return ( Equal((*this)[0][0], 0.0) && Equal((*this)[0][1], 0.0) &&
               Equal((*this)[0][2], 0.0) && Equal((*this)[1][0], 0.0) &&
               Equal((*this)[1][1], 0.0) && Equal((*this)[1][2], 0.0) &&
               Equal((*this)[2][0], 0.0) && Equal((*this)[2][1], 0.0) &&
               Equal((*this)[2][2], 0.0) );
    }

    /** @brief averages transposed elements to make symmetric */
    void MakeSymmetric();

    /** @brief checks whether matrix is symmetric */
    bool CheckSymmetry() const;
    
    /** @brief Sets from matrix and enforces symmetry */
    void SetAndMakeSymmetric(const Matrix3x3<double> &m);    

  };
    
}

#endif // _HomgPoint2DCov_hh_
