/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _HomgPoint3DCov_hh_
#define _HomgPoint3DCov_hh_

#include <Base/Math/Matrix4x4.hh>
#include <Base/Math/Vector4.hh>
#include <Base/Common/CompareFloatingPoint.hh>

namespace BIAS {

  template <class T> class BIASMathBase_EXPORT Matrix3x3;
  class HomgPoint3D;

  /** @class HomgPoint3DCov
      @brief class representing the covariance matrix of a homogenous point 3D 
      @author woelk 03/2006 */
  class BIASGeometryBase_EXPORT HomgPoint3DCov
    : public Matrix4x4<double>
  {
  public:
    HomgPoint3DCov();

    HomgPoint3DCov(const MatrixInitType& i);

    HomgPoint3DCov(const Matrix3x3<double>& m);

    HomgPoint3DCov(const Matrix4x4<double>& m);

    HomgPoint3DCov(const Matrix<double>& m);

    HomgPoint3DCov(const TNT::Matrix<double>& m);

    virtual ~HomgPoint3DCov();
    
    /** @brief homogenizes p and *this
        @author woelk 03/2006 */
    void Homogenize(HomgPoint3D& p);

    Matrix3x3<double> GetEuclidean() const;

    void SetEuclidean(const Matrix3x3<double>& m);

    inline bool IsHomogenized() const {
      return (Equal((*this)[0][3], 0.0) && Equal((*this)[1][3], 0.0) &&
              Equal((*this)[2][3], 0.0) && Equal((*this)[3][0], 0.0) &&
              Equal((*this)[3][1], 0.0) && Equal((*this)[3][2], 0.0) &&
              Equal((*this)[3][3], 0.0));
    }

    inline bool IsZero() const {
      return ( Equal((*this)[0][0], 0.0) && Equal((*this)[0][1], 0.0) &&
               Equal((*this)[0][2], 0.0) && Equal((*this)[0][3], 0.0) &&
               Equal((*this)[1][0], 0.0) && Equal((*this)[1][1], 0.0) &&
               Equal((*this)[1][2], 0.0) && Equal((*this)[1][3], 0.0) &&
               Equal((*this)[2][0], 0.0) && Equal((*this)[2][1], 0.0) &&
               Equal((*this)[2][2], 0.0) && Equal((*this)[2][3], 0.0) &&
               Equal((*this)[3][0], 0.0) && Equal((*this)[3][1], 0.0) &&
               Equal((*this)[3][2], 0.0) && Equal((*this)[3][3], 0.0) );
    }
  protected:
    inline bool CheckSymmetry_() const
    { return ( Equal((*this)[0][1], (*this)[1][0]) && 
               Equal((*this)[0][2], (*this)[2][0]) && 
               Equal((*this)[0][3], (*this)[3][0]) && 
               Equal((*this)[1][2], (*this)[2][1]) &&
               Equal((*this)[1][3], (*this)[3][1]) &&
               Equal((*this)[2][3], (*this)[3][2]) ); }
               
  inline void MakeSymmetric_() { 
    (*this)[1][0] = (*this)[0][1] = ((*this)[0][1] + (*this)[1][0])/2.0;
    (*this)[2][0] = (*this)[0][2] = ((*this)[0][2] + (*this)[2][0])/2.0;
    (*this)[2][1] = (*this)[1][2] = ((*this)[1][2] + (*this)[2][1])/2.0;
  }

               

  };
    
    
}

#endif // _HomgPoint3DCov_hh_
