/**
@example ExampleImageWinApi.cpp
Demonstrates loading a .bmp file 
via MS Windows system calls without any third party library.
@author Jan Woetzel
*/

#include <bias_config.h>

#include <Base/Debug/Error.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Common/FileHandling.hh>
#include <iostream>


using namespace std;
using namespace BIAS;

// globals
// filename
string fn = BIAS_TESTS_DATA "r4_small.bmp";
// run a darttest?
bool darttest=false;


// parse command line parameters
int parseCmdLine(int argc, char** argv){

  // parse args
  if (argc>1) {
    if (strcmp(argv[1],"-darttest")==0){
      if (argc>2){
        darttest=true;
        fn=argv[2]; // exe --darttest <file.bmp>
      } else {
        fn=argv[1]; // exe <file.bmp>
      }
    }
  }
  return 0;
}

// usage: 
// ExampleImageWinApi [--darttest] [<file.bmp>]
int main( int argc, char** argv)
{ 
  if (argc>0)
    cout<<"started "<<argv[0]<<endl;

  parseCmdLine(argc, argv);

  // check if file exists/readable at all
  if (!BIAS::FileReadable(fn)){
    cout<<"file "<<fn<<" does not exists/is unreadable due to file system problems!";
    return -1;
  }

  ImageBase img;
  // load the image (bmp)
  int res = ImageIO::Import_BITMAP_winapi(fn, img );
  // check result
  if (res==0){
    cout<<"successfully loaded "
      <<img.GetWidth()<<"x"<<img.GetHeight()<<" image from "<<fn<<endl;
  } else {
    cout<<"could not load image from "<<fn<<endl
      <<"is it a simple BMP file?"<<endl;
  }

#ifdef BIAS_HAVE_OPENCV
  // show image to screen
  if (!darttest)
    img.Display();
#endif

  return 0;
}

