/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
#ifndef __GenerateGauss_hh__
#define __GenerateGauss_hh__

#include <cmath>

namespace BIAS {

  // forwrad declaration
  template <class T> class Vector;
  template <class T> class Matrix;

  /** @class GenerateGauss
      @brief compute one and two dimensional gauss normal distribution 
      @author woelk 01/2008 (c) www.vision-n.de */
  class GenerateGauss
  {
  public:
    template <class T>
    static void Gauss1DSigma(const double &sigma, const double &ratio, 
                             BIAS::Vector<T> &result,const bool normalize=true);

    template <class T>
    static void Gauss1DSize(const unsigned size, const double &ratio,
                            BIAS::Vector<T> &result, double & sigma, 
                            const bool normalize=true);

    template <class T>
    static void Gauss1D(const unsigned size, const double &sigma,
                        BIAS::Vector<T> &result, const bool normalize = true);
    
    template <class T>
    static void Gauss2DSigma(const double &sigma, const double &ratio,
                             BIAS::Matrix<T> &result,const bool normalize=true);

    template <class T>
    static void Gauss2DSize(const unsigned size, const double &ratio,
                            BIAS::Matrix<T> &result, double &sigma, 
                            const bool normalize=true);

    template <class T>
    static void Gauss2D(const unsigned size, const double &sigma,
                        BIAS::Matrix<T> &result, const bool normalize = true);

  protected:
    /// Given a sigma and fixed ratio between biggest and smallest entry
    /// of the discrete gauss distribution, compute the required size of the
    /// vector
    inline static void Sigma2Size_(const double &sigma, const double &ratio, 
                                   unsigned &size)
    { size = (unsigned)ceil(2.0*sqrt(-2.0*sigma*sigma*log(ratio))+1.0); }

    /// Given a size and fixed ratio between biggest and smallest entry
    /// of the discrete gauss distribution, compute the required sigma of the
    /// distribution
    inline static void Size2Sigma_(const unsigned size, const double &ratio,
                                   double &sigma)
    { const double dx=(double)(size-1)*0.5; sigma=sqrt(-dx*dx*0.5/log(ratio)); }
  }; // 

  

} // namespace


#endif // __GenerateGauss_hh__
