# -Find (compiler flags) for OpenMP support, see www.openmp.org
# Once run this will define: 
#
# OpenMP_FOUND
# 
# NOTES
# tested with vs8 and gcc 4.2
#
# USAGE
#
# FIND_PACKAGE(OPENMP)
# # method (1):  add flags globally: 
# SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} ${OPENMP_FLAG}")
# 
# # method (2): add flags conditionnaly per target (preferred by jw)
# ## conditionally add openmp compiler flags
# IF(OPENMP_FOUND)
#  ## executables
#  SET_TARGET_PROPERTIES( fooEXE
#    PROPERTIES COMPILE_FLAGS ${OPENMP_FLAG} 
#    )
#  ## libraries
#  SET_TARGET_PROPERTIES( barLIB
#    PROPERTIES COMPILE_FLAGS ${OPENMP_FLAG} )
# ENDIF(OPENMP_FOUND)

# AUTHOR    Jan Woetzel
# COPYRIGHT 2006,2007,2008 Jan Woetzel <woetzelweb.de> 
# LICENSE:  GPL, see  http://www.gnu.org/licenses/agpl.txt




## check OpenMP support
INCLUDE(CheckFunctionExists)
MESSAGE(STATUS "Check for compiler OpenMP support...")
SET(OPENMP_FLAG)
SET(OPENMP_FLAG_FOUND FALSE)
SET(
  OPENMP_FLAGS
  "-openmp"   # cl, icc (/ and - works both)
  "/openmp"   # the above shoudl work, too.
  "-fopenmp"  # gcc
  #   "-mp"       # SGI & PGI
  #   "-xopenmp"  # Sun
  #   "-omp"      # Tru64
  #   "-qsmp=omp" # AIX
  )
SET(OPENMP_FUNCTION omp_set_num_threads)

FOREACH(FLAG ${OPENMP_FLAGS})
  IF(NOT OPENMP_FLAG_FOUND)
     MESSAGE(STATUS "Check if \"${FLAG}\" OpenMP flag is working...")
    
    SET(CMAKE_REQUIRED_FLAGS ${FLAG})
    CHECK_FUNCTION_EXISTS(${OPENMP_FUNCTION} OPENMP_FUNCTION_${FLAG}_FOUND)
    
    IF   (OPENMP_FUNCTION_${FLAG}_FOUND)
#      MESSAGE(" \"${FLAG}\" works.")
      SET(OPENMP_FLAG ${FLAG})
      SET(OPENMP_FLAG_FOUND TRUE)
      SET(OPENMP_FOUND TRUE)
    ELSE(OPENMP_FUNCTION_${FLAG}_FOUND)
#      MESSAGE( " \"${FLAG}\" does not work.")
    ENDIF(OPENMP_FUNCTION_${FLAG}_FOUND)
  ENDIF(NOT OPENMP_FLAG_FOUND)
ENDFOREACH(FLAG)

IF   (OPENMP_FOUND)
  IF   (NOT OPENMP_FIND_QUIETLY)
    MESSAGE(STATUS "OpenMP support detected as flag: ${OPENMP_FLAG}")
  ENDIF(NOT OPENMP_FIND_QUIETLY)    
ELSE (OPENMP_FOUND)
  IF   (OPENMP_FIND_REQUIRED AND NOT OPENMP_FIND_QUIETLY)
    MESSAGE(FATAL_ERROR
      "OpenMP required but no supporting compiler flags found.")
  ENDIF(OPENMP_FIND_REQUIRED AND NOT OPENMP_FIND_QUIETLY)
ENDIF(OPENMP_FOUND)

