# 
# Try to find OpenEXR, 
# a high dynamic rang image library.
# (see  www.openexr.org)
# 
# Once run this will define: 
# 
# OPENEXR_FOUND
# OPENEXR_INCLUDE_DIR
# OPENEXR_LIBRARIES
# OPENEXR_LINK_DIRECTORIES
#
# Jan Woetzel 07/2004
# www.mip.informatik.uni-kiel.de/~jw
#

# tested with
# -Linux Suse 9.2,  gcc 3.3: 1.21, 1.22
# -Linux Suse 10.1, gcc 4.1: 1.22
# -MS Windows XP, Visual Studio .NET 2003 7.1: (1.21, 1.22, 1.30)
# -MS Windows XP, Visual Studio 2005 (9.0): 1.4.0 (precompiled binaries)


#
# set (additional) directories to search for headers: 
#
SET(OPENEXR_POSSIBLE_INC_DIRS
  "$ENV{OPENEXR_DIR}/vc/vc7/include"
  "$ENV{OPENEXR_DIR}/vc/vc7/include/OpenEXR"
  "$ENV{OPENEXR_DIR}/vc/vc7/include/IlmImf"
  "$ENV{OPENEXR_DIR}/vc/vc7/include/Imath"
  "$ENV{OPENEXR_DIR}/vc/vc7/include/Iex" 

  "$ENV{OPENEXR_DIR}/include"
  "$ENV{OPENEXR_DIR}/include/OpenEXR"
  "$ENV{OPENEXR_DIR}/include/IlmImf"
  "$ENV{OPENEXR_DIR}/include/Imath"
  "$ENV{OPENEXR_DIR}/include/Iex"

  "$ENV{OPENEXR_HOME}/include"
  "$ENV{OPENEXR_HOME}/include/OpenEXR"
  "$ENV{OPENEXR_HOME}/include/IlmImf"
  "$ENV{OPENEXR_HOME}/include/Imath"
  "$ENV{OPENEXR_HOME}/include/Iex" 

  "$ENV{EXTERN_LIBS_DIR}/OpenEXR/include"
  "$ENV{EXTERN_LIBS_DIR}/OpenEXR/include/OpenEXR"

  "$ENV{EXTRA}/include"
  "$ENV{EXTRA}/include/OpenEXR"
  "$ENV{EXTRA}/include/IlmImf"
  "$ENV{EXTRA}/include/Imath"
  "$ENV{EXTRA}/include/Iex" 
  
  /usr/include/OpenEXR
  /usr/local/include/OpenEXR
  )


# MIP Uni Kiel /opt/net network installation 
# get correct prefix for current gcc compiler version for gcc 3.x  4.x
IF    (${CMAKE_COMPILER_IS_GNUCXX})
  EXEC_PROGRAM(${CMAKE_CXX_COMPILER} ARGS --version OUTPUT_VARIABLE CXX_COMPILER_VERSION)  
  #for gcc3x use /opt/net/gcc33/OpenEXR
  IF   (CXX_COMPILER_VERSION MATCHES ".*3\\.[0-9].*")
    SET(IS_GNUCXX3 TRUE)
    SET(OPENEXR_POSSIBLE_INC_DIRS ${OPENEXR_POSSIBLE_INC_DIRS}
         /opt/net/gcc33/OpenEXR
         /opt/net/gcc33/OpenEXR/include
         /opt/net/gcc33/OpenEXR/include/OpenEXR
    )
  ENDIF(CXX_COMPILER_VERSION MATCHES ".*3\\.[0-9].*")  
  #for gcc4x use /opt/net/gcc41/OpenEXR-1.2.2_moved
  IF   (CXX_COMPILER_VERSION MATCHES ".*4\\.[0-9].*")
    SET(IS_GNUCXX4 TRUE)
    SET(OPENEXR_POSSIBLE_INC_DIRS ${OPENEXR_POSSIBLE_INC_DIRS}
         /opt/net/gcc41/OpenEXR-1.2.2_moved
         /opt/net/gcc41/OpenEXR-1.2.2_moved/include
         /opt/net/gcc41/OpenEXR-1.2.2_moved/include/OpenEXR
         )
  ENDIF(CXX_COMPILER_VERSION MATCHES ".*4\\.[0-9].*")
ENDIF (${CMAKE_COMPILER_IS_GNUCXX})


#MESSAGE("DBG OPENEXR_POSSIBLE_INC_DIRS=${OPENEXR_POSSIBLE_INC_DIRS}")

##
# find specific headers fo rinclude dir(s):
##
FIND_PATH(OPENEXR_HALF_INCLUDE_DIR half.h
  ${OPENEXR_POSSIBLE_INC_DIRS} )
#MESSAGE("DBG OPENEXR_HALF_INCLUDE_DIR=${OPENEXR_HALF_INCLUDE_DIR}")

FIND_PATH(OPENEXR_IMFRGBA_INCLUDE_DIR ImfRgba.h
  ${OPENEXR_POSSIBLE_INC_DIRS}  )
#MESSAGE("DBG OPENEXR_IMFRGBA_INCLUDE_DIR=${OPENEXR_IMFRGBA_INCLUDE_DIR}")

FIND_PATH(OPENEXR_IEX_INCLUDE_DIR Iex.h
  ${OPENEXR_POSSIBLE_INC_DIRS}  )

FIND_PATH(OPENEXR_IMATHBOX_INCLUDE_DIR ImathBox.h
  ${OPENEXR_POSSIBLE_INC_DIRS}  )


##
# find (all) required libraries: 
##

# set additional possible dirs to search for libs: 
SET(OPENEXR_POSSIBLE_LIB_DIRS 
  "${OPENEXR_DIR}/vc/vc7/lib"
  "$ENV{OPENEXR_DIR}/vc/vc7/lib"
  
  "${OPENEXR_DIR}/lib"
  "${OPENEXR_DIR}"
  "$ENV{OPENEXR_DIR}/lib" 	 
  "$ENV{OPENEXR_DIR}"

  "${OPENEXR_HOME}/lib"
  "${OPENEXR_HOME}"
  "$ENV{OPENEXR_HOME}/lib"
  "$ENV{OPENEXR_HOME}"
  
  # libs may reside in subdirs on Windows or source tree on Linux
  "$ENV{OPENEXR_HOME}/lib/half"
  "$ENV{OPENEXR_HOME}/Half"
  "${OPENEXR_DIR}/lib/half"
  "${OPENEXR_DIR}/Half"
  
  "$ENV{OPENEXR_HOME}/lib/iex"
  "$ENV{OPENEXR_HOME}/Iex"
  "${OPENEXR_DIR}/lib/iex"
  "${OPENEXR_DIR}/Iex"

  "$ENV{OPENEXR_HOME}/lib/ilmimf"
  "$ENV{OPENEXR_HOME}/ilmimf"
  "${OPENEXR_DIR}/lib/ilmimf"
  "${OPENEXR_DIR}/ilmimf"

  "$ENV{OPENEXR_HOME}/lib/imath"
  "$ENV{OPENEXR_HOME}/imath"
  "${OPENEXR_DIR}/lib/imath"
  "${OPENEXR_DIR}/imath"

  "$ENV{EXTERN_LIBS_DIR}/OpenEXR/lib"

  /usr/lib
  /usr/local/lib  
  "$ENV{EXTRA}/lib"
  )


# MIP Uni Kiel /opt/net network installation 
# get correct prefix for current gcc compiler version for gcc 3.x  4.x
IF    (${CMAKE_COMPILER_IS_GNUCXX})
  EXEC_PROGRAM(${CMAKE_CXX_COMPILER} ARGS --version OUTPUT_VARIABLE CXX_COMPILER_VERSION)  
  #for gcc3x use /opt/net/gcc33/OpenEXR
  IF   (CXX_COMPILER_VERSION MATCHES ".*3\\.[0-9].*")
    SET(IS_GNUCXX3 TRUE)
    SET(OPENEXR_POSSIBLE_LIB_DIRS ${OPENEXR_POSSIBLE_LIB_DIRS}
        /opt/net/gcc33/OpenEXR
        /opt/net/gcc33/OpenEXR/lib
        )
  ENDIF(CXX_COMPILER_VERSION MATCHES ".*3\\.[0-9].*")  
  #for gcc4x use /opt/net/gcc41/OpenEXR-1.2.2_moved
  IF   (CXX_COMPILER_VERSION MATCHES ".*4\\.[0-9].*")
    SET(IS_GNUCXX4 TRUE)
    SET(OPENEXR_POSSIBLE_LIB_DIRS ${OPENEXR_POSSIBLE_LIB_DIRS}
         /opt/net/gcc41/OpenEXR-1.2.2_moved
         /opt/net/gcc41/OpenEXR-1.2.2_moved/lib
        )
  ENDIF(CXX_COMPILER_VERSION MATCHES ".*4\\.[0-9].*")
ENDIF (${CMAKE_COMPILER_IS_GNUCXX})


#MESSAGE("DBG OPENEXR_POSSIBLE_LIB_DIRS=${OPENEXR_POSSIBLE_LIB_DIRS}")


FIND_LIBRARY(OPENEXR_Half_STATIC_LIBRARY
  NAMES Half 
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
#MESSAGE("DBG OPENEXR_Half_STATIC_LIBRARY=${OPENEXR_Half_STATIC_LIBRARY}")
IF (WIN32)
  FIND_LIBRARY(OPENEXR_Half_STATIC_DEBUG_LIBRARY
    NAMES Halfd Half_d
    PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
  #MESSAGE("DBG OPENEXR_Half_STATIC_DEBUG_LIBRARY=${OPENEXR_Half_STATIC_DEBUG_LIBRARY}")
ENDIF(WIN32)

FIND_LIBRARY(OPENEXR_IlmImf_STATIC_LIBRARY
  NAMES IlmImf 
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
#MESSAGE("DBG OPENEXR_IlmImf_STATIC_LIBRARY=${OPENEXR_IlmImf_STATIC_LIBRARY}")
IF (WIN32)
  FIND_LIBRARY(OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY
    NAMES IlmImfd  IlmImf_d
    PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
  #MESSAGE("DBG OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY=${OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY}")
ENDIF(WIN32)

FIND_LIBRARY(OPENEXR_Imath_STATIC_LIBRARY
  NAMES Imath 
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
#MESSAGE("DBG OPENEXR_Imath_STATIC_LIBRARY=${OPENEXR_Imath_STATIC_LIBRARY}")
IF(WIN32)
  FIND_LIBRARY(OPENEXR_Imath_STATIC_DEBUG_LIBRARY
    NAMES Imathd Imath_d
    PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})
  #MESSAGE("DBG OPENEXR_Imath_STATIC_DEBUG_LIBRARY=${OPENEXR_Imath_STATIC_DEBUG_LIBRARY}")
ENDIF(WIN32)


FIND_LIBRARY(OPENEXR_Iex_STATIC_LIBRARY
  NAMES Iex 
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})  
#MESSAGE("DBG OPENEXR_Iex_STATIC_LIBRARY=${OPENEXR_Iex_STATIC_LIBRARY}")
IF (WIN32)
  FIND_LIBRARY(OPENEXR_Iex_STATIC_DEBUG_LIBRARY
    NAMES Iexd Iex_d 
    PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})  
  #MESSAGE("DBG OPENEXR_Iex_STATIC_DEBUG_LIBRARY=${OPENEXR_Iex_STATIC_DEBUG_LIBRARY}")
ENDIF(WIN32)


# from 1.30 on there is an additinal lib (which does not exist in 1.22)
FIND_LIBRARY(OPENEXR_IlmThread_STATIC_LIBRARY
  NAMES IlmThread
  PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})  
#MESSAGE("DBG OPENEXR_IlmThread_STATIC_LIBRARY=${OPENEXR_IlmThread_STATIC_LIBRARY}")
IF (WIN32)
  FIND_LIBRARY(OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY
    NAMES IlmThreadd IlmThread_d 
    PATHS ${OPENEXR_POSSIBLE_LIB_DIRS})  
  #MESSAGE("DBG OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY=${OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY}")
ENDIF(WIN32)

SET(OPENEXR_z_STATIC_LIB_POSSIBLE_PATHS
#"${OPENEXR_POSSIBLE_LIB_DIRS}"
  /lib
  /usr/lib
  /usr/local/lib
  "${OPENEXR_DIR}/lib"
  "${OPENEXR_HOME}/lib"
  "$ENV{OPENEXR_DIRE}/lib"
  "$ENV{OPENEXR_HOME}/lib"
  "${OPENEXR_DIR}/lib/zlib"
  "${OPENEXR_HOME}/lib/zlib"
  "$ENV{OPENEXR_DIR}/lib/zlib"
  "$ENV{OPENEXR_HOME}/lib/zlib"
  "$ENV{EXTERN_LIBS_DIR}/OpenEXR/lib"
  "$ENV{EXTRA}/lib"  
  )

#MESSAGE("Search path: ${OPENEXR_z_STATIC_LIB_POSSIBLE_PATHS}")

FIND_LIBRARY(OPENEXR_z_STATIC_LIBRARY
  NAMES z zlib zlib_a
  PATHS ${OPENEXR_z_STATIC_LIB_POSSIBLE_PATHS}  
  )
#MESSAGE("DBG OPENEXR_z_LIBRARY=${OPENEXR_z_STATIC_LIBRARY}")

IF (WIN32)
  FIND_LIBRARY(OPENEXR_z_STATIC_DEBUG_LIBRARY
    NAMES zd zlibd z_d zlib_d      
    PATHS
    /lib
    /usr/lib
    /usr/local/lib
    "${OPENEXR_DIR}/lib"
    "${OPENEXR_HOME}/lib"
    "$ENV{OPENEXR_DIRE}/lib"
    "$ENV{OPENEXR_HOME}/lib"
    "${OPENEXR_DIR}/lib/zlib"
    "${OPENEXR_HOME}/lib/zlib"
    "$ENV{OPENEXR_DIR}/lib/zlib"
    "$ENV{OPENEXR_HOME}/lib/zlib"
    "$ENV{EXTERN_LIBS_DIR}/OpenEXR/lib"
    "$ENV{EXTRA}/lib"
    )
  #MESSAGE("DBG OPENEXR_z_LIBRARY=${OPENEXR_z_STATIC_DEBUG_LIBRARY}")
ENDIF(WIN32)


# OK so far (until something is missing)
SET(OPENEXR_FOUND TRUE)

##
# evaluate results: 
##
IF   (NOT OPENEXR_HALF_INCLUDE_DIR)
  SET(OPENEXR_FOUND FALSE)
ENDIF(NOT OPENEXR_HALF_INCLUDE_DIR)

IF   (NOT OPENEXR_IMFRGBA_INCLUDE_DIR)
  SET(OPENEXR_FOUND FALSE)
ENDIF(NOT OPENEXR_IMFRGBA_INCLUDE_DIR)

IF   (NOT OPENEXR_IEX_INCLUDE_DIR )
  SET(OPENEXR_FOUND FALSE)
ENDIF(NOT OPENEXR_IEX_INCLUDE_DIR )

IF   (NOT OPENEXR_IMATHBOX_INCLUDE_DIR) 
  SET(OPENEXR_FOUND FALSE)
ENDIF(NOT OPENEXR_IMATHBOX_INCLUDE_DIR)


IF   (NOT OPENEXR_IlmImf_STATIC_LIBRARY)
  IF   (NOT OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY)
    SET(OPENEXR_FOUND FALSE)
    MESSAGE(SEND_ERROR "IlmImf not found")
  ENDIF(NOT OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY)
ENDIF(NOT OPENEXR_IlmImf_STATIC_LIBRARY)

IF   (NOT OPENEXR_Imath_STATIC_LIBRARY)
  IF   (NOT OPENEXR_Imath_STATIC_DEBUG_LIBRARY)
    SET(OPENEXR_FOUND FALSE)
    MESSAGE(SEND_ERROR " Imath not found")
  ENDIF(NOT OPENEXR_Imath_STATIC_DEBUG_LIBRARY)
ENDIF(NOT OPENEXR_Imath_STATIC_LIBRARY)

IF   (NOT OPENEXR_Iex_STATIC_LIBRARY)
  IF   (NOT OPENEXR_Iex_STATIC_DEBUG_LIBRARY)
    SET(OPENEXR_FOUND FALSE)
    MESSAGE(SEND_ERROR " Iex not found")
  ENDIF(NOT OPENEXR_Iex_STATIC_DEBUG_LIBRARY)
ENDIF(NOT OPENEXR_Iex_STATIC_LIBRARY)

IF   (NOT OPENEXR_z_STATIC_LIBRARY)
  IF   (NOT OPENEXR_z_STATIC_DEBUG_LIBRARY)
    SET(OPENEXR_FOUND FALSE)
    MESSAGE(SEND_ERROR " zlib not found")
  ENDIF(NOT OPENEXR_z_STATIC_DEBUG_LIBRARY)    
ENDIF(NOT OPENEXR_z_STATIC_LIBRARY)


SET(OPENEXR_INCLUDE_DIR 
  "${OPENEXR_HALF_INCLUDE_DIR}"
  "${OPENEXR_IMFRGBA_INCLUDE_DIR}"
  "${OPENEXR_IEX_INCLUDE_DIR}"
  "${OPENEXR_IMATHBOX_INCLUDE_DIR}"
  )
#MESSAGE("DBG in Find OPENEXR_INCLUDE_DIR=${OPENEXR_INCLUDE_DIR}")

IF  (WIN32)

  SET(OPENEXR_LIBRARIES 
    debug     ${OPENEXR_Half_STATIC_DEBUG_LIBRARY}
    optimized ${OPENEXR_Half_STATIC_LIBRARY} )

  SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
    debug     ${OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY}
    optimized ${OPENEXR_IlmImf_STATIC_LIBRARY} )

  SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
    debug     ${OPENEXR_Imath_STATIC_DEBUG_LIBRARY}
    optimized ${OPENEXR_Imath_STATIC_LIBRARY} )

  SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
    debug     ${OPENEXR_Iex_STATIC_DEBUG_LIBRARY} 
    optimized ${OPENEXR_Iex_STATIC_LIBRARY} )

  SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
    debug     ${OPENEXR_z_STATIC_DEBUG_LIBRARY} 
    optimized ${OPENEXR_z_STATIC_LIBRARY} )

  # new lib in 1.30: 
  IF   (OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY)
    SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
      debug     ${OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY} )  
  ENDIF(OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY)
  IF   (OPENEXR_IlmThread_STATIC_LIBRARY)
    SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES}    
      optimized ${OPENEXR_IlmThread_STATIC_LIBRARY} )
  ENDIF(OPENEXR_IlmThread_STATIC_LIBRARY)


ELSE(WIN32)
  IF   (UNIX) 

    SET(OPENEXR_LIBRARIES 
      ${OPENEXR_Half_STATIC_LIBRARY} )

    SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
      ${OPENEXR_IlmImf_STATIC_LIBRARY} )

    SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
      ${OPENEXR_Imath_STATIC_LIBRARY} )

    SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
      ${OPENEXR_Iex_STATIC_LIBRARY} )

    SET(OPENEXR_LIBRARIES ${OPENEXR_LIBRARIES} 
      ${OPENEXR_z_STATIC_LIBRARY} )

  ENDIF(UNIX)
ENDIF(WIN32)
# MESSAGE("DBG OPENEXR_LIBRARIES=${OPENEXR_LIBRARIES}") 


# HACK: use just one of the potentially multiple paths: 
GET_FILENAME_COMPONENT(OPENEXR_LINK_DIRECTORIES ${OPENEXR_IlmImf_STATIC_LIBRARY} PATH)


MARK_AS_ADVANCED(
  OPENEXR_HALF_INCLUDE_DIR

  OPENEXR_Half_STATIC_LIBRARY
  OPENEXR_Half_STATIC_DEBUG_LIBRARY

  OPENEXR_IlmImf_STATIC_LIBRARY
  OPENEXR_IlmImf_STATIC_DEBUG_LIBRARY

  OPENEXR_Imath_STATIC_LIBRARY
  OPENEXR_Imath_STATIC_DEBUG_LIBRARY

  OPENEXR_Iex_STATIC_LIBRARY
  OPENEXR_Iex_STATIC_DEBUG_LIBRARY

  OPENEXR_IEX_INCLUDE_DIR 
  OPENEXR_IMATHBOX_INCLUDE_DIR
  OPENEXR_IMFRGBA_INCLUDE_DIR

  OPENEXR_IlmThread_STATIC_LIBRARY
  OPENEXR_IlmThread_STATIC_DEBUG_LIBRARY

  OPENEXR_z_STATIC_LIBRARY
  OPENEXR_z_STATIC_DEBUG_LIBRARY
  )



# ==========================================
IF(NOT OPENEXR_FOUND)
  # make FIND_PACKAGE friendly
  IF(NOT OPENEXR_FIND_QUIETLY)
    IF(OPENEXR_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "OPENEXR required, please specify it's location.")
    ELSE(OPENEXR_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: OPENEXR was not found.")
    ENDIF(OPENEXR_FIND_REQUIRED)
  ENDIF(NOT OPENEXR_FIND_QUIETLY)
ENDIF(NOT OPENEXR_FOUND)

# make FIND_PACKAGE case sensitive compatible
SET(OpenEXR_FOUND            ${OPENEXR_FOUND})
SET(OpenEXR_LIBRARIES        ${OPENEXR_LIBRARIES})
SET(OpenEXR_INCLUDE_DIR      ${OPENEXR_INCLUDE_DIR})
SET(OpenEXR_LINK_DIRECTORIES ${OPENEXR_LINK_DIRECTORIES})
