# 
# This module finds if VisionN is available and determines where the
# include files and libraries are. 
# On Unix/Linux it relies on the output of mip-config.
# This code sets the following variables:
#
#
# VN_FOUND            = system has VisionN lib
#
# VN_LIBRARIES        = full path to the libraries
#
# VN_INCLUDE_DIR      = where to find headers 
#
# VN_LINK_DIRECTORIES = link directories, useful for rpath on Unix
# VN_EXE_LINKER_FLAGS = rpath on Unix#
#
#
# typical USAGE in user projects CMakeLists.txt: 
#    FIND_PACKAGE(VisionN REQUIRED)
#    IF (NOT VN_FOUND)
#      MESSAGE(FATAL_ERROR "VisionN was not found. Please set VN_DIR pointing to VisionNConfig.cmake build dir.")
#    ENDIF (NOT VN_FOUND)
#    IF (VN_CONFIG_CMAKE_SCRIPT)
#      INCLUDE(${VN_USE_FILE})
#    ENDIF (VN_CONFIG_CMAKE_SCRIPT)
#
# author Felix Woelk 

FIND_FILE(VN_CONFIG_CMAKE_SCRIPT VNConfig.cmake 
  ${VN_ROOT_DIR}
  $ENV{VN_ROOT_DIR}
  ${VN_DIR}
  $ENV{VN_DIR}
  "${CMAKE_CURRENT_BINARY_DIR}/../VisionN")

IF(VN_CONFIG_CMAKE_SCRIPT)

  #MESSAGE("Found: ${VN_CONFIG_CMAKE_SCRIPT}")
  INCLUDE(${VN_CONFIG_CMAKE_SCRIPT})
  SET(VN_FOUND 1)
  MESSAGE(STATUS "FindVisionN is Using ${VN_CONFIG_CMAKE_SCRIPT}")

ELSE(VN_CONFIG_CMAKE_SCRIPT)

  IF(NOT VisionN_FIND_QUIETLY)
    IF(VisionN_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "VisionN required, please specify it's location.")
    ELSE(VisionN_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: VisionN was not found.")
    ENDIF(VisionN_FIND_REQUIRED)
  ENDIF(NOT VisionN_FIND_QUIETLY)
  
ENDIF(VN_CONFIG_CMAKE_SCRIPT)


