#ifndef __CheckerboardDetector_hh_572718_
#define __CheckerboardDetector_hh_572718_

#include <bias_config.h>
#ifndef BIAS_HAVE_OPENCV
#  error Please recompile BIAS with USE_OPENCV to use this code. 
#endif 

#include <Base/Image/ImageBase.hh>

#include <vector>

#include <cv.h>
//#include <highgui.h>

namespace BIAS {

  /// helper vector of coords
  typedef std::vector< std::pair<float,float> >  t_CoordVecf;


  /**
  @class CheckerboardDetector
  @brief Detect am asymmetric checkerboard pattern, typically 7x4 inner corners.
  Based on OpenCV implementation. 
  @author Jan Woetzel (copyright 2006) */
  class BIASFeatureDetector_EXPORT CheckerboardDetector {
  public:

    //~CheckerboardDetector(){};
    //CheckerboardDetector(){}


    /** detect asymmetric checker pattern 
    @return 0 on OK, other: error, e.g. could not detect all inner corners */
    static int Compute(BIAS::Image<unsigned char> & img 
      ,const unsigned int & xCorners
      ,const unsigned int & yCorners
      ,t_CoordVecf & coords
      ,const bool & saveDebugImage=false
      );


    /** takes number of inner corners on x axis and y axis, 
    and number of images to builds an array of corners.  
    Assumes all corners on z = 0 plane. */
    static void DrawPoints(CvArr *img
      ,CvPoint2D32f *ptArray
      //,t_CoordVecf & coords
      ,int numPts
      ,const int xCorners
      ,const int yCorners
      ,const bool DRAW_IDEAL_OVERLAY=true
      ,const bool DRAW_AXES=true
      ,const bool DRAW_POINT_LABEL=true
      );

  }; // end of class
} // end of namespace
#endif
