/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __CornerDetectorBase_hh__
#define __CornerDetectorBase_hh__

#include <bias_config.h>

#include <string>

#include <Base/Image/Image.hh>
#include <Base/Geometry/HomgPoint2D.hh>

#define QUAL float

#define D_CD_SUSAN          0x00000001
#define D_CD_SUSAN_QUALITY  0x00000002
#define D_CD_WRITE_DEBUG_IM 0x00000004
#define D_CD_FEATURES       0x00000008
#define D_CD_INIT           0x00000010

namespace BIAS {

  /** @class CornerDetectorBase
      @ingroup g_feature
      @brief purly virtual interface defining class for corner detectors
      @author woelk 09/2004 */
  template <class StorageType>
  class BIASFeatureDetector_EXPORT CornerDetectorBase : public Debug
  {
  public:
    /** the differnet types of corner detectors */
    enum ECDType {CDT_Invalid, CDT_Susan, CDT_Harris, 
                  CDT_Foerstner, CDT_KLT, CDT_Laplace};
  public:
    CornerDetectorBase();
    virtual ~CornerDetectorBase();

    /** purely virtual interface definition */
    virtual int Detect(const Image<StorageType>& image, 
                       std::vector<HomgPoint2D>& p, 
                       std::vector<QUAL>& quality);
    
    ///////////////////////////////////////////////////////////////////////

    inline void SetMaxNumFeatures(const int maxnum)
    { _MaxNum = maxnum; }
    /** @brief @author Dennis Herzog */
    inline int GetMaxNumFeatures()
    { return _MaxNum; }

  protected:
    /** @todo implement tiling mechanism */
    int _NumTilesX, _NumTilesY;

    /// maximum number of corners to return
    int _MaxNum;
  }; // class


} // namespace

#endif // __CornerDetectorBase_hh__
