/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __CornerDetectorHarris_hh__
#define __CornerDetectorHarris_hh__

#include "CornerDetectorGradient.hh"

namespace BIAS {

  /** @class CornerDetectorHarris
      @ingroup g_feature
      @brief harris corner detector, detects grey value corners in images
      @bug CalculationType int does not work, use float
      @author koeser 09/2004 */
  template <class StorageType, class CalculationType>
  class BIASFeatureDetector_EXPORT CornerDetectorHarris 
    : public CornerDetectorGradient<StorageType, CalculationType>
  {
  public:
    CornerDetectorHarris();
    ~CornerDetectorHarris();

 
  protected:
    /// weight of trace in harris approximation
    double _Scale;

    /** @brief Computes the Harris approximation of the smaller eigenvalue of 
        the structure tensor over the image and stores it in _Cornerness. 

        c = det - scale * trace
        Also fills up _FeatList.
        @author koeser 09/2004 
        @todo optimize for ROI usage */
    virtual int _ComputeCornerness(Image<CalculationType>& Cornerness);
  }; // class

} // namespace


#endif // __CornerDetectorHarris_hh__
