/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Filter/DeInterlace.hh>
#include <Filter/Median.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Debug/TimeMeasure.hh>
using namespace BIAS;
using namespace std;

int main(int argc, char *argv[]) {

  DeInterlace<float, float> deinterlace;
  Median<float, float> median;

  // set parameters
  deinterlace.SetUseEvenLines(true);
  deinterlace.SetDeInterlaceType(BIAS_DI_121);
  deinterlace.SetDownSamplingByTwo(true);

  ImageBase ucim;
  Image<float> im;
  Image<float> med;
  Image<float> dst;

  if (argc<4) {
    cerr << argv[0] << "useEvenLines(bool) dodownsamplingbytwo(bool) src1 src2 ... \n";
    return -2;
  }

  // loop through all images
  for (int countImgs = 0; countImgs < (argc-3); countImgs++) {
    if (ImageIO::Load(argv[countImgs + 3], ucim)!=0) {
      BIASERR("error loading image "<<argv[countImgs + 3]);
      return -1;
    } else {
      cerr << "read "<<argv[countImgs + 3]<<endl;
    }

    if (ImageConvert::ConvertST(ucim, im, ImageBase::ST_float)!=0) {
      BIASERR("error converting image "<<argv[countImgs + 3]);
    }

    //    cerr << "Size of inputImage " << ucim.GetWidth() << " " << ucim.GetHeight() << endl;
    //    cerr << "Size of FloatImage " << im.GetWidth() << " " << im.GetHeight() << endl;

    if (im.GetChannelCount() != 1) {
      if (ImageConvert::IP_ToGrey(im)!=0) {
        BIASERR("error converting image "<<argv[countImgs + 3]);
      }
    }

    //   median.Filter(im, med);
    deinterlace.SetUseEvenLines( 0!=atoi(argv[2]) );
    // deinterlace.SetDownSamplingByTwo( 0!=atoi(argv[4]) );
    deinterlace.SetDeInterlaceType(BIAS_DI_121);
    deinterlace.Filter(im, med);
    //    cerr << "Done with deinterlacer" << endl;
    median.SetSize(0, 2);
    median.Filter(med, dst);

    //    cerr << "Size of ResImage " << dst.GetWidth() << " " << dst.GetHeight() << endl;

    ostringstream name;
    name << "deint-" << setw(4) << setfill('0') << countImgs << ".mip";

    ImageIO::Write(name.str(), dst);
  }

  return 0;
}
