/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __FilterNTo3N_hh__
#define __FilterNTo3N_hh__

#include "FilterBase.hh"


namespace BIAS 
{
  /** @class FilterNTo3N
      @ingroup g_filter
      @brief base class for simple n->3n filter implementations
      
      This is the base class for n->3n filters, i.e. for every input channel 
      three output channels are generated by filters derived from this class.
      
      @author skoglund, koeser, woelk 11/2004 */
  template <class InputStorageType, class OutputStorageType>
  class BIASFilter_EXPORT FilterNTo3N : 
    public FilterBase<InputStorageType, OutputStorageType>
  {
  public:
    FilterNTo3N();
    FilterNTo3N(const FilterNTo3N<InputStorageType, OutputStorageType>& other);
    virtual ~FilterNTo3N();
    
    /** dst.GetChannelCount()==3*src.GetCHannelCount() */
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& dst);

    virtual int FilterInt(const Image<InputStorageType>& src, 
                          Image<OutputStorageType>& dst);

    virtual int FilterFloat(const Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst);
    virtual FilterNTo3N<InputStorageType, OutputStorageType>* Clone() const = 0;

    /** dstX.GetChannelCount()==src.GetCHannelCount() */
    virtual int Filter(const Image<InputStorageType>& src, 
                       Image<OutputStorageType>& dst1, 
                       Image<OutputStorageType>& dst2,
                       Image<OutputStorageType>& dst3);

    virtual int FilterInt(const Image<InputStorageType>& src, 
                          Image<OutputStorageType>& dst1, 
                          Image<OutputStorageType>& dst2,
                          Image<OutputStorageType>& dst3);

    virtual int FilterFloat(const Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst1, 
                            Image<OutputStorageType>& dst2,
                            Image<OutputStorageType>& dst3);
    

  }; // class
} // namespace

#endif // __FilterNTo3N_hh__
