/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Median.hh"

#include "bias_config.h"

#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

template <class InputStorageType, class OutputStorageType> Median<InputStorageType,
    OutputStorageType>::Median() {
  _MedianSize = 2;
  _secondSize = 2;
}

template <class InputStorageType, class OutputStorageType> Median<InputStorageType,
    OutputStorageType>::Median(const Median<InputStorageType, OutputStorageType>& other) :
  _MedianSize(other._MedianSize) {
  _secondSize = _MedianSize;
}

template <class InputStorageType, class OutputStorageType> Median<InputStorageType,
    OutputStorageType>::~Median() {
}

int comp(const void *a, const void *b) {
  if (*(float*)a< *(float*)b)
    return -1;
  else if (*(float*)a> *(float*)b)
    return 1;
  else
    return 0;
}

template <class InputStorageType, class OutputStorageType> int Median<InputStorageType,
    OutputStorageType>::Filter(const Image<InputStorageType>& src, Image<OutputStorageType>& dst) {
  unsigned int halfsizeX = _MedianSize;
  unsigned int halfsizeY = _secondSize;
  float minvalue = -1.0;
  if (src.GetChannelCount()!=1) {
    BIASERR("Median() not implemented for more than one channel");
    return -1;
  }

  cerr << "halfsizeX " << halfsizeX << endl;
  cerr << "halfsizeY " << halfsizeY << endl;

  Image<OutputStorageType> tmp;
  OutputStorageType **dstida = dst.GetImageDataArray();
  const InputStorageType **srcida = src.GetImageDataArray();
  bool identimg = false;
  int sizeX = 2*halfsizeX+1;
  int sizeY = 2*halfsizeY+1;

  if ((void*)src.GetImageData() != (void*)dst.GetImageData()) {
    if (!src.SamePixelAndChannelCount(dst)) {
      if (!dst.IsEmpty())
        dst.Release();
      dst.Init(src.GetWidth(), src.GetHeight(), 1);
      dstida = dst.GetImageDataArray();
    }
  } else {
    identimg = true;
    tmp.Init(src.GetWidth(), src.GetHeight(), 1);
    dstida = tmp.GetImageDataArray();
  }

  float *z, zvalue;
  int startx, starty, endx, endy;
  unsigned int found;

  z = new float[sizeX*sizeY];
 
  for (unsigned int j=0; j<src.GetHeight(); j++) {

    for (unsigned int i=0; i<src.GetWidth(); i++) {
      found = 0;
      startx = int(i)-(sizeX-1)/2;
      endx = int(i)+(sizeX-1)/2;
      starty = int(j)-(sizeY-1)/2;
      endy = int(j)+(sizeY-1)/2;
      if (startx<0)
        startx = 0;
      if (starty<0)
        starty = 0;
      if (endx>int(src.GetWidth()-1))
        endx = src.GetWidth()-1;
      if (endy>int(src.GetHeight()-1))
        endy = src.GetHeight()-1;
      for (int y = starty; y<= endy; y++)
        for (int x = startx; x<= endx; x++) {
          zvalue =(float)srcida[y][x];
          if (zvalue > minvalue) {
            z[found] = zvalue;
            found++;
          }
        }
      if (found >0) {
        qsort(z, found, sizeof(float), comp);
        dstida[j][i] = (OutputStorageType)z[found/2];
      } else
        dstida[j][i] = (OutputStorageType)0.0;
    }
  }

  delete[] z;

  if (identimg)
    dst.StealImage(tmp);

  return 0;

}

template <class InputStorageType, class OutputStorageType> int Median<InputStorageType,
    OutputStorageType>::FilterInt(const Image<InputStorageType>& src, Image<OutputStorageType>& dst) {
  BIASERR("FilterInt makes no sense here, using general Filter interface.");
  return Filter(src, dst);
}

template <class InputStorageType, class OutputStorageType> int Median<InputStorageType,
    OutputStorageType>::FilterFloat(const Image<InputStorageType>& src, Image<OutputStorageType>& dst) {
  BIASERR("FilterFloat makes no sense here, using general Filter interface.");
  return Filter(src, dst);
}

/*
 template <class InputStorageType, class OutputStorageType>
 int Median<InputStorageType, OutputStorageType>::
 FilterIgnoreZero(Image<InputStorageType>& src, 
 Image<OutputStorageType>& dst, 
 unsigned int halfsize)
 {
 #ifdef BIAS_DEBUG
 if (src.GetChannelCount()!=1) {
 BIASERR("Median() not implemented for more than one channel");
 return -1;
 }
 #endif  
 Image<StorageType> tmp;
 StorageType **dstida, **srcida;
 int size = 2*halfsize+1;
 float *z,zvalue;
 int startx,starty,endx,endy;
 unsigned int found ;
 
 if (!src.SamePixelAndChannelCount(dst)){
 if (!dst.IsEmpty())
 dst.Release();
 dst.Init(src.GetWidth(), src.GetHeight(),1);
 }

 dstida = dst.GetImageDataArray();
 srcida = src.GetImageDataArray();
 z = new float[size*size];

 for (unsigned int j=0; j<src.GetHeight(); j++){
 for (unsigned int i=0; i<src.GetWidth(); i++) {
 found = 0;
 startx = int(i)-(size-1)/2;
 endx = int(i)+(size-1)/2;
 starty = int(j)-(size-1)/2;
 endy = int(j)+(size-1)/2;
 if (startx<0) startx = 0;
 if (starty<0) starty = 0;
 if (endx>int(src.GetWidth()-1) ) endx = src.GetWidth()-1;
 if (endy>int(src.GetHeight()-1) ) endy = src.GetHeight()-1;
 for (int y = starty; y<= endy; y++)
 for (int x = startx; x<= endx; x++) {
 zvalue = (float)srcida[y][x];
 if (zvalue !=(StorageType)0.0) {
 z[found] = zvalue;
 found++;
 }
 }
 if (found>1) {
 qsort(z,found, sizeof(float),comp);
 dstida[j][i] = (StorageType)z[found>>1];
 } else {
 dstida[j][i] = (StorageType)0.0;
 }
 }
 }
 
 delete[] z;
 return 0;
 }
 */
template <class InputStorageType, class OutputStorageType> int Median<InputStorageType,
    OutputStorageType>::FilterIgnoreZero5x5(Image<InputStorageType>& src,
                                            Image<OutputStorageType>& dst) {
#ifdef BIAS_DEBUG
  if (src.GetChannelCount()!=1) {
    BIASERR("Median() not implemented for more than one channel");
    return -1;
  }
#endif  
  OutputStorageType **dstida;
  InputStorageType **srcida;
  float z[25], zvalue;
  unsigned found, xm, x2m, xp, x2p, ym, y2m, yp, y2p, w, maxwidth, maxheight, h, x, y;

  if (!src.SamePixelAndChannelCount(dst)) {
    if (!dst.IsEmpty())
      dst.Release();
    dst.Init(src.GetWidth(), src.GetHeight(), 1);
  }

  dstida = dst.GetImageDataArray();
  srcida = src.GetImageDataArray();
  memset((void*)dstida[0], 0, sizeof(OutputStorageType)*dst.GetPixelCount());

  w=src.GetWidth();
  h=src.GetHeight();
  maxheight=h-2;
  maxwidth=w-2;
  for (y=2; y<maxheight; y++) {
    for (x=2; x<maxwidth; x++) {
      if ((zvalue = (float)srcida[y][x])!=0.0) {
        z[0] = zvalue;
        found=1;
        xm=x-1;
        x2m=x-2;
        xp=x+1;
        x2p=x+2;
        ym=y-1;
        y2m=y-2;
        yp=y+1;
        y2p=y+2;
        if ((zvalue = (float)srcida[y2m][x2m])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][xm])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][x])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][xp])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][x2p])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x2m])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][xm])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][xp])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x2p])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][x2m])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][xm])>0.0) {
          z[found] = zvalue;
          found++;
        }

        if ((zvalue = (float)srcida[y][xp])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][x2p])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x2m])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][xm])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][xp])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x2p])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][x2m])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][xm])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][x])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][xp])>0.0) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][x2p])>0.0) {
          z[found] = zvalue;
          found++;
        }

        if (found>4) {
          qsort(z, found, sizeof(float), comp);
          dstida[y][x] = (OutputStorageType)(0.33*z[(found>>1)-1] + 0.34*z[found>>1] + 0.33*z[(found>>1)+1]);

        }
      }
    }
  }

  return 0;
}

template <class InputStorageType, class OutputStorageType> int Median<InputStorageType,
    OutputStorageType>::FilterOnlyZeroIgnoreZero5x5(Image<InputStorageType>& src,
                                                    Image<OutputStorageType>& dst,
                                                    const float& Threshold) {
#ifdef BIAS_DEBUG
  if (src.GetChannelCount()!=1) {
    BIASERR("Median() not implemented for more than one channel");
    return -1;
  }
#endif  
  OutputStorageType **dstida;
  InputStorageType **srcida;
  float z[25], zvalue;
  unsigned found, xm, x2m, xp, x2p, ym, y2m, yp, y2p, w, maxwidth, maxheight, h, x, y;

  if (!src.SamePixelAndChannelCount(dst)) {
    if (!dst.IsEmpty())
      dst.Release();
    dst.Init(src.GetWidth(), src.GetHeight(), 1);
  }

  dstida = dst.GetImageDataArray();
  srcida = src.GetImageDataArray();
  memset((void*)dstida[0], 0, sizeof(OutputStorageType)*dst.GetPixelCount());

  w=src.GetWidth();
  h=src.GetHeight();
  maxheight=h-2;
  maxwidth=w-2;
  for (y=2; y<maxheight; y++) {
    for (x=2; x<maxwidth; x++) {
      if ((zvalue = (float)srcida[y][x])<=Threshold) {
        found=0;
        xm=x-1;
        x2m=x-2;
        xp=x+1;
        x2p=x+2;
        ym=y-1;
        y2m=y-2;
        yp=y+1;
        y2p=y+2;
        if ((zvalue = (float)srcida[y2m][x2m])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][x])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2m][x2p])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x2m])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x2p])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][x2m])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }

        if ((zvalue = (float)srcida[y][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][x2p])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x2m])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x2p])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][x2m])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][x])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y2p][x2p])>Threshold) {
          z[found] = zvalue;
          found++;
        }

        if (found>4) {

          qsort(z, found, sizeof(float), comp);
          dstida[y][x] = (OutputStorageType)(0.33*z[(found>>1)-1] + 0.34*z[found>>1] + 0.33*z[(found>>1)+1]);
        } else {
          dstida[y][x] = (OutputStorageType)Threshold;
        }
      } else {
        dstida[y][x]= (OutputStorageType) zvalue;
      }
    }
  }

  return 0;
}

template <class InputStorageType, class OutputStorageType> int Median<InputStorageType,
    OutputStorageType>::FilterOnlyZeroIgnoreZero3x3(Image<InputStorageType>& src,
                                                    Image<OutputStorageType>& dst,
                                                    const float& Threshold) {
#ifdef BIAS_DEBUG
  if (src.GetChannelCount()!=1) {
    BIASERR("Median() not implemented for more than one channel");
    return -1;
  }
#endif  
  OutputStorageType **dstida;
  InputStorageType **srcida;
  float z[9], zvalue;
  unsigned found, xm, xp, ym, yp, w, maxwidth, maxheight, h, x, y;

  if (!src.SamePixelAndChannelCount(dst)) {
    if (!dst.IsEmpty())
      dst.Release();
    dst.Init(src.GetWidth(), src.GetHeight(), 1);
  }

  dstida = dst.GetImageDataArray();
  srcida = src.GetImageDataArray();
  memset((void*)dstida[0], 0, sizeof(OutputStorageType)*dst.GetPixelCount());

  w=src.GetWidth();
  h=src.GetHeight();
  maxheight=h-1;
  maxwidth=w-1;
  for (y=1; y<maxheight; y++) {
    for (x=1; x<maxwidth; x++) {
      if ((zvalue = (float)srcida[y][x])<=Threshold) {
        found = 0;
        xm = x-1;
        xp = x+1;
        ym = y-1;
        yp = y+1;
        if ((zvalue = (float)srcida[ym][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][x])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[ym][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[y][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }

        if ((zvalue = (float)srcida[y][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][xm])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][x])>Threshold) {
          z[found] = zvalue;
          found++;
        }
        if ((zvalue = (float)srcida[yp][xp])>Threshold) {
          z[found] = zvalue;
          found++;
        }

        if (found>=4) {
          qsort(z, found, sizeof(float), comp);
          // average median elements
          dstida[y][x] = (OutputStorageType)(0.33*z[(found>>1)-1] + 0.34*z[found>>1] + 0.33*z[(found>>1)+1]);
        } else {
          //dstida[y][x] = (OutputStorageType)Threshold;
        }
      } else {
        dstida[y][x]= (OutputStorageType) zvalue;
      }
    }
  }
  return 0;
}

#ifdef allmediancode
template <class InputStorageType, class OutputStorageType>
int Median<InputStorageType, OutputStorageType>::
NonMaxSupression3x3(Image<InputStorageType>& src,
    Image<OutputStorageType>& dst)
{
  if (!src.SamePixelAndChannelCount(dst)) {
    if (!dst.IsEmpty())
    dst.Release();
    dst.Init(src.GetWidth(), src.GetHeight(), src.GetChannelCount());
  }
  if (src.GetChannelCount()!=1) {
    BIASERR("only for 1 channel images");
    return -1;
  }
  StorageType *ev=src.GetImageData(), *d=dst.GetImageData();
  unsigned border=1, height=src.GetHeight();
  int width=src.GetWidth();
  unsigned offs=border*width+border;
  StorageType *pev, *end, act;
  int wp=width+1, wm=width-1;

  // nonmax suppression in 8 neighbourhood
  end=ev+width*(height-border)-border;
  pev=ev+offs;
  d+=offs;
  memset((void *)d, 0, sizeof(StorageType)*src.GetPixelCount());
  while (pev<end) {
    act=*pev;
    if (pev[-wp]<act && pev[-width]<act && pev[-wm]<act &&
        pev[-1]<act && pev[1]<act &&
        pev[wm]<act && pev[width]<act && pev[wp]<act) {
      *d=*pev;
    }
    pev++; d++;
  }
  return 0;
}

/** downloaded from 
 http://home.od.ua/~relayer/algo/dsp/median/median.html
 24.11.2004 woelk */
/*
 *  This Quickselect routine is based on the algorithm described in
 *  "Numerical recipies in C", Second Edition,
 *  Cambridge University Press, 1992, Section 8.5, ISBN 0-521-43108-5
 */

#define ELEM_SWAP(a,b) { t=(a); (a)=(b); (b)=t; }

template <class InputStorageType, class OutputStorageType>
int Median<InputStorageType, OutputStorageType>::
Median::Filter(InputStorageType arr[], int n)
{
  int low, high;
  int median;
  int middle, ll, hh;
  register StorageType t;

  low = 0; high = n-1; median = (low + high) / 2;
  for (;;) {
    if (high <= low) /* One element only */
    return arr[median];

    if (high == low + 1) { /* Two elements only */
      if (arr[low]> arr[high])
      ELEM_SWAP(arr[low], arr[high]);
      return arr[median];
    }

    /* Find median of low, middle and high items; swap into position low */
    middle = (low + high) / 2;
    if (arr[middle]> arr[high]) ELEM_SWAP(arr[middle], arr[high]);
    if (arr[low]> arr[high]) ELEM_SWAP(arr[low], arr[high]);
    if (arr[middle]> arr[low]) ELEM_SWAP(arr[middle], arr[low]);

    /* Swap low item (now in position middle) into position (low+1) */
    ELEM_SWAP(arr[middle], arr[low+1]);

    /* Nibble from each end towards middle, swapping items when stuck */
    ll = low + 1;
    hh = high;
    for (;;) {
      do ll++; while (arr[low]> arr[ll]);
      do hh--; while (arr[hh]> arr[low]);

      if (hh < ll)
      break;

      ELEM_SWAP(arr[ll], arr[hh]);
    }

    /* Swap middle item (in position low) back into correct position */
    ELEM_SWAP(arr[low], arr[hh]);

    /* Re-set active partition */
    if (hh <= median)
    low = ll;
    if (hh >= median)
    high = hh - 1;
  }
}

#undef ELEM_SWAP

#endif

template <class InputStorageType, class OutputStorageType> void Median<InputStorageType,
    OutputStorageType>::GetBordersValid_(int &border_x, int &border_y) const {
  if (_secondSize > _MedianSize) {
    border_x = border_y = _secondSize;
  } else {
    border_x = border_y = _MedianSize;
  }
}

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////
#define FILTER_INSTANTIATION_CLASS Median
#include "Filterinst.hh"
