/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __Morphology_hh__
#define __Morphology_hh__

#include "./FilterBase.hh"
#include <Base/Image/Image.hh>

namespace BIAS {
  /** @class Morphology
      @ingroup g_filter
      @brief base class for Erosion, Dilation, and single Deletion
      
      @author grest, Oct. 2004
  */

  template <class InputStorageType, class OutputStorageType> 
  class BIASFilter_EXPORT Morphology : public FilterBase<InputStorageType, OutputStorageType>
  {
  public:
    Morphology();
    Morphology(const Morphology<InputStorageType, OutputStorageType>& other);
    virtual ~Morphology();
  
    /// fills the XBorderSize respectivly YBorderSize nearest 
    /// Pixels to picture border with Value
    /// XBorderSize defines the border parallel to the x axis 
    int FillBorderConst(Image<OutputStorageType>& Image,
                        unsigned short int XBorderSize,
                        unsigned short int YBorderSize, 
                        OutputStorageType Value);
  
    inline void SetKernelSize(int size) { kernelSize_=size; }

    /** Very fast deletion of pixels with no neighbours.
        All values, which are not zero, are treated as foreground. 
        The border of one pixel is always deleted (set to zero).
        default is 8-neighborhood.
        src and dest must NOT be the same!
        @author Daniel Grest, Sept. 2002
        @status tested */
    //int DeleteSingles(Image<unsigned char>& src, 
    //                  Image<unsigned char>& dest,
    //                  bool Neighbor4=false);
  protected:
  
    /// pointer into param object, determines influence region size 
    int kernelSize_;
  
  }; // class
  

} // namespace


#endif // __Morphology_hh__
