# -Generates the bias library config shell script: "bias-config"
# from bias-config.in (in BIAS_SOURCE_DIR) 
# 
# For compatibility with old-style handwritten Makefiles.
# CMake has many benefits compared to handwritten Makefiles.
# Please think about switching to cmake 
# 
# NOTES
# Should NOT be used by projects using CMake build system!
# See PackageConfig/BIASConfig for CMAKE user projects
#
# AUTHOR
# Jan-Friso Evers-Senne and Jan Woetzel


# debug output, should be disabled for distribution 
MACRO(DBG_MSG _MSG)
  MESSAGE(STATUS "${CMAKE_CURRENT_LIST_FILE}(${CMAKE_CURRENT_LIST_LINE}): ${_MSG}")
ENDMACRO(DBG_MSG)


IF (UNIX)
  SET(FILE_BIAS_CONFIG_SCRIPT_IN "${BIAS_SOURCE_DIR}/bias-config.in")
  SET(FILE_BIAS_CONFIG_SCRIPT    "${BIAS_BINARY_DIR}/bias-config")
  
  ## prefix - base dir of install install or not install?
  SET(BIAS_PREFIX "${CMAKE_INSTALL_PREFIX}")
  
  # init
  SET(BIAS_CONFIG_SCRIPT_CFLAGS  "")
  SET(BIAS_CONFIG_SCRIPT_LDFLAGS "")
  
  # extra compiler flags rpaths searched at runtime for .so libs
  SET(BIAS_RPATH "${BIAS_PREFIX}/lib") # for make install
  IF(LIBRARY_OUTPUT_PATH)
    # using subdir lib instead of make install
    SET(BIAS_RPATH "${BIAS_RPATH}:${LIBRARY_OUTPUT_PATH}")
    SET(BIAS_RPATH "${BIAS_RPATH}:${LIBRARY_OUTPUT_PATH}/${CMAKE_CFG_INTDIR}")    
  ENDIF(LIBRARY_OUTPUT_PATH)
  
  FOREACH( EXTLIB 
      BLAS
      LAPACK
      DC1394
      DEVIL      
      DV
      FFTW3
      GLEW
      GLUT
      GSL
      GTK
      GTK2
      IMLIB
      IMAGEMAGICKLIB
      OPENGL # is full path correct here (jw) ?
      OPENCV
      OPENEXR
      OPENMP
      # PTHREADS  
      STEREOI
      V4L
      XML2 LIBXML2
      1394CAMERACMU
      )
    IF   ( USE_${EXTLIB} AND ${EXTLIB}_FOUND AND ${EXTLIB}_LIBRARIES)
      #MESSAGE("${EXTLIB}: ${EXTLIB}_LIBRARIES = ${${EXTLIB}_LIBRARIES}")
      SET(BIAS_EXTERN_LIBRARIES  "${BIAS_EXTERN_LIBRARIES} ${${EXTLIB}_LIBRARIES}")
    ENDIF( USE_${EXTLIB} AND ${EXTLIB}_FOUND AND ${EXTLIB}_LIBRARIES)
    
  ENDFOREACH( EXTLIB )

#
# things that do not map a lib 1:1: 
#
IF   (wxWidgets_FOUND AND USE_WXWIDGETS)
  SET(BIAS_EXTERN_LIBRARIES  "${BIAS_EXTERN_LIBRARIES} ${wxWidgets_LIBRARIES}")
  #MESSAGE("wxWidgets: wxWidgets_LIBRARIES = ${wxWidgets_LIBRARIES}")
ENDIF(wxWidgets_FOUND AND USE_WXWIDGETS)


  
  ## generate old-style bias-config shell script 
  ## to be used just by NON-CMAKIFIED user projects. 
  ## See PackageConfig/BIASConfig for CMAKE user projects
  
  # BIAS own installed include dirs first
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "")
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${BIAS_PREFIX}/include")
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${BIAS_PREFIX}/include/BIAS")
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${CMAKE_INSTALL_PREFIX}/include")
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${CMAKE_INSTALL_PREFIX}/include/BIAS")
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${BIAS_BINARY_DIR}")
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${BIAS_SOURCE_DIR}")  
  
  # append third party include dirs
  FOREACH(DIR ${BIAS_EXTERN_INCLUDE_DIR})
    SET(BIAS_CONFIG_SCRIPT_CFLAGS "${BIAS_CONFIG_SCRIPT_CFLAGS} -I${DIR} -I${BIAS_SOURCE_DIR}")
  ENDFOREACH(DIR)
  
  # prefix with BIAS own installed include paths
  SET(BIAS_CONFIG_SCRIPT_CFLAGS "-I${BIAS_PREFIX}/include -I${BIAS_PREFIX}/include/BIAS ${BIAS_CONFIG_SCRIPT_CFLAGS}")
  
  # third party link directories into rpath 
  FOREACH(DIR ${BIAS_EXTERN_LINK_DIR})
    SET(BIAS_RPATH "${BIAS_RPATH}:${DIR}")
    #MESSAGE("DBG BIAS_RPATH=${BIAS_RPATH} DIR=${DIR}")
  ENDFOREACH(DIR)
  
  # add rpath to linker flags
  SET(BIAS_CONFIG_SCRIPT_LDFLAGS "${BIAS_CONFIG_SCRIPT_LDFLAGS} -Wl,-rpath,${BIAS_RPATH}")

  ##
  ## add "-l" to "real" library names  
  ## skip *.so  and all back-quoted shell scripts
  ##
  #MESSAGE("DBG  BIAS_EXTERN_LIBRARIES=${BIAS_EXTERN_LIBRARIES}")
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBRARIES "")
  FOREACH(LIB ${BIAS_EXTERN_LIBRARIES})
    # do not prefix back-quoted expressions, e.g/ `/bin/foo-config`
    IF (NOT ${LIB} MATCHES "[`].+[`]")
      IF (NOT ${LIB} MATCHES ".+[.][s][o]")
	IF (NOT ${LIB} MATCHES "[-][l].+")
	  IF (NOT ${LIB} MATCHES "[-][L].+")
	    IF (NOT ${LIB} MATCHES "[-][W].+")
	    
	      # OK, this a 'real' library to be prefixed: 
	      # add prefix and convert semicolon-separated vector to space-separtaed string 
	      SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -l${LIB} ")
	    ELSE (NOT ${LIB} MATCHES "[-][W].+")
	      #MESSAGE("DBG skipping  -W prefixed LIB=${LIB}")
	      SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")
	    ENDIF  (NOT ${LIB} MATCHES "[-][W].+")
	  ELSE (NOT ${LIB} MATCHES "[-][L].+")
	    #MESSAGE("DBG skipping  already -L prefixed LIB=${LIB}")
	    SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")
	  ENDIF (NOT ${LIB} MATCHES "[-][L].+")
	ELSE (NOT ${LIB} MATCHES "[-][l].+")
	  #MESSAGE("DBG skipping  already -l prefixed LIB=${LIB}")
	  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")	
	ENDIF (NOT ${LIB} MATCHES "[-][l].+")
      ELSE (NOT ${LIB} MATCHES ".+[.][s][o]")
	#MESSAGE("DBG skipping  prefixing of .so LIB=${LIB}")
	SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")	
      ENDIF (NOT ${LIB} MATCHES ".+[.][s][o]")      
    ELSE (NOT ${LIB} MATCHES "[`].+[`]")
      #MESSAGE("DBG skipping prefixing of back-quoted LIB=${LIB}")
      SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")
    ENDIF (NOT ${LIB} MATCHES "[`].+[`]")
  ENDFOREACH(LIB)
  #MESSAGE("DBG  BIAS_CONFIG_SCRIPT_EXTERN_LIBS=${BIAS_CONFIG_SCRIPT_EXTERN_LIBS}")
  
  # prepend with BIAS own lib dir
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  " -L${BIAS_PREFIX}/lib ${BIAS_CONFIG_SCRIPT_EXTERN_LIBS}")
  # append hard coded system dirs for SM
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -L/usr/X11R6/lib -L/usr/lib -L/usr/local/lib ")
  IF    (LIBRARY_OUTPUT_PATH)
    SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -L${LIBRARY_OUTPUT_PATH}")
    
  ELSE  (LIBRARY_OUTPUT_PATH)
    SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -L${BIAS_BINARY_DIR}")
  ENDIF (LIBRARY_OUTPUT_PATH)

  CONFIGURE_FILE(
    ${FILE_BIAS_CONFIG_SCRIPT_IN} 
    ${FILE_BIAS_CONFIG_SCRIPT}
    @ONLY IMMEDIATE )  
  IF(UNIX)
    # set executable permissions
    EXEC_PROGRAM(chmod ARGS 755  ${FILE_BIAS_CONFIG_SCRIPT} )
  ENDIF(UNIX)


ELSE(UNIX)
  IF(WIN32)
    # OK, do nothing for now. (JW)
  ELSE(WIN32)
    MESSAGE("GenerateConfigScript works only on Unix/Linux/WIN32, sorry.")
  ENDIF(WIN32)
ENDIF(UNIX)

