#include "CovTransformPose.hh"

#include <Base/Math/Vector3.hh>
#include <Base/Math/Matrix.hh>
#include <Geometry/RMatrix.hh>
#include <Base/Geometry/Quaternion.hh>

using namespace std;
using namespace BIAS;

CovTransformPose::CovTransformPose()
{
  tType_ = NONE;
}

int CovTransformPose::
PoseQuatToEulerZYX(const Vector3<double> &posQ, const Quaternion<double> &rotQ, 
                   const Matrix<double> &covQ,
                   Vector3<double> &posE, Vector3<double> &rotE,
                   Matrix<double> &covE)
{
  int res = 0;

  //tell Transform_ what we want to do
  tType_ = Quat2EulerZYX;

  //generate source vector
  Vector<double> src(7), dest;
  for (unsigned int i=0; i<3; i++) src[i] = posQ[i];
  for (unsigned int i=0; i<4; i++) src[i+3] = rotQ[i];

  //call uncented transform (uses Transform_ to transform src and samplepoints)
  if (Transform(src, covQ, dest, covE) != 0) {
    BIASERR("Error in UncentedTransform!");
    res = -1;
  }

  //get resulting pose from dest-vector
  for (int i=0; i<3; i++) posE[i] = dest[i];
  for (int i=0; i<3; i++) rotE[i] = dest[i+3];

  //transformation is done
  tType_ = NONE;

  return res;
}

int CovTransformPose::
PoseEulerZYXToQuat(const Vector3<double> &posE, const Vector3<double> &rotE,
                   const Matrix<double> &covE,
                   Vector3<double> &posQ, Quaternion<double> &rotQ, 
                   Matrix<double> &covQ)
{
  int res = 0;

  //tell Transform_ what we want to do
  tType_ = EulerZYX2Quat;

  //generate source vector
  Vector<double> src(6), dest;
  for (int i=0; i<3; i++) src[i] = posE[i];
  for (int i=0; i<3; i++) src[i+3] = rotE[i];

  //call uncented transform (uses Transform_ to transform src and samplepoints)
  if (Transform(src, covE, dest, covQ) != 0) {
    BIASERR("Error in UncentedTransform!");
    res = -1;
  }

  //get resulting pose from dest-vector
  for (int i=0; i<3; i++) posQ[i] = dest[i];
  for (int i=0; i<4; i++) rotQ[i] = dest[i+3];

  //transformation is done
  tType_ = NONE;

  return res;
}


int CovTransformPose::
Transform_(const Vector<double>& src, Vector<double>& dst) const
{
  int res = 0;

  //switch what's todo
  switch (tType_) {

    //convert quaternion to euler-angles
  case Quat2EulerZYX:
    {
      //Get rotation-matrix from quaternion
      RMatrix R;
      R.SetFromQuaternion(Quaternion<double>(src[3], src[4], src[5], src[6]));
      //get ratation-angles from RMatrix
      double xPhi, yPhi, zPhi;
      if (R.GetRotationAnglesZYX(xPhi, yPhi, zPhi) != 0) {
        //decomposition failed, return error
        BIASERR("Error in matrix decomposition!");
        res = -1;
      } else {
        //decomposition successfull, compose destination-vector...
        dst.newsize(6);
        //...copy position...
        for (int i=0; i<3; i++) dst[i] = src[i];
        //...and set euler-angles
        dst[3] = xPhi;
        dst[4] = yPhi;
        dst[5] = zPhi;
        res = 0;
      }
    }
    break;

    //convert euler-angles to quaternion
  case EulerZYX2Quat:
    {
      //Generate quaternion from given euler-angles...
      Quaternion<double> q;
      //should be SetZYX, but Quaternion.SetZYX does not
      //correspond to RMatrix.SetZYX!!!
      q.SetXYZ(src[3], src[4], src[5]);
      //compose destination-vector...
      dst.newsize(7);
      //...copy position...
      for (int i=0; i<3; i++) dst[i] = src[i];
      //...set quaternion-values
      for (int i=0; i<4; i++) dst[i+3] = q[i];
      res = 0;
    }
    break;

  default:
    BIASERR("We should't end up here!!!");
    BIASABORT;
  }

  return res;
}
