#include <iostream>
#include <sstream>

#include <Image/Camera.hh>
#include <Base/Image/ImageIO.hh>
#include <Geometry/Projection.hh>
#include <Geometry/ProjectionParametersPerspective.hh>
#include <Base/Geometry/KMatrix.hh>

using namespace BIAS;
using namespace std;



int main(int argc, char** argv) {
  
  // argument specified?
  if (argc != 2) {
    BIASERR("don't know which image to use!/n/n  usage: ExampleExtractKFromMipImage <path_to_image>");
    BIASABORT;
  }
  
  Camera<unsigned char> image;
  
  // load image
  if (ImageIO::Load(argv[1], image) != 0) {
    BIASERR("specified image could not be loaded!");
    BIASABORT;
  }
  cout << "image loaded successfully!" << endl;
  
  // check for valid projection
  image.ParseMetaData();
  if (!image.IsProjValid()) {
    BIASERR("image contains no valid projection!");
    BIASABORT;
  }
  
  // extract projection and K matrix
  cout << "getting K... ";
  Projection proj = image.GetProj();
  ProjectionParametersPerspective* ppp;
  ppp = (ProjectionParametersPerspective*)proj.GetParameters();
  KMatrix K = ppp->GetK();
  
  cout << "done!" << endl;
  
  // save K matrix
  ostringstream filenameK;
  filenameK << argv[1] << ".mat";
  cout << "saving K as " << filenameK.str() << endl;
  K.Save(filenameK.str());
  
  return 0;
}
