/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Geometry/ProjectionParametersPerspectiveDepth.hh" 
#include "MathAlgo/PolynomialSolve.hh"

using namespace BIAS;
using namespace std;

#ifdef BIAS_HAVE_XML2
 
int ProjectionParametersPerspectiveDepth::
XMLGetClassName(std::string& TopLevelTag, double& Version) const {
  TopLevelTag = "ProjectionParametersPerspectiveDepth";
  Version = 0.1;
  return 0;
}

int ProjectionParametersPerspectiveDepth::
XMLOut(const xmlNodePtr Node, XMLIO& XMLObject) const {
  ProjectionParametersPerspective::XMLOut (Node, XMLObject);
  xmlNodePtr childNode;
  childNode = XMLObject.addChildNode(Node,"DepthCalibration");
  for (unsigned int i=0; i<DepthCalibrationParameters_.size(); i++) {
    std::ostringstream o;
    o << "d" << i;
    XMLObject.addAttribute(childNode, o.str(), DepthCalibrationParameters_[i]);
  }
  return 0; 
 }

int ProjectionParametersPerspectiveDepth::
XMLIn(const xmlNodePtr Node, XMLIO& XMLObject) {
  ProjectionParametersPerspective::XMLIn(Node, XMLObject);
  xmlNodePtr childNode;
  DepthCalibrationParameters_.resize(DEPTH_CALIB_PARAMS, 0.0);
  if ((childNode = XMLObject.getChild(Node, "DepthCalibration"))!=NULL) {
    for (unsigned int i=0; i<DepthCalibrationParameters_.size(); i++) {
      std::ostringstream o;
      o << "d" << i;
      DepthCalibrationParameters_[i] = XMLObject.getAttributeValueDouble(childNode, o.str());
    }
  }
  return 0;
}


void ProjectionParametersPerspectiveDepth::
DistortDepth (HomgPoint2D const &pos, float &d) const {  
  BIASASSERT(DepthCalibrationParameters_.size()==6);
  d = float(DepthCalibrationParameters_[0])
      + (1.0f+float(DepthCalibrationParameters_[1]))*d 
      + float(DepthCalibrationParameters_[2]*pos[0]/pos[2]) 
      + float(DepthCalibrationParameters_[3]*pos[1]/pos[2])
      + float(DepthCalibrationParameters_[4])*d*d
      + float(DepthCalibrationParameters_[5])*d*d*d;
}


void ProjectionParametersPerspectiveDepth::
UnDistortDepth (HomgPoint2D const &pos, float &d) const {
  BIASASSERT(DepthCalibrationParameters_.size()==6);
  std::vector<double> poly(4);
  poly[0] = DepthCalibrationParameters_[0] 
    + DepthCalibrationParameters_[2]*pos[0]/pos[2] 
    + DepthCalibrationParameters_[3]*pos[1]/pos[2]
    - d;
  poly[1] = (1+DepthCalibrationParameters_[1]);
  poly[2] = DepthCalibrationParameters_[4];
  poly[3] = DepthCalibrationParameters_[5];  
  std::vector<double> sol;
  PolynomialSolve solver;
  solver.Solve (poly, sol);
  BIASASSERT(sol.size()>0);
  double dd=sol[0];
  for (unsigned int i=1; i<sol.size(); i++) {
    if (fabs(sol[i]-d)<fabs(dd-d))
      dd = sol[i];
  }
  d = float(dd);
}


#endif // BIAS_HAVE_XML2
