/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Geometry/ProjectionParametersProjective.hh" 


using namespace std;
using namespace BIAS;

void ProjectionParametersProjective::
Print(std::ostream& os) const
{}

HomgPoint3D ProjectionParametersProjective::
UnProjectToImagePlane(const HomgPoint2D& pos, const double& depth,
                      bool IgnoreDistortion) const 
{
  Vector3<double> p = UnProjectLocal(pos, IgnoreDistortion);
  if(Equal(p[2], 0.0))
    return HomgPoint3D(0,0,0,0);
  p*= depth / p[2];
  if(Equal(p.NormL2(), 0.0))
    return HomgPoint3D(0,0,0,0);
  HomgPoint3D local(p);
  return Pose_.LocalToGlobal(local);
}

void ProjectionParametersProjective::
Distort(BIAS::HomgPoint2D& point2d) const
{
  BIASERR("unfinished");
  BIASABORT;
}


void ProjectionParametersProjective::
Undistort(BIAS::HomgPoint2D& point2d) const
{
  BIASERR("unfinished");
  BIASABORT;
}

Matrix3x3<double> ProjectionParametersProjective::
UnProjectCovLocal(const HomgPoint2D& /*pos*/, const Matrix3x3<double>& /*cov*/,
               bool /* IgnoreDistortion */, bool /* Normalize */) const
{
  BIASERR("covariance propagation for projective space not implemented");
  return Matrix3x3<double>(MatrixZero);
}

#ifdef BIAS_HAVE_XML2

int ProjectionParametersProjective::XMLGetClassName(std::string& TopLevelTag,
                                                    double& Version) const {
  TopLevelTag = "ProjectiveP";
  Version = 1.0;
  return 0;
}
 
int ProjectionParametersProjective::XMLOut(const xmlNodePtr Node, 
                                           XMLIO& XMLObject) const {
  Matrix3x4<double> MatrixCache = Pose_.GetMatrix3x4();
  xmlNodePtr childNode;
  childNode = XMLObject.addChildNode(Node,"MatrixRows");
  stringstream datastream;
  for (unsigned int i=0; i<12; i++) {
    datastream << (double) MatrixCache.GetData()[i] <<" ";
  }
  XMLObject.addContent(childNode, datastream.str());
  return 0; 
}

int ProjectionParametersProjective::XMLIn(const xmlNodePtr Node, 
                                          XMLIO& XMLObject) {
  Matrix3x4<double> MatrixCache;
  xmlNodePtr childNode;
  if ((childNode = XMLObject.getChild(Node, "MatrixRows"))==NULL) {
    BIASERR("Error in xml structure.");
    return -1;
  } 
  
  // get data fields 
  stringstream ssData(XMLObject.getNodeContentString(childNode));
  unsigned int CurrentData=0;
  double buffer;
  while (ssData.good() && CurrentData<12) {
    ssData >> buffer;
    MatrixCache.GetData()[CurrentData++] = buffer;
  }
  Pose_.SetFromMatrix3x4(MatrixCache);
  if (CurrentData<12) {
    // reached end of stream before expected end of data
    BIASERR("Error parsing XML code ...");
    return -1;
  }

  return 0;   
}

#endif


