/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __GuiEnhanced_hh__
#define __GuiEnhanced_hh__
#include "bias_config.h" 

#ifndef BIAS_HAVE_GTK 	 
#  error __FILE__: __LINE__: BIAS_HAVE_GTK not defined. Please recompile BIAS with USE_GTK enabled if you want to use GuiGTK.
#endif // BIAS_HAVE_GTK
#ifndef BIAS_HAVE_PTHREADS
#  error You need pthreads. please recompile BIAS with USE_PTHREADS.
#endif // BIAS_HAVE_PTHREADS


#include <Base/Image/Image.hh>
#include "GuiGTK.hh"

#include "bias_config.h"

namespace BIAS{
  /** @class GuiEnhanced
      @ingroup g_gui
      @brief extended GuiGTK with zoom and marking options
    
      
  */
  class BIASGui_EXPORT GuiEnhanced : public GuiGTK
  {
  public:
    GuiEnhanced();
    
    virtual ~GuiEnhanced();

    void GetZoomImage(unsigned int x, unsigned int y, unsigned int factor, 
                      unsigned int size, Image<unsigned char>& zoomimage);

    void GetZoomImage(unsigned int x, unsigned int y, unsigned int factor, 
                      unsigned int size, Image<unsigned char>& zoomimage,
                      int& xoffset, int& yoffset);

    void GetZoomImage(unsigned int x, unsigned int y, unsigned int factor, 
                      unsigned int size, ImageBase& zoomimage);

    void GetZoomImage(unsigned int x, unsigned int y, unsigned int factor, 
                      unsigned int size, ImageBase& zoomimage,
                      int& xoffset, int& yoffset);

    int SelectPoint(int point[2]);

    /** Select a rectangle using the mouse
        First LEFT button selects upper left corner
        Second LEFT button selects lower right corner
        Pressing the RIGHT cancels
        @param upperleft  Here the resulting upper left corner (x and y) 
           is stored
        @param lowerright Here the resulting lower right corner (x and y) 
           is stored
        @return Returns != 0 in case of error
    **/
    int SelectRectangle(int upperleft[2],int lowerright[2]);


    /** Select a quadrate using the mouse
        First LEFT button selects upper left corner
        Second LEFT button selects lower right corner
        (While moving the mouse the selecting box is adjusted to form a 
        quadrate)
        Pressing the RIGHT button cancels

        @param upperleft  Here the resulting upper left corner (x and y) 
        is stored
        @param lowerright Here the resulting lower right corner (x and y) 
        is stored
        @return Returns != 0 in case of error		
        @author Ingo Thomsen**/
    int SelectQuadrate(int upperleft[2],int lowerright[2]);

     /** select a line in actual image
        left mouse button to select start point
        left mouse button again to select end point
        any other button at any time cancels selection
        returns 0 if selection succesfull, negative value else
        @author fw */
    int SelectLine(int start[2], int end[2]);

    /** select a line in actual image
        left mouse button to get zoom window for start point
        left mouse button in zoom window to select starting point
        same procedure for end point
        any other button at any time cancels selection
        returns 0 if selection succesfull, negative value else
        @author fw */
    int SelectLineZoom(int start[2],int end[2]);

    /** draws a rectangle between tl and br using color
        the image buffer is not saved, it cannot be undone 
        tl must be the top  left corner, br must be the bottom right corner */
    void DrawRectangle(unsigned int tl[2], unsigned int br[2]);

    /** draws a line between start and end, no boundary checks
        the image buffer is not saved, it cannot be undone 
        @author fw */
    void DrawLine(int start[2], int end[2]);
  
  };

} // namespace
#endif // __GuiEnhanced_hh__
