#include "ImageCanvasCheckerboardExtract.hh"
#include <FeatureDetector/CheckerboardDetector.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageDraw.hh>
#include <Base/Image/ColourRGB.hh>

using namespace BIAS;
using namespace std;


ICECCallbackInterface::ICECCallbackInterface()
{}

ICECCallbackInterface::~ICECCallbackInterface()
{}

void ICECCallbackInterface::CheckerboardSelected(std::vector<BIAS::HomgPoint2D> &corners,
                                                 bool bOnlyFourPointsGiven)
{
  BIASERR("error: function of derived class should be called");
}

/////////////////////////////////////////////////////////////////

ImageCanvasCheckerboardExtract::
ImageCanvasCheckerboardExtract(wxWindow* parent, 
                               wxStatusBar* sbar,
                               int sbarid, wxWindowID id, 
                               const wxPoint& pos, 
                               const wxSize& size, 
                               long style, 
                               const wxString& name)
  : ImageCanvas(parent, sbar, sbarid, id, pos, size, style, name)
{
  NumLeftClicks_=0;
  SelectCheckerboard_=false;
  CallbackObj_=NULL;
  first_=0;
  second_=0;
  third_=0;
  forth_=0;
  bEnforceAllCorners_=false;
}

ImageCanvasCheckerboardExtract::
~ImageCanvasCheckerboardExtract()
{}



void ImageCanvasCheckerboardExtract::
SetPatternParameters (unsigned int xCorners, unsigned int yCorners) {
  xCorners_ = xCorners;
  yCorners_ = yCorners;
}

void ImageCanvasCheckerboardExtract::
SelectCheckerboardCorners(ICECCallbackInterface& obj)
{
  bEnforceAllCorners_=false;
  BIAS_MUTEX_LOCK(origim_mutex_);
  BIASDOUT(D_IC_IM,"origim_ in SelectRectangle adr: "<<&origim_);
  BIAS_MUTEX_UNLOCK(origim_mutex_);
  
  CallbackObj_=&obj;

  //Here we try to automatically extract the checkerboard
  t_CoordVecf coords;
  int res = CheckerboardDetector::Compute(rgbim_,xCorners_,yCorners_,coords);
  if(res!= 0){
    SelectCheckerboard_ = true;
    NumLeftClicks_=0; 
    return;
  }
  else{
    first_[0] = coords[0].first;
    first_[1] = coords[0].second;
    second_[0]= coords[xCorners_-1].first;   
    second_[1]= coords[xCorners_-1].second;

    third_[0] = coords[(xCorners_*yCorners_-1)].first;
    third_[1] = coords[(xCorners_*yCorners_-1)].second;
    
    forth_[0] = coords[(yCorners_-1)*(xCorners_)].first;
    forth_[1] = coords[(yCorners_-1)*(xCorners_)].second;

    checkerboardCorners_.clear();
    BIAS::HomgPoint2D point;
    for(unsigned int i=0;i< coords.size();i++){
      point[0] = coords[i].first;
      point[1] = coords[i].second;
      point[2] = 1;
      checkerboardCorners_.push_back(point);
    } 
   
    DrawCheckerboardCorners_(first_,second_,third_,forth_); 
    CallbackObj_->CheckerboardSelected(checkerboardCorners_, false);      
#ifdef WIN32
    this->Refresh();
#else
    wxPaintEvent pe;
    wxPostEvent(this->GetEventHandler(),pe);
#endif
    NumLeftClicks_=0; 
  }
 
}

void ImageCanvasCheckerboardExtract::
SelectAllCheckerboardCorners(ICECCallbackInterface& obj)
{
  bEnforceAllCorners_=true;
  BIAS_MUTEX_LOCK(origim_mutex_);
  BIASDOUT(D_IC_IM,"origim_ in SelectRectangle adr: "<<&origim_);
  BIAS_MUTEX_UNLOCK(origim_mutex_);
  
  CallbackObj_=&obj;

  //Here we try to automatically extract the checkerboard
  t_CoordVecf coords;
  int res = CheckerboardDetector::Compute(rgbim_,xCorners_,yCorners_,coords);
  if(res!= 0){
    SelectCheckerboard_ = true;
    NumLeftClicks_=0; 
  }
  else{
    
    // sort all corners in  checkerboardCorners_
    checkerboardCorners_.clear();
    BIAS::HomgPoint2D point;
    for(unsigned int i=0;i< coords.size();i++){
      point[0] = coords[i].first;
      point[1] = coords[i].second;
      point[2] = 1;
      checkerboardCorners_.push_back(point);
    }  
    
    DrawAllCheckerboardCorners_();  
    CallbackObj_->CheckerboardSelected(checkerboardCorners_, false); 
  
    //paint it
#ifdef WIN32
  this->Refresh();
#else
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(),pe);
#endif
    NumLeftClicks_=0; 
  }
}


void  ImageCanvasCheckerboardExtract::
DisplayCheckerboardCorners(ICECCallbackInterface& obj,
                           std::vector<BIAS::HomgPoint2D> &corners,
                           double rescaleFactor){ 
  checkerboardCorners_.clear();
  for(unsigned i=0;i<corners.size();i++)
    checkerboardCorners_.push_back(corners[i]);
  DrawAllCheckerboardCorners_(rescaleFactor); 
}
    

void ImageCanvasCheckerboardExtract::
ManuallySetSelectionMode(bool SelectAllCorners){
  bEnforceAllCorners_ = SelectAllCorners;
}


BEGIN_EVENT_TABLE (ImageCanvasCheckerboardExtract, ImageCanvas)
    EVT_LEFT_DOWN  (ImageCanvasCheckerboardExtract::OnLeftMouseButton) 
    EVT_RIGHT_DOWN  (ImageCanvasCheckerboardExtract::OnRightMouseButton)
    EVT_MOTION      (ImageCanvasCheckerboardExtract::OnMouseMove)
    EVT_PAINT       (ImageCanvasCheckerboardExtract::OnPaint)
END_EVENT_TABLE()


void ImageCanvasCheckerboardExtract::
OnLeftMouseButton(wxMouseEvent &event)
{
  SelectCheckerboard_=true;
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  double x =  dc.DeviceToLogicalX( pos.x );
  double y =  dc.DeviceToLogicalY( pos.y );

  if(bEnforceAllCorners_){
    if(NumLeftClicks_==0)
      checkerboardCorners_.clear();

    BIAS::HomgPoint2D point(x,y);
    checkerboardCorners_.push_back(point);
    NumLeftClicks_++;
    DrawAllCheckerboardCorners_(); 
    
    if(NumLeftClicks_ == xCorners_*yCorners_){     
      SelectCheckerboard_=false;
      NumLeftClicks_=0;     
      CallbackObj_->CheckerboardSelected(checkerboardCorners_,false);             
    }
  }
  else{
    switch (NumLeftClicks_){
    case 0:
      first_[0]=x; first_[1]=y;first_[2] = 1;
      second_ = third_ = forth_ = 0.0;
      NumLeftClicks_++;
      break;
    case 1:
      {
        second_[0]=x; second_[1]=y; second_[2] = 1;
        NumLeftClicks_++;
        break;        
      }
    case 2:
      {
        third_[0]=x; third_[1]=y; third_[2] = 1;
        NumLeftClicks_++;
        break;        
      }
    case 3:
      {
        forth_[0]=x; forth_[1]=y; forth_[2] = 1;      
        checkerboardCorners_.clear();
        checkerboardCorners_.push_back(first_);
        checkerboardCorners_.push_back(second_);
        checkerboardCorners_.push_back(third_);
        checkerboardCorners_.push_back(forth_);

        DrawCheckerboardCorners_(first_,second_,third_,forth_); 
        SelectCheckerboard_=false;
        NumLeftClicks_=0;  
        CallbackObj_->CheckerboardSelected(checkerboardCorners_,true);             
      }
      break;
    default:
      BIASERR("something strange happend");
      break;
    }
  }
}



void ImageCanvasCheckerboardExtract::
OnRightMouseButton(wxMouseEvent &event)
{
  if(SelectCheckerboard_){
    if(bEnforceAllCorners_){
      checkerboardCorners_.clear();      
    }
    else{
      first_ = second_ = third_ = forth_ = 0.0;
    }  

    NumLeftClicks_ = 0;
    SelectCheckerboard_=false;
    RefreshDrawIm_();
#ifdef WIN32
  this->Refresh();
#else
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(),pe);
#endif
  }//end  if(SelectCheckerboard_)
  else
    event.Skip();
}

void ImageCanvasCheckerboardExtract::
OnPaint(wxPaintEvent &event)
{  
  event.Skip();
}


void ImageCanvasCheckerboardExtract::
OnMouseMove(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
   
  wxPoint pos = event.GetPosition();
  int x = dc.DeviceToLogicalX( pos.x );
  int y = dc.DeviceToLogicalY( pos.y );
  if(bEnforceAllCorners_){    
    // DrawAllCheckerboardCorners_(); 
  }
  else{
    if( SelectCheckerboard_ && NumLeftClicks_>=1){    
      if (NumLeftClicks_==1){    
        second_[0]=x;
        second_[1]=y;   
      } 
      else if(NumLeftClicks_==2){     
        third_[0]=x;
        third_[1]=y;     
      }
      else if(NumLeftClicks_==3){     
        forth_[0]=x;
        forth_[1]=y;     
      }   
      DrawCheckerboardCorners_(first_,second_,third_,forth_);   
    }   
  }
  event.Skip();
}

void ImageCanvasCheckerboardExtract::
DrawAllCheckerboardCorners_(double rescaleFactor)
{
  unsigned char red[]={255, 0, 0};
  unsigned char green[]={0,255, 0};
  unsigned char blue[]={0,0,255};

  RefreshDrawIm_();

  for(unsigned i =0 ;i<checkerboardCorners_.size();i++){
    stringstream text;
    text<<i;
    
    if((rgbim_.GetWidth() % 4) == 0){
      ImageDraw<unsigned char>::Text(rgbim_,text.str().c_str(),
                                     (unsigned)(checkerboardCorners_[i][0]*rescaleFactor),
                                     (unsigned)(checkerboardCorners_[i][1]*rescaleFactor),
                                     ColourRGB<unsigned char>(255, 0, 0));
    }
    if(i==0)
      ImageDraw<unsigned char>::
        CircleCenterFilled(rgbim_,
                           (unsigned)(checkerboardCorners_[i][0]*rescaleFactor),
                           (unsigned)(checkerboardCorners_[i][1]*rescaleFactor), 3,green);      
    else if(i==checkerboardCorners_.size()-1)
      ImageDraw<unsigned char>::
        CircleCenterFilled (rgbim_,
                            (unsigned)(checkerboardCorners_[i][0]*rescaleFactor),
                            (unsigned)(checkerboardCorners_[i][1]*rescaleFactor), 3,blue);

    else
      ImageDraw<unsigned char>::
        CircleCenterFilled (rgbim_,
                            (unsigned)(checkerboardCorners_[i][0]*rescaleFactor),
                            (unsigned)(checkerboardCorners_[i][1]*rescaleFactor), 3,red);      
  }

#ifdef WIN32
  this->Refresh();
#else
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(),pe);
#endif
}


void ImageCanvasCheckerboardExtract::
DrawCheckerboardCorners_(BIAS::HomgPoint2D first,
                         BIAS::HomgPoint2D second, 
                         BIAS::HomgPoint2D third,
                         BIAS::HomgPoint2D forth,
                         double rescaleFactor)
{
  unsigned char red[3] = {255, 0, 0};
  unsigned char green[3] = {0, 255, 0};
  unsigned char blue[3] = {0, 0, 255};
  
  unsigned int start[2], stop[2];

  RefreshDrawIm_();

  if(first[0]!=0.0 && first[1]!=0.0 && second[0]!=0.0 && second[1]!=0.0)
    { 
      start[0] = (int)(first[0]*rescaleFactor);
      start[1] = (int)(first[1]*rescaleFactor);
      stop[0] = (int)(second[0]*rescaleFactor);
      stop[1] = (int)(second[1]*rescaleFactor);
      ImageDraw<unsigned char>::Line(rgbim_, start,stop, green);

      if(third_[0] != 0.0 && third_[1] != 0.0) {
        start[0] = (int)(second[0]*rescaleFactor);
        start[1] = (int)(second[1]*rescaleFactor);
        stop[0] = (int)(third[0]*rescaleFactor);
        stop[1] = (int)(third[1]*rescaleFactor);
        ImageDraw<unsigned char>::Line(rgbim_,start,stop, green);
      }
      if(forth_[0] != 0.0 && forth_[1] != 0.0){
        start[0] = (int)(third[0]*rescaleFactor);
        start[1] = (int)(third[1]*rescaleFactor);
        stop[0] = (int)(forth[0]*rescaleFactor);
        stop[1] = (int)(forth[1]*rescaleFactor);
        ImageDraw<unsigned char>::Line(rgbim_,start,stop, blue);
        start[0] = (int)(forth[0]*rescaleFactor);
        start[1] = (int)(forth[1]*rescaleFactor);
        stop[0] = (int)(first[0]*rescaleFactor);
        stop[1] = (int)(first[1]*rescaleFactor);
        ImageDraw<unsigned char>::Line(rgbim_,start,stop, red);
       
      }
  
    }


#ifdef WIN32
  this->Refresh();
#else
   wxPaintEvent pe;
   wxPostEvent(this->GetEventHandler(),pe);
#endif
}


void ImageCanvasCheckerboardExtract::RefreshDrawIm_()
{
  Image<unsigned char> tmprgb;
  if (origim_.GetColorModel()!=ImageBase::CM_RGB){
    if (ImageConvert::Convert(origim_, tmprgb, ImageBase::CM_RGB)!=0){
      BIASERR("error converting image");
    }
  } else {
    tmprgb=origim_;
  }
  Show(origim_, tmprgb, ImageName_);
}
