#ifndef __ImageCanvasGLBase_hh__
#define __ImageCanvasGLBase_hh__


/** \author Jan Woetzel 01/2003 */

// must be first
#include "biasgl.h"

#include <bias_config.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error Please recompile BIAS with USE_WXWIDGETS to use this lib. 
#endif // BIAS_HAVE_WXWIDGETS

#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use this lib. 
#endif // BIAS_HAVE_OPENGL

// BIAS
#include <Gui/OpenGLCanvas.hh>
#include <Gui/ImgObjGL.hh>

// STD
#include <string>

//#include <Base/Image/Image.hh>
#include <Base/Image/ImageBase.hh>




/** 
@class ImageCanvasGLBase
@ingroup g_gui
@brief 2D image display canvas using OpenGL rendering internally. 

Thus zooming, interpolation etc is easy and very fast. 
In particular useful if you use shared contexts to display OPenGL Image (ids).
This class should not have extended dependencies. 
Use The CG Version to display non power of two textures bceause you need a shder for display. 

An OpenGL texture (id) is displayed. 
This can be created by feeding in a BIAS image, the "value" image.

Thus the true values from "valueimage" are displayed even with fewer bit depth display.
\author Jan Woetzel */
namespace BIAS {
  class BIASGui_EXPORT ImageCanvasGLBase : public BIAS::OpenGLCanvas 
  {	
  public:
    virtual ~ImageCanvasGLBase(); ///< destructor

    // ctors
    ImageCanvasGLBase(
      wxFrame* parent
      ,wxWindowID id = -1
      ,const wxPoint& pos = wxDefaultPosition
      ,const wxSize& size = wxDefaultSize
      ,long style = 0
      ,const wxString& name = AsciiToWx("ImageCanvasGLBase")
      ,int* attribList = NULL
      ,const wxPalette& palette = wxNullPalette );

    /// 
    /// constructor using a shared context 
    ImageCanvasGLBase(
      wxFrame* parent
      ,wxGLContext* sharedContext = NULL
      ,wxWindowID id = -1
      ,const wxPoint& pos = wxDefaultPosition
      ,const wxSize& size = wxDefaultSize
      ,long style=0
      ,const wxString& name=AsciiToWx("ImageCanvasGLBase")
      ,int* attribList = 0
      ,const wxPalette& palette = wxNullPalette );

    virtual void DisplayCalls();

    virtual void InitMembers();

    // unique id for OpenGL texture object
    GLuint GetTexId() const;

    // GL_TEXTURE_2D or GL_TEXTURE_RECTANGLE_NV
    GLenum GetTexTarget() const;

    bool ValidTexobj(const ImgObjGL & t) const;

    /** feed in an existing, possibly shared, GL texture obj for display
    note: it is *no* good idea  to call thsi method Show because it conflicts with WX::Show(bool) (JW) */
    int Set(const ImgObjGL & NewTexobj, 
      const std::string & name=std::string("ImgObjGL") );

    int Set(
      BIAS::ImageBase & img,
      //BIAS::Image<unsigned char>& img, 
      const std::string & name=std::string("Image<unsigned char>"),
      const bool & forcePow2tex=true,
      const bool & flipY=true
      );

    /** sets imaeg as a RECT texture JW */
    int SetRECT(
      BIAS::ImageBase & img,
      //BIAS::Image<unsigned char>& img, 
      const std::string & name=std::string("RECT Image<unsigned char>"),
      const bool & flipY=true
      );


    //int Set(BIAS::ImageBase& img, const std::string & name=std::string("ImageBase"));


    //void CreateTexObjFromValueImage_andBind();

    ImgObjGL GetTexobj() const {return texobj;};
    ImgObjGL & GetTexobj() {return texobj;};

    /** set to true for prettier OpenGL texture mipmapping display (which costs performance) JW */
    void EnableMipMapping(const bool & avoidAliasingByMipmapping=true);



  protected:
    // OpenGL texture really used for display: 
    // textraget must be GL_TEXTURE_2D in this Base class 
    // because a shader is required to display a GL_TEXTURE_RECTANGLE_NV
    // look for a derived class adding Cg shaders to use rectangle textures without padding them to pow2. 
    // holds all gl specific texture information 
    BIAS::ImgObjGL texobj;

    // name identifier
    std::string name;

    // 0..1 (Orig) coordinate of an interest point used to determine a zoom region
    BIAS::Vector2<float> InterestPoint; 
    // scale factor for zoom, >1 to zoom in 
    float scalefactor; 

    //DECLARE_EVENT_TABLE();
  }; // ImageCanvasGL

} // namespace BIAS 

#endif
