#ifndef __VideoSource_Controller_USBPMD_HH__
#define __VideoSource_Controller_USBPMD_HH__

#include <bias_config.h>

#include <Base/Common/W32Compat.hh>
#include <VideoSource/VideoSource_Base.hh>

#ifdef BIAS_HAVE_PMD_PMDZess
#include <VideoSource/VideoSource_usbPMD.hh>
#endif

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>

#ifndef BIAS_HAVE_WXWIDGETS
#  error You need BIAS_HAVE_WXWIDGETS. Recompile BIAS with USE_WXWIDGETS.
#endif

#include <wx/wx.h>



namespace BIAS {

class BIASGui_EXPORT VideoSource_Controller_usbPMD : public ::wxFrame{
public:

  VideoSource_Controller_usbPMD(BIAS::VideoSource_usbPMD *cam, wxFrame *parent, 
		double min2DGain = 0.0,double max2DGain=3000.0,
    double min2DShutter=1000.02, double max2DShutter=8000.02,		
    double min3DShutter =1000.0,double max3DShutter=8000.0);

  ~VideoSource_Controller_usbPMD();

  void Initialize();

  void SetVideoSource(BIAS::VideoSource_usbPMD *cam);
  void SetShutter3DRange(unsigned int min,unsigned int max);
  void SetShutter2DRange(unsigned int min,unsigned int max);
  void SetGain2DRange(unsigned int min,unsigned int max);
  void Update(); // trigger an update, useful for auto-mode
  

  wxBoxSizer        *vBox;
    
  enum {
    ID_HIDE,
    ID_GAIN2D,
    ID_SHUTTER3D,
    ID_SHUTTER2D,
    ID_MODULATIONFREQUENCY,
    ID_BUTTON_UPDATE,
    ID_MODULATION_TEXT,
    ID_TEXT
  };

private:

  void OnEscape(wxCommandEvent& event);
  void OnHide(wxCommandEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnShutter3D(wxScrollEvent& event);
  void OnShutter2D(wxScrollEvent& event);
  void OnGain2D(wxScrollEvent& event);
  void OnUpdate(wxCommandEvent& event) ;
  void OnModFreq(wxCommandEvent& event) ;

protected:
  
  BIAS::VideoSource_usbPMD *Cam_;
  
  wxSlider          *SliderGain2D;
  wxSlider          *SliderShutter2D;
  wxSlider          *SliderShutter3D;
  wxBoxSizer        *sliderBox_ShGa;
  bool               sliderBox_ShGa_Visible;
 
  bool               vBoxVisible;

  wxButton          *showButton;
  wxButton          *setModFreqButton;
  wxButton          *update_Button;
  wxTextCtrl        *modulationFreqText;
  double minShutter2D_, maxShutter2D_,minShutter3D_, maxShutter3D_, minGain2D_,maxGain2D_;

  DECLARE_EVENT_TABLE()

private:
    };

}
#endif
