#ifndef __CylindricDepthTrackingPanorama_hh_
#define __CylindricDepthTrackingPanorama_hh_

#include <Base/Common/W32Compat.hh>
#include <Geometry/Projection.hh>
#include <Geometry/ProjectionParametersCylindric.hh>
#include <Image/CylindricDepthPanorama.hh>
#include <Image/MixtureOfGaussians.hh>

namespace BIAS
{

class BIASImage_EXPORT CylindricDepthTrackingPanorama : public CylindricDepthPanorama 
{
public:
  CylindricDepthTrackingPanorama(Projection *initialProjection_ = NULL, unsigned int channels=1, float depthScale_ = 1.0);
	virtual ~CylindricDepthTrackingPanorama();
  /** Call Process to update the measurement */
  int ProcessCylindricProjection(BIAS::Image<float> &depthMap,
                                 BIAS::Projection & p,
                                 BIAS::Image<unsigned char> *texture=NULL);

  int LoadPanorama(const std::string filename);
  int SavePanorama(const std::string filename);
  
  int GetTriangleMesh(BIAS::TriangleMesh &mesh);
  
  int GetPanorama(BIAS::Image<float> &image, 
                  BIAS::Image<unsigned char> &texture);

protected:
  BIAS::Image<float> panoramaNonStatic_;
  BIAS::Image<unsigned char> panoramaNonStaticTexture_;
  BIAS::MixtureOfGaussians<float> *mog_;
  BIAS::Image<unsigned char> visualisation_;
  BIAS::Image<unsigned char> difference_;
  unsigned int dTexChannels_;
  float fDepthScale_;
private:
  BIAS::MOGHelper::Corners bb_;
  bool bInited_;
  int doPanorama(BIAS::Image<float> &depthMap,
      BIAS::Projection & p,
      BIAS::Image<unsigned char> *texture=NULL);
  
  int doTrack(BIAS::Image<float> &depthMap,
      BIAS::Projection & p,
      BIAS::Image<unsigned char> *texture=NULL);
  
  BIAS::Image<float> normalizedDepthMap_;
};

}

#endif /*__CylindricDepthTrackingPanorama_hh_*/
