#include "ForwardMappingNearestNeighbour.hh"
#include <Base/Common/BIASpragma.hh>

using namespace BIAS;
using namespace std;

template <class InputStorageType, class OutputStorageType>
ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
ForwardMappingNearestNeighbour() {
  fillblack_ = true;
}

template <class InputStorageType, class OutputStorageType>
ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
ForwardMappingNearestNeighbour(const ForwardMappingNearestNeighbour& src) {
  
}

template <class InputStorageType, class OutputStorageType>
ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
~ForwardMappingNearestNeighbour() {
  
}

template <class InputStorageType, class OutputStorageType>
ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>&
ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
operator=(const ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>& src) {
  SourceP_ = src.SourceP_;
  SinkP_ = src.SinkP_;
  intermediate_ = src.intermediate_;
  intermediateTex_ = src.intermediateTex_;
  dSourceWidth_ = src.dSourceWidth_;
  dSourceHeight_ = src.dSourceHeight_;
  dTargetWidth_ = src.dTargetWidth_;
  dTargetHeight_ = src.dTargetHeight_;

  return *this;
}

template <class InputStorageType, class OutputStorageType> 
int ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
Map(const BIAS::Image<InputStorageType> *in, BIAS::Image<OutputStorageType>& out, const BIAS::Image<InputStorageType> *texin, BIAS::Image<OutputStorageType>& texout) {
  bool withTex_= (texin == NULL) ? false : true;
  in->GetROI()->GetCorners(ulx_, uly_, lrx_, lry_);
  lrx_--; lry_--;
  const InputStorageType** idaSource_ = in->GetImageDataArray();
  if (out.IsEmpty()) {
    out.Init(dTargetWidth_, dTargetHeight_);
  }
  out.FillImageWithConstValue((OutputStorageType)0);
  OutputStorageType** idaTarget_ = out.GetImageDataArray();
  const  InputStorageType** idaTexIn_ = NULL; 
  OutputStorageType** idaTexOut_ = NULL; 
  if (withTex_) {
    idaTexIn_ = texin->GetImageDataArray();
    if (texout.IsEmpty())
      texout.Init(dTargetWidth_, dTargetHeight_, dTexChannels_);
    texout.FillImageWithConstValue((OutputStorageType)0);
    idaTexOut_ = texout.GetImageDataArray();
  }
  intermediate_.FillImageWithConstValue((OutputStorageType)0);
  OutputStorageType** idaIntermediate_ = intermediate_.GetImageDataArray();
  if (withTex_) {
    intermediateTex_.FillImageWithConstValue((OutputStorageType)0);
  }
  OutputStorageType** idaIntermediateTex_ = intermediateTex_.GetImageDataArray();
  int cornerpos[8] = { ulx_,uly_, lrx_,uly_, ulx_,lry_, lrx_,lry_ };
  HomgPoint2D corners[4];
  for (int i=0;i<4;i++) {
    HomgPoint2D p2d;
    p2d[0]=cornerpos[2*i];
    p2d[1]=cornerpos[2*i+1];
    p2d[2]=1;
    float fDepth = idaSource_[ cornerpos[2*i+1] ][ cornerpos[2*i] ];
    if (fDepth < 5.0) 
      fDepth = 1000.0;
    HomgPoint3D p3d = (BIAS::HomgPoint3D)SourceP_.UnProjectToPoint(p2d, fDepth);
    if (SinkP_.DoesPointProjectIntoImage(p3d, p2d)) {
      //cout<<"Point:"<<p3d<<" projects to sink at:"<<p2d<<endl;
      p2d.Homogenize();
      corners[i] = p2d;
    }
    else { 
      //cout<<"Point:"<<p3d<<" does not project into sink"<<endl;
      if (p2d[0] < 0)
        p2d[0] = 0;
      if (p2d[0] >= dTargetWidth_)
        p2d[0] = dTargetWidth_ -1;
      if (p2d[1] < 0)
        p2d[1] = 0;
      if (p2d[1] >= dTargetHeight_)
        p2d[1] = dTargetHeight_ -1;
      corners[i] = p2d;
    }
  }
  boundMin_[0]=0; boundMin_[1]=0; boundMin_[2] = 1;
  boundMax_[0]=0; boundMax_[1]=0; boundMax_[2] = 1;
  if (corners[0][0]<corners[2][0])
    boundMin_[0] = corners[0][0];
  else
    boundMin_[0] = corners[2][0];
  
  if (corners[0][1]<corners[1][1])
    boundMin_[1] = corners[0][1];
  else
    boundMin_[1] = corners[1][1];

  // max x
  if (corners[3][0]>corners[1][0])
    boundMax_[0] = corners[3][0];
  else
    boundMax_[0] = corners[1][0];

  // max y
  if (corners[3][1]>corners[2][1])
    boundMax_[1] = corners[3][1];
  else
    boundMax_[1] = corners[2][1];

  for (int y=uly_;y<lry_;y++) {
    for (int x=ulx_;x<lrx_;x++) {
      BIAS::HomgPoint2D point2d_src;
      point2d_src[0] = x;
      point2d_src[1] = y;
      point2d_src[2] = 1;
      float depth_src = idaSource_[y][x];
      if (depth_src > 500) {
        BIAS::HomgPoint3D point3d = (BIAS::HomgPoint3D)SourceP_.UnProjectToPoint(point2d_src, depth_src);
        BIAS::HomgPoint2D point2d_dst;
        if (SinkP_.DoesPointProjectIntoImage(point3d, point2d_dst)) {
          //cout<<"Point:"<<point3d<<" projects to sink at:"<<point2d_dst<<endl;
          point2d_dst.Homogenize();
          int xdst = point2d_dst[0];
          int ydst = point2d_dst[1];
          idaIntermediate_[ydst][xdst] = depth_src;
          if (withTex_) {
            for (unsigned int i=0;i<dTexChannels_;i++)
              idaIntermediateTex_[ydst][xdst*dTexChannels_+i] = idaTexIn_[y][x*dTexChannels_+i];
          }
        }
      }
    }
  }

  // fill black areas with neighbouring pixels
  if (fillblack_) {
    for (unsigned int y=boundMin_[1]+5;y<boundMax_[1]-5;y++) {
      for (unsigned int x=boundMin_[0]+5;x<boundMax_[0]-5;x++) {
        if (idaIntermediate_[y][x] == 0) {
          for (unsigned int mod=1;mod<6;mod++) {
            if (idaIntermediate_[y][x+mod] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y][x+mod];
              break;
            }
            if (idaIntermediate_[y][x-mod] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y][x-mod]; 
              break;
            }
            if (idaIntermediate_[y+mod][x+mod] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y+mod][x+mod]; 
              break;
            }
            if (idaIntermediate_[y+mod][x] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y+mod][x];
              break;
            }
            if (idaIntermediate_[y+mod][x-mod] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y+mod][x-mod]; 
              break;
            }
            if (idaIntermediate_[y-mod][x+mod] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y-mod][x+mod]; 
              break;
            }
            if (idaIntermediate_[y-mod][x] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y-mod][x]; 
              break;
            }
            if (idaIntermediate_[y-mod][x-mod] != 0) {
              idaTarget_[y][x] = idaIntermediate_[y-mod][x-mod]; 
              break;
            }
          }
        } else {
          idaTarget_[y][x] = idaIntermediate_[y][x];
        }
      }
    }
  }
  
  if (withTex_ && fillblack_) {
    if (dTexChannels_ == 1) {
      for (unsigned int y=boundMin_[1]+5;y<boundMax_[1]-5;y++) {
        for (unsigned int x=boundMin_[0]+5;x<boundMax_[0]-5;x++) {
          if (idaIntermediateTex_[y][x] == 0) {
            for (unsigned int mod=1;mod<6;mod++) {
              if (idaIntermediateTex_[y][x+mod] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y][x+mod];
                break;
              }
              if (idaIntermediateTex_[y][x-mod] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y][x-mod];
                break;
              }
              if (idaIntermediateTex_[y+mod][x+mod] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y+mod][x+mod];
                break;
              }
              if (idaIntermediateTex_[y+mod][x] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y+mod][x];
                break;
              }
              if (idaIntermediateTex_[y+mod][x-mod] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y+mod][x-mod];
                break;
              }
              if (idaIntermediateTex_[y-mod][x+mod] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y-mod][x+mod];
                break;
              }
              if (idaIntermediateTex_[y-mod][x] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y-mod][x];
                break;
              }
              if (idaIntermediateTex_[y-mod][x-mod] != 0) {
                idaTexOut_[y][x] = idaIntermediateTex_[y-mod][x-mod];
                break;
              }
            }
          } else {
            idaTexOut_[y][x] = idaIntermediateTex_[y][x];
          }
        }
      }
    } else {
      for (unsigned int y=boundMin_[1]+5;y<boundMax_[1]-5;y++) {
        for (unsigned int x=((int)boundMin_[0]+5)*dTexChannels_;x<((int)boundMax_[0]-5)*dTexChannels_;x+=dTexChannels_) {
          unsigned int val = 0;
          val = GetSummedValue_(&(idaIntermediateTex_[y][x]));
          if (val == 0) {
            for (unsigned int mod=1;mod<6;mod++) {
              unsigned int xmod = mod * dTexChannels_;
              if (GetSummedValue_(&(idaIntermediateTex_[y][x+xmod])) != 0) {
                SetValue_(&(idaIntermediateTex_[y][x+xmod]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y][x-xmod])) != 0) {
                SetValue_(&(idaIntermediateTex_[y][x-xmod]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y+mod][x+xmod])) != 0) {
                SetValue_(&(idaIntermediateTex_[y+mod][x+xmod]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y+mod][x])) != 0) {
                SetValue_(&(idaIntermediateTex_[y+mod][x]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y+mod][x-xmod])) != 0) {
                SetValue_(&(idaIntermediateTex_[y+mod][x-xmod]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y-mod][x+xmod])) != 0) {
                SetValue_(&(idaIntermediateTex_[y-mod][x+xmod]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y-mod][x])) != 0) {
                SetValue_(&(idaIntermediateTex_[y-mod][x]), &(idaTexOut_[y][x]));
                break;
              }
              if (GetSummedValue_(&(idaIntermediateTex_[y-mod][x-xmod])) != 0) {
                SetValue_(&(idaIntermediateTex_[y-mod][x-xmod]), &(idaTexOut_[y][x]));
                break;
              }
            }
          } else {
            SetValue_(&(idaIntermediateTex_[y][x]), &(idaTexOut_[y][x]));
          }
        }
      }
    }
  }

  return 0;
}

template <class InputStorageType, class OutputStorageType> 
unsigned int ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
GetSummedValue_(const OutputStorageType* pixel) {
  unsigned int ret=0;
  for (unsigned int i=0;i<dTexChannels_;i++) {
    ret+=pixel[i];
  }
  return ret;
}

template <class InputStorageType, class OutputStorageType> 
void ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>::
SetValue_(const OutputStorageType* in, OutputStorageType* out) {
  for (unsigned int i=0;i<dTexChannels_;i++) {
    out[i]=in[i];
  }
}

template class ForwardMappingNearestNeighbour<unsigned char, unsigned char>;
template class ForwardMappingNearestNeighbour<float, float>;
template class ForwardMappingNearestNeighbour<unsigned char, float>;
template class ForwardMappingNearestNeighbour<float, unsigned char>;
