/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Image/TextureMapping.hh"
#include "Image/TextureTransformDisparity.hh"

using namespace BIAS;
using namespace std;


template <class InputStorageType, class OutputStorageType>
TextureMapping<InputStorageType, OutputStorageType>::
~TextureMapping() 
{
  if (T_!=NULL) delete T_;
}

template <class InputStorageType, class OutputStorageType>
TextureMapping<InputStorageType, OutputStorageType>::
TextureMapping()
{
   T_ = new TextureTransformDisparity;
}


template <class InputStorageType, class OutputStorageType>
TextureMapping<InputStorageType, OutputStorageType>::
TextureMapping(const TextureMapping & src) : 
  T_(NULL)
{
  (*this) = src;
}

template <class InputStorageType, class OutputStorageType>
TextureMapping<InputStorageType, OutputStorageType> & 
TextureMapping<InputStorageType, OutputStorageType>::
operator=(const TextureMapping & src)
{
  if (T_!=NULL) delete T_;
  T_ = src.T_->Clone();
  return *this;
}

template <class InputStorageType, class OutputStorageType>
int TextureMapping<InputStorageType, OutputStorageType>::
GetSourceCoordinates_(const HomgPoint2D & sink,
                      HomgPoint2D & source) const {
  return T_->MapBackward(sink, source);
}

template <class InputStorageType, class OutputStorageType>
int TextureMapping<InputStorageType, OutputStorageType>::
GetJacobian_(const HomgPoint2D& sink, Matrix2x2<double>& Jacobian) const {
  return T_->TextureJacobianBackward(sink, Jacobian);
}


// TODO FIXME win32 template instantiation problem (JW)
template class TextureMapping<unsigned char, unsigned char>;
template class TextureMapping<float, float>;
template class TextureMapping<unsigned char, float>;
template class TextureMapping<float, unsigned char>;

#if defined(BUILD_IMAGE_CHAR)
// TODO FIXME win32 template instantiation problem (JW)
template class TextureMapping<unsigned char, char>;
template class TextureMapping<char, char>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class TextureMapping<unsigned short,unsigned short>;
#endif

#if defined(BUILD_IMAGE_SHORT)
template class TextureMapping<short, short>;
#endif

#if defined(BUILD_IMAGE_SHORT)&&defined(BUILD_IMAGE_USHORT)
template class TextureMapping<unsigned short, short>;
#endif

#if defined(BUILD_IMAGE_INT)
template class TextureMapping<int,int>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class TextureMapping<unsigned short,float>;
#endif

#if defined(BUILD_IMAGE_USHORT) && defined(BUILD_IMAGE_INT)
template class TextureMapping<unsigned short,int>;
#endif

#if defined(BUILD_IMAGE_DOUBLE)
template class TextureMapping<double,double>;
#endif
