/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragmaStart.hh>

#ifndef __TextureMapping_hh__
#define __TextureMapping_hh__

#include <Image/BackwardMapping.hh>
#include <Image/TextureTransform.hh>

namespace BIAS {

  /** @class TextureMapping
      @ingroup g_filter
      @brief  Maps source pixel to sink pixel given any TextureTransform

      @author koeser
      @date   2008
  */
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT TextureMapping 
    : public  BackwardMapping<InputStorageType, OutputStorageType> {

  public:

    virtual ~TextureMapping();

    TextureMapping();

    /**  required because assignment is 
         @author Jan Woetzel */
    TextureMapping(const TextureMapping & src);

    /**  required because of const members 
         @author Jan Woetzel */
    TextureMapping<InputStorageType, OutputStorageType> & 
      operator=(const TextureMapping & src);

    /** @brief create clone of your texture tranform and set for mapping  */
    inline void SetTextureTransform(const TextureTransform& T) {
      if (T_!=NULL) delete T_;
      T_ = T.Clone();
    }
      

    virtual int GetBoundingBox(unsigned int srcwidth, 
                               unsigned int srcheight,
                               unsigned int sinkwidth, 
                               unsigned int sinkheight,
                               int &tlx, int &tly, 
                               int &brx, int &bry)
    { 
      tlx=0; tly=0;     
      brx=sinkwidth; bry=sinkheight;   

      HomgPoint2D sink;
      T_->MapForward(HomgPoint2D(0,0), sink);
      if (tlx>sink[0]) tlx = int(rint(sink[0]-1.0));
      if (tly>sink[1]) tly = int(rint(sink[1]-1.0));
      if (brx<sink[0]) brx = int(rint(sink[0]+1.0));
      if (bry<sink[1]) bry = int(rint(sink[1]+1.0));
      T_->MapForward(HomgPoint2D(srcwidth-1,0), sink);
      if (tlx>sink[0]) tlx = int(rint(sink[0]-1.0));
      if (tly>sink[1]) tly = int(rint(sink[1]-1.0));
      if (brx<sink[0]) brx = int(rint(sink[0]+1.0));
      if (bry<sink[1]) bry = int(rint(sink[1]+1.0));
      T_->MapForward(HomgPoint2D(0,srcheight-1), sink);
      if (tlx>sink[0]) tlx = int(rint(sink[0]-1.0));
      if (tly>sink[1]) tly = int(rint(sink[1]-1.0));
      if (brx<sink[0]) brx = int(rint(sink[0]+1.0));
      if (bry<sink[1]) bry = int(rint(sink[1]+1.0));
      T_->MapForward(HomgPoint2D(srcwidth-1, srcheight-1), sink);
      if (tlx>sink[0]) tlx = int(rint(sink[0]-1.0));
      if (tly>sink[1]) tly = int(rint(sink[1]-1.0));
      if (brx<sink[0]) brx = int(rint(sink[0]+1.0));
      if (bry<sink[1]) bry = int(rint(sink[1]+1.0));
     
      return 0; 
    } 

  protected:
    /** @brief analytic jacobian */
    virtual int GetJacobian_(const HomgPoint2D& sink,
                             Matrix2x2<double>& Jacobian) const;  

    /** @brief Reimplementation for Pixel transformation, 
               takes sink and computes coordinates in source.
     */
    virtual int GetSourceCoordinates_(const HomgPoint2D& sink,
                                      HomgPoint2D& source) const;

    /// Textures for backward mapping: Run over sink and compute source
    TextureTransform *T_;
  
  };

} // namespace 

#endif

#include <Base/Common/BIASpragmaEnd.hh>

