/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel


   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
#include "Base/Image/ScanLine.hh"
#include <float.h>

template <class StorageType> 
int RegionMatcher::
LineMatcherNCC(const unsigned int p1[2], const unsigned int start2[2], 
               const unsigned int end2[2], 
               const StorageType **ida1, const StorageType **ida2,
               const StorageType **grad1, const StorageType **grad2, 
               const unsigned int halfwinsize, 
               const unsigned int epsilon,
               const double gradientscale, 
               unsigned int result[2], 
               double& correlation) const
{
  int res=0;
  ScanLine scan(start2, end2, epsilon);
  unsigned int next[2];
  StorageType grad_threshold=
    (StorageType)rint((double)grad1[p1[1]][p1[0]]*gradientscale);
  double tmpcorr;

  correlation=-1.1;
  while (scan.GetNext(next)){
    BIASDOUT(D_REGION_MATCHER_LINE, "point ("<<next[0]<<", "<<next[1]<<")");
    if (grad2[next[1]][next[0]]>=grad_threshold){
      NCC(p1, next, ida1, ida2, halfwinsize, tmpcorr);
//        cerr << "p1 ("<<p1[0]<<", "<<p1[1]<<")\tnext ("<<next[0]<<", "
//  	   <<next[1]<<")\tcorr: "<<tmpcorr<<endl;
      BIASDOUT(D_REGION_MATCHER_LINE, "gradient is big enough "
	       <<(int)(grad2[next[1]][next[0]])<<" > "<<gradientscale<<" * "
	       <<(int)(grad1[p1[1]][p1[0]])<<"\tcorrelation: "<<tmpcorr);
      if (tmpcorr>correlation){ 
	result[0]=next[0];
	result[1]=next[1];
	correlation=tmpcorr;
      }
    } else { // if (grad2[next[1]][next[0]]>grad_threshold){
      BIASDOUT(D_REGION_MATCHER_LINE, "gradient too small "
	       <<(int)(grad2[next[1]][next[0]])<<" <= "<<gradientscale<<"*"
	       <<(int)(grad1[p1[1]][p1[0]])<<" = "<<(int)grad_threshold);
    }
  } // while
  if (correlation<-1.0) res=-1;

  return res;
}


template <class StorageType> int 
RegionMatcher::
LineMatcherSAD(const unsigned int p1[2], const unsigned int start2[2], 
               const unsigned int end2[2], 
               const StorageType **ida1, const StorageType **ida2,
               const StorageType **grad1, const StorageType **grad2, 
               const unsigned int halfwinsize, const unsigned int epsilon,
               const double gradientscale, unsigned int result[2], 
               double& sad)const 
{
  int res=0;
  ScanLine scan(start2, end2, epsilon);
  unsigned int next[2];
  StorageType grad_threshold=
    (StorageType)rint((double)grad1[p1[1]][p1[0]]*gradientscale);
  double tmpcorr;

  sad=DBL_MAX;
  while (scan.GetNext(next)){
    BIASDOUT(D_REGION_MATCHER_LINE, "point ("<<next[0]<<", "<<next[1]<<")");
    if (grad2[next[1]][next[0]]>=grad_threshold){
      SAD(p1, next, ida1, ida2, halfwinsize, tmpcorr);
//        cerr << "p1 ("<<p1[0]<<", "<<p1[1]<<")\tnext ("<<next[0]<<", "
//  	   <<next[1]<<")\tcorr: "<<tmpcorr<<endl;
      BIASDOUT(D_REGION_MATCHER_LINE, "gradient is big enough "
	       <<(int)(grad2[next[1]][next[0]])<<" > "<<gradientscale<<" * "
	       <<(int)(grad1[p1[1]][p1[0]])<<"\tsad: "<<tmpcorr);
      if (tmpcorr<sad){ 
	result[0]=next[0];
	result[1]=next[1];
	sad=tmpcorr;
      }
    } else { // if (grad2[next[1]][next[0]]>grad_threshold){
      BIASDOUT(D_REGION_MATCHER_LINE, "gradient too small "
	       <<(int)(grad2[next[1]][next[0]])<<" <= "<<gradientscale<<"*"
	       <<(int)(grad1[p1[1]][p1[0]])<<" = "<<(int)grad_threshold);
    }
  } // while
  if (sad==DBL_MAX) res=-1;

  return res;
}


