/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef WIN32
// WIN32 max definition conflicts with numeric_limits::max
#  define NOMINMAX
#  pragma warning(disable: 4251) // DLL interface
//#  include <Base/Common/BIASpragma.hh>
#endif


#include "TrackerSynth.hh"
#include <Base/Math/Random.hh>
#include <limits>


using namespace BIAS;
using namespace std;


template <class StorageType, class CalculationType>
TrackerSynth<StorageType, CalculationType>::
TrackerSynth()
{
  Frame_ = 0;
}

template <class StorageType, class CalculationType>
TrackerSynth<StorageType, CalculationType>::
~TrackerSynth()
{
}

template <class StorageType, class CalculationType>
int TrackerSynth<StorageType, CalculationType>::
Track(PyramidImageInterface<CalculationType>& pim, 
      PyramidImageInterface<CalculationType>& gradx,
      PyramidImageInterface<CalculationType>& grady)
{
  // std::vector<std::vector<HomgPoint2D> > tmp = Points_;
  if (Points_.size()>0){
    for (int i=0; i<5; i++)
      BIASCDOUT(D_TS_REPLACE, "Points_[PyIndex_]["<<i<<"] = "
                <<Points_[PyIndex_][i]<<endl);
  }

  int res = Tracker<StorageType, CalculationType>::Track(pim, gradx, grady);
  //  int res = 0;
  //  Points_ = tmp;
  
  Track_();
  return res;
}


template <class StorageType, class CalculationType>
int TrackerSynth<StorageType, CalculationType>::
Track(PyramidImageInterface<CalculationType>& pim, 
      PyramidImageInterface<CalculationType>& gradx,
      PyramidImageInterface<CalculationType>& grady,
      const std::vector<HomgPoint2D>& prediction,
      const std::vector<Matrix2x2<KLT_TYPE> >& affineprediction)
{
  // std::vector<std::vector<HomgPoint2D> > tmp = Points_;
  if (Points_.size()>0){
    for (int i=0; i<5; i++)
      BIASCDOUT(D_TS_REPLACE, "Points_[*PyIndex_]["<<i<<"] = "
                <<Points_[PyIndex_][i]<<endl);
  }
  int res = Tracker<StorageType, CalculationType>::
    Track(pim, gradx, grady, prediction, affineprediction);
  //int res=0;
  //Points_ = tmp;
  Track_();
  return res;
}


template <class StorageType, class CalculationType>
void TrackerSynth<StorageType, CalculationType>::
Track_()
{
  //AddDebugLevel(D_TS_REPLACE);
  const int pyindex = PyIndex_;
  int index;
  double diff;
  if ((int)Points_.size()>pyindex){
    const int num = Points_[pyindex].size();
    Frame_++;
    cout << "correction : ";
    for (int i=0; i<num; i++){
      if (!Points_[pyindex][i].IsAtInfinity()){
        index = FindNearest_(Points_[pyindex][i], Frame_);
        diff = Points_[pyindex][i].Distance(Tracks_[Frame_][index]);
        BIASCDOUT(D_TS_REPLACE, "old point "<< Tracks_[Frame_-1][index] <<"  "
                  << "correcting: "<<Points_[pyindex][i]<<" --> "
                  <<Tracks_[Frame_][index]<<"  = "<<diff<<endl);
        cout << setprecision(3) << setw(8) << diff ;
        // assert(diff<2.0);
        Points_[pyindex][i] = Tracks_[Frame_][index];
      }
    }
    cout << endl;
    //AddGaussianNoise_();
    //AddOutlier_();
  }
}


template <class StorageType, class CalculationType>
void TrackerSynth<StorageType, CalculationType>::
AddGaussianNoise_()
{
  Random ran;
  const double Sigma = 0.05;
  const int pyindex = PyIndex_; 
  if ((int)Points_.size()>pyindex){
    const int num = (int)Points_[pyindex].size();
    cout << "adding noise: "<<endl;
    for (int i=0; i<num; i++){
      if (!Points_[pyindex][i].IsAtInfinity()){
        cout << setw(4) << i << "  "<<Points_[pyindex][i]<<" --> ";
        Points_[pyindex][i][0] += ran.GetNormalDistributed(0.0, Sigma);
        Points_[pyindex][i][1] += ran.GetNormalDistributed(0.0, Sigma);
        cout << Points_[pyindex][i] << endl;
      }
    }
  }
}

template <class StorageType, class CalculationType>
void TrackerSynth<StorageType, CalculationType>::
AddOutlier_()
{
  Random ran;
  const double fraction = 0.025;
  const double offs = 5.0;
  const int pyindex = PyIndex_; 
  int act_num = 0;
  vector<int> used;
  if ((int)Points_.size()>pyindex){
    const int num = (int)Points_[pyindex].size();
    cout << "adding outlier: "<<endl;
    for (int i=0; i<num; i++){
      if (!Points_[pyindex][i].IsAtInfinity()){
        used.push_back(i);
        act_num++;
      }
    }
    int num_outl = (int)rint(fraction * (double)act_num);
    vector<unsigned> outl;
    for (int i=0; i<num_outl; i++)
      outl.push_back(ran.GetUniformDistributedInt(0u, (unsigned)(act_num-1)));
    
    int index = 0;
    for (unsigned j=0; j<outl.size(); j++){
      index = used[outl[j]];
      assert(!Points_[pyindex][index].IsAtInfinity());
      cout << setw(4)<<index<<"  "<<Points_[pyindex][index] << " --> ";
      Points_[pyindex][index][0] +=
        ran.GetUniformDistributed(-offs, offs);
      Points_[pyindex][index][1] +=
        ran.GetUniformDistributed(-offs, offs);
      cout << Points_[pyindex][index] << endl;
    }

  }

}

template <class StorageType, class CalculationType>
void TrackerSynth<StorageType, CalculationType>::
ReadTracks_(const string& filename)
{
  //AddDebugLevel(D_TS_READ);
  ifstream is(filename.c_str());
  if (!is.good()){
    BIASERR("error opening \""<<filename<<"\"");
    BIASABORT;
  }
  
  string tmp;
  int frame, num_tracks=0, num_frames=0;
  is >> tmp >> num_tracks >> tmp >> num_frames >> tmp;
  BIASCDOUT(D_TS_READ, "found "<<num_tracks<<" in "<<num_frames<<" frames\n");
  const int num_c = 2048;
  char *ttt = new char[num_c];
  is.getline(ttt, num_c);
  //cout << "tmp: "<<tmp<<endl;
  //cout << "line: "<<ttt<<endl;
  delete[] ttt;
  Tracks_.resize(num_frames);
  for (int f=0; f<num_frames; f++){
    is >> frame;
    //cout << "frame: "<<frame<<endl;
    if (frame != f+1){ 
      BIASERR("error reading \""<<filename<<"\"");
      BIASABORT; 
    }
    BIASCDOUT(D_TS_READ, setw(4)<< frame<<" ");
    Tracks_[f].resize(num_tracks);
    for (int t=0; t<num_tracks; t++){
      is >> Tracks_[f][t][0] >> Tracks_[f][t][1];
      BIASCDOUT(D_TS_READ, setw(7)<<Tracks_[f][t][0] <<" "
                << setw(7)<<Tracks_[f][t][1]<< "  ");
      Tracks_[f][t][2] = 1.0;
      if (!is.good()){
        BIASERR("error reading \""<<filename<<"\"");
        BIASABORT;
      }
    }
  }
  is.close();
}


template <class StorageType, class CalculationType>
void TrackerSynth<StorageType, CalculationType>::
FindNearest_(HomgPoint2D& p, int &nearest_frame_index, 
                  int &nearest_track_index)
{
  const int num_frames = (int)Tracks_.size();
  double dist=0;
  double nearest_dist = numeric_limits<double>::max();

  int t;
  nearest_frame_index = -1;
  nearest_track_index = -1;
  for (int f=0; f<num_frames; f++){
    t = FindNearest_(p, f);
    dist = p.Distance(Tracks_[f][t]);
    if (dist <= nearest_dist){
      nearest_dist = dist;
      nearest_frame_index = f;
      nearest_track_index = t;
    }
  }
  BIASASSERT(nearest_frame_index>0);
  BIASASSERT(nearest_dist<0.1);
}

template <class StorageType, class CalculationType>
int TrackerSynth<StorageType, CalculationType>::
FindNearest_(HomgPoint2D& p, const int frame)
{
  int nearest_track_index = -1;
  double dist=0;
  double nearest_dist = numeric_limits<double>::max();
  const int num_tracks = (int)Tracks_[frame].size();
  for (int t=0; t<num_tracks; t++){
    dist = p.Distance(Tracks_[frame][t]);
    if (dist <= nearest_dist){
      nearest_dist = dist;
      nearest_track_index = t;
    }
  }
  BIASASSERT(nearest_track_index>=0);
  return nearest_track_index;
}


//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class TrackerSynth<unsigned char, float>; 
template class TrackerSynth<float, float>; 
