/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004, 2005    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Image/ImageDraw.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Math/Matrix.hh>
#include <Base/Math/Vector.hh>
#include <Base/Math/Vector2.hh>
#include <MathAlgo/BSplineCurve.hh>
 
using namespace BIAS;
using namespace std;

/**
   Example draws:
   - regular open quadratic bspline into ExampleBSplineCurve0.mip
   - regular closed quadratic bspline into ExampleBSplineCurve1.mip
   - regular open cubic bspline into ExampleBSplineCurve2.mip
   - regular closed cubic bspline into ExampleBSplineCurve3.mip
 */

BIAS::Image<unsigned char> img;
unsigned int imgCounter=0;
unsigned char value[1];


void drawQuadratic(BIAS::BSplineCurve bspline){  
  img.FillImageWithConstValue(value);
  double t=0;
  double d=0.01;//stepsize
  Vector2<double> tmp;//remember last result
  Vector2<double> res;

  bspline.QuadraticUniformBSpline(res,t);//first result
  tmp=res;
  t+=d;
  while(t<1.+d){
    bspline.QuadraticUniformBSpline(res,t);
    ImageDraw<unsigned char>::Line(img,(unsigned int)tmp[0], 
                                   (unsigned int)tmp[1], 
                                   (unsigned int)res[0], 
                                   (unsigned int)res[1],
                                   255);
    tmp=res;
    t+=d;
  }
  char title[255];
  sprintf(title, "ExampleBSplineCurve%01i.mip",imgCounter++);
  BIAS::ImageIO::Write(title, img);
}

void drawCubic(BIAS::BSplineCurve bspline){  
  img.FillImageWithConstValue(value);
  double t=0;
  double d=0.01;//stepsize
  Vector2<double> tmp;//remember last result
  Vector2<double> res;

  bspline.CubicUniformBSpline(res,t);//first result
  tmp=res;
  t+=d;
  while(t<1.+d){
    bspline.CubicUniformBSpline(res,t);
    ImageDraw<unsigned char>::Line(img,(unsigned int)tmp[0], 
                                   (unsigned int)tmp[1], 
                                   (unsigned int)res[0], 
                                   (unsigned int)res[1],
                                   255);
    tmp=res;
    t+=d;
  }  
  char title[255];
  sprintf(title, "ExampleBSplineCurve%01i.mip",imgCounter++);
  BIAS::ImageIO::Write(title, img);
}


//int main(int args, char * argv[])
int main()
{
  //prepage image for dawing
  value[0]=0;
  img.Init(400,400);
  img.FillImageWithConstValue(value);
  
  //bspline constructor
  BSplineCurve bsplineQuadraticOpen,bsplineQuadraticClosed,
    bsplineCubicOpen,bsplineCubicClosed;

  //set controlpoints for simple open bspline (cubic and quadratic)
  std::vector<BIAS::Vector2<double> > cPnt;
  BIAS::Vector2<double> item;
  item[0]=200.;
  item[1]=150.;
  cPnt.push_back(item);
  item[0]=100.;
  item[1]=100.;
  cPnt.push_back(item);
  item[0]=100.;
  item[1]=200.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=300.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=300.;
  cPnt.push_back(item);
  item[0]=300.;
  item[1]=200.;
  cPnt.push_back(item);
  item[0]=300.;
  item[1]=100.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=150.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=200.;
  cPnt.push_back(item);
  bsplineQuadraticOpen.SetControlPoints(cPnt);
  bsplineCubicOpen.SetControlPoints(cPnt);


  //set controlpoints for simple closed bspline
  //cPnt[n],cPnt[0],...,cPnt[n],cPnt[0] //quadratic case
  //cPnt[n],cPnt[0],...,cPnt[n],cPnt[0],cPnt[1] //cubic case
  cPnt.clear();
  item[0]=200.;
  item[1]=200.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=150.;
  cPnt.push_back(item);  
  item[0]=100.;
  item[1]=100.;
  cPnt.push_back(item);
  item[0]=100.;
  item[1]=200.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=300.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=300.;
  cPnt.push_back(item);
  item[0]=300.;
  item[1]=200.;
  cPnt.push_back(item);
  item[0]=300.;
  item[1]=100.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=150.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=200.;
  cPnt.push_back(item);
  item[0]=200.;
  item[1]=150.;
  cPnt.push_back(item);
  bsplineQuadraticClosed.SetControlPoints(cPnt);
  item[0]=100.;
  item[1]=100.;
  cPnt.push_back(item); 
  bsplineCubicClosed.SetControlPoints(cPnt);

  //init bsplines (compute coefficient matrices for segments)
  bsplineQuadraticOpen.InitQuadraticUniformBSpline();
  bsplineQuadraticClosed.InitQuadraticUniformBSpline();
  bsplineCubicOpen.InitCubicUniformBSpline();
  bsplineCubicClosed.InitCubicUniformBSpline();


  //draw open quadratic bspline into ExampleBSplineCurve0.mip
  drawQuadratic(bsplineQuadraticOpen);

  //draw closed quadratic bspline into ExampleBSplineCurve1.mip
  drawQuadratic(bsplineQuadraticClosed);

  //draw open cubic bspline into ExampleBSplineCurve2.mip
  drawCubic(bsplineCubicOpen);

  //draw closed cubic bspline into ExampleBSplineCurve3.mip
  drawCubic(bsplineCubicClosed);
  return 0;
}
