/* This file is part of the BIAS library (Basic ImageAlgorithmS).
   
   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
#ifndef __Histogram1D_hh__
#define __Histogram1D_hh__

#include <Base/Debug/Debug.hh>
#include <Base/Debug/Error.hh>

#include <vector>

namespace BIAS {

  /** @class Histogram1D
      @brief simple one dimensional histogram computation
      @author woelk 01/2008 (c) www.vision-n.de */
  template <class T>
  class Histogram1D
    : public Debug
  {
  public:
    Histogram1D();

    ~Histogram1D();

    inline void Clear() { Data_.clear(); }

    void AddData(const std::vector<T>& data);

    int GetHistogram(std::vector<T>& bin_center, std::vector<unsigned>& hist);
 
    inline void SetHistogramBoundaries(const T &min_val, const T &max_val)
    { BIASASSERT(min_val<max_val); MinValue_=min_val; MaxValue_=max_val; }
    
    inline void SetAutoScale(const bool val)
    { AutoScale_ = val; }

    inline void SetNumBins(const unsigned val)
    { BIASASSERT(val>0); NumBins_ = val; }

  protected:
    std::vector<T> Data_;
    unsigned NumBins_;
    T MinValue_, MaxValue_, BinSize_;
    bool AutoScale_; ///< compute min and max value automatically

    inline int GetBinNum_(const T& data){
      if (data<MinValue_ || data>MaxValue_) return -1;
      return (data-MinValue_)/BinSize_;
    }

    void InitInternalsFromData_();

    void InitInternals_();

    void ComputeBinCenters_(std::vector<T>& center) const;
  };

  // implementation for implicit template instantiation
#include <MathAlgo/Histogram1DImpl.hh>

}


#endif // __Histogram1D_hh__
