/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CScommClasses_
#define _CScommClasses_

#include <bias_config.h>

#ifndef BIAS_HAVE_PTHREADS
#  error BIAS_HAVE_PTHREADS not defined. But BIAS::ImageCanvas needs pthreads (on Unix and Win32). Please enable USE_PTHREADS in CMake configure step.
#endif

namespace BIAS {

  enum EdataType { CS_NONE, CS_STRING, CS_FLOAT, CS_INT, CS_BINARY};

  /** helper classes for CScommBase
  */
  class CScommMsg {
  public:
    CScommMsg() :  newData(false) {}
    std::string msgName;
    std::string commName;
    bool server; // if this msg is accepted by the server or client
    EdataType dataType; // which kind of data (for parsing)
    unsigned int dataSize; // how many FLOATS or INTs...
    bool newData;
    pthread_mutex_t dataLock;

    // only one of the following is used for a msg
    // and the main thread is only allowed to touch one of these
    // with GetData()
    std::vector<float> floatData;
    std::vector<int> intData;
    std::vector<std::string> stringData;
    char* binaryData;
    unsigned int binaryDataSize;
  };

  class CScommData 
  {
  public:
    CScommData() : connected(false),fd(-1),firstcontact(true) 
    {}
    ~CScommData() {};

    std::string name;
    pthread_t threadid;
    bool connected;
    int fd;
    bool firstcontact;
    std::vector<char> msgBuffer; // for buffering of incomplete msgs
    unsigned int lastProcessed;
  };

  // forward declaration
  class CScommBase;

  /** needed for the creation of receivethreads
  */
  class CSCommWithClientNr {
  public:
    CScommBase *comm;
    int commNr;
  };



}

#endif
