#include <Base/Common/BIASpragma.hh>
#include <PanTilt/DPPanTiltControl.hh>
#include <pthread.h>
#include "Control.hh"
#include "SoftMotion.hh"
#include "AutoControl.hh"

using namespace std;
using namespace BIAS;

bool AutoControl::waiting;
bool AutoControl::exiting;
bool AutoControl::started;
bool AutoControl::bSpeedMode_;
long int AutoControl::des_pan;
long int AutoControl::des_tilt;
pthread_mutex_t AutoControl::condition_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t AutoControl::condition_cond = PTHREAD_COND_INITIALIZER;


AutoControl::AutoControl() {
	started = false;
	bSpeedMode_ = false;
}

AutoControl::~AutoControl() {
	if (waiting)
		resumeControl();
  if (started) {
    exiting = true;
    while (started)
      pthread_cond_wait( &condition_cond, &condition_mutex );
  }
  ptControl.ClosePanTiltUnit();
}

void AutoControl::startControl() {
	if (started)
		return;
	started = true;
	waiting = false;
	exiting = false;
	ptControl.GetCurrentPosition(cur_pan, cur_tilt);
	cout << "current pos: " << cur_pan << " " << cur_tilt << endl;
	ptControl.SetMode(SPEEDMODE);
	await_completion();
	ptControl.SetSpeed(0,0);
	await_completion();
	ptControl.GetCurrentPosition(cur_pan, cur_tilt);
	bSpeedMode_ = true;
	cout << "current pos: " << cur_pan << " " << cur_tilt << endl;
	pthread_create(&control_thread, NULL, this->threadFunc_pan_tilt, &ptControl); 
}

void AutoControl::stopControl() {
	pthread_mutex_lock( &condition_mutex );
	waiting = true;
	ptControl.SetSpeed(0,0);
	pthread_mutex_unlock( &condition_mutex );
}

void AutoControl::resumeControl() {
	pthread_mutex_lock( &condition_mutex );
	waiting = false;
	pthread_cond_signal( &condition_cond );
	pthread_mutex_unlock( &condition_mutex );
}

void AutoControl::startMoveTo(const long int pan, const long int tilt) {
  if (!bSpeedMode_) {
    ptControl.GetCurrentPosition(cur_pan, cur_tilt);
    ptControl.SetMode(SPEEDMODE);
    ptControl.SetSpeed(0,0);
    ptControl.GetCurrentPosition(cur_pan, cur_tilt);
    bSpeedMode_ = true;
  }
  // if moving new command will be skipped, use stopMoveTo() to stop first
  if (started)
    return;
  started = true;
  des_pan = pan;
  des_tilt = tilt;
  pthread_create(&control_thread, NULL, this->threadFunc_move_to, &ptControl);
}

void AutoControl::stopMoveTo() {
  pthread_mutex_lock( &condition_mutex );
  if (started) {
    exiting = true;
    while (started)
      pthread_cond_wait( &condition_cond, &condition_mutex );
  }
  pthread_mutex_unlock( &condition_mutex );
}

void* AutoControl::threadFunc_pan_tilt(void *ptr) {
	DPPanTiltControl *ptc = (DPPanTiltControl*)ptr;
	t_SoftMotion sm;
	float speed_damp = 32.0f; 
  //	sm.setLimits(-2000,2000,100,MAXSPEED/speed_damp,50,MAXSPEED/speed_damp);
	int k=0;
	int pspeed = 0;
	int tspeed = 0;
	int start_pan=0, start_tilt=0, end_pan=0, end_tilt=0, tilt_in_start=0, tilt_in_end=0;
	bool loop = true;
	while (loop) {
		ptc->GetCurrentPosition(cur_pan, cur_tilt);
		pthread_mutex_lock( &condition_mutex );
		while (waiting && !exiting)
			pthread_cond_wait( &condition_cond, &condition_mutex );
		pthread_mutex_unlock( &condition_mutex );

		if (exiting) {
			loop = false;
			ptc->SetSpeed(0,0);
			await_completion();
			started = false;
	    pthread_cond_signal( &condition_cond );
			break;
		}

		int pmod = 1;
    tilt_in_start = -600; tilt_in_end = -1000;
		if (k%2==0) {
			pmod = -1;
      tilt_in_start = 600; tilt_in_end = 1000;
		} 
		start_pan = -1*pmod*1000; end_pan = pmod*1000;
		switch (k) {
			case 0: 
				start_pan = 0; end_pan = -1000;
				start_tilt = 0; end_tilt = -400;
				tilt_in_start = 0; tilt_in_end = -1000;
			break;
			case 2:
				start_tilt = -400; end_tilt = -200;
			break;
      case 3:
        start_tilt = -200; end_tilt = 0;
      break;
      case 4:
        start_tilt = 0; end_tilt = 200;
      break;
      case 5:
        start_tilt = 200; end_tilt = 400;
      break;
      case 6:
        start_tilt = 400; end_tilt = 600;
      break;
      case 7:
        start_tilt = 600; end_tilt = 800;
      break;
			case 8:
				//ptc->SetMode(ABSOLUTEMODE);
				//ptc->SetPosition(0,0);
				//k = 0;
				exiting = true;
				continue;
			break;
		}

		pspeed = sm.getPanSpeed(start_pan,end_pan,cur_pan,speed_damp);	
		tspeed = sm.getTiltSpeed(tilt_in_start,tilt_in_end,start_tilt,end_tilt,cur_pan,cur_tilt,speed_damp);
		ptc->GetCurrentPosition(cur_pan, cur_tilt);
		pthread_mutex_lock( &condition_mutex );
		if (!waiting) {
				ptc->SetSpeed(pspeed,tspeed);
		}
		pthread_mutex_unlock( &condition_mutex );
		if (pmod < 0 && cur_pan <= end_pan  || pmod > 0 && cur_pan >= end_pan) {
	    k++;
		}
		ptc->GetCurrentPosition(cur_pan, cur_tilt);
	}
	return 0;
}

void* AutoControl::threadFunc_pan_only(void *ptr) {
	DPPanTiltControl *ptc = (DPPanTiltControl*)ptr;
	int min_pan = -2000;
	int max_pan = 2000;
	int pmod = 1;
	
	ptc->GetCurrentPosition(cur_pan, cur_tilt);
	while (1) {
		pthread_mutex_lock( &condition_mutex );
		while (waiting && !exiting)
			pthread_cond_wait( &condition_cond, &condition_mutex );
		pthread_mutex_unlock( &condition_mutex );

		if (exiting) {
			ptc->SetSpeed(0,0);
			await_completion();
			break;
		}
		if (cur_pan >=max_pan) 
			pmod = -1;
		if (cur_pan <=min_pan)
			pmod = 1;
		
		int pspeed = 0;
		float speedmod = (1+cos((float)cur_pan*M_PI / 2000.0f)) / 2.0f;
		pspeed = abs(MAXSPEED*speedmod) / 4;		
		pspeed = max(100,pspeed);
		pspeed = min(MAXSPEED/2,pspeed);
		pspeed *= pmod;
		//pspeed = pmod*(MAXSPEED/4);
		pthread_mutex_lock( &condition_mutex );
		if (!waiting)
			ptc->SetSpeed(pspeed, 0);
		pthread_mutex_unlock( &condition_mutex );
		ptc->GetCurrentPosition(cur_pan, cur_tilt);
		cout << cur_pan << " " << cur_tilt << endl;
	}
	return 0;
}

void* AutoControl::threadFunc_move_to(void *ptr) {
  exiting = false;
  DPPanTiltControl *ptc = (DPPanTiltControl*)ptr;
  t_SoftMotion sm;
  //  sm.setLimits(-2000,2000,100,MAXSPEED/2,50,MAXSPEED/2);
  long int start_pan, start_tilt;
  int pspeed = 0, tspeed = 0;
  ptc->GetCurrentPosition(start_pan, start_tilt);
  while (1) {
    ptc->GetCurrentPosition(cur_pan, cur_tilt);
    if (abs(des_pan - cur_pan) < 10 && abs(des_tilt - cur_tilt) < 10)
      exiting = true;
    if (exiting) {
      ptc->SetSpeed(0,0);
      await_completion();
      started = false;
      pthread_cond_signal( &condition_cond );
      break;
    }
    pspeed = sm.getPanSpeed(start_pan, des_pan, cur_pan, 4);
    tspeed = sm.getTiltSpeed(0, 0, start_tilt, des_tilt, cur_pan, cur_tilt, 8);
    pthread_mutex_lock( &condition_mutex );
    if (!waiting)
      ptc->SetSpeed(pspeed,tspeed);
    pthread_mutex_unlock( &condition_mutex );
    if (pspeed == 0 && tspeed == 0)
      exiting = true;
    ptc->GetCurrentPosition(cur_pan, cur_tilt);
  }
  return 0;
}

int AutoControl::abs(int val) {
	return (val < 0) ? -val : val;
}

int AutoControl::min(int val1, int val2) {
	return (val1 < val2) ? val1 : val2;
}

int AutoControl::max(int val1, int val2) {
	return (val1 < val2) ? val2 : val1;
}

