#ifndef AUTOCONTROL_HH_
#define AUTOCONTROL_HH_

#include <pthread.h>
#include "Control.hh"

namespace BIAS{

class AutoControl : public Control {
public:
	// constructor has nothing to do, super-constructor will startup communication 
	AutoControl();
	// destructor moves to (0,0), then calls super-destructor to release ttyS0
	~AutoControl();
	// start moving around
	void startControl();
	// stop/pause moving around
	void stopControl();
	// resume moving around
	void resumeControl();
	
	// move to desired position 
	void startMoveTo(const long int pan, const long int tilt);
	// stop move to desired position
	void stopMoveTo();
	// check if position is reached
	inline bool stopped() {
	  return !started;
	}
	
private:
	// thread to control camera movement
	pthread_t control_thread;
	// actual movement function, ptr should hold a pointer to this classes ptu-object
	static void* threadFunc_pan_only(void *ptr);
	// actual movement function, ptr should hold a pointer to this classes ptu-object
	static void* threadFunc_pan_tilt(void *ptr);
	
	// actual movement to a desired position, ptr should hold a pointer to this classes ptu-object
	static void* threadFunc_move_to(void *ptr);
	
	// conditions for the thread
	static pthread_mutex_t condition_mutex;
	static pthread_cond_t  condition_cond;
	
	// thread control variables
	static bool exiting;
	static bool waiting;
	static bool started;
  static bool bSpeedMode_;

	// some useful helper functions, should be in some more general class
	static int abs(int val);
	static int min(int val1, int val2);
	static int max(int val1, int val2);
	
	static long int des_pan;
	static long int des_tilt;

};

class AutoControlFactory : public ControlFactory {
public:
	AutoControl* create() {
		return new AutoControl();
	}
};
}
#endif /*AUTOCONTROL_HH_*/
