/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Math/Random.hh>
#include <StateEstimator/ExtendedKalman.hh>

#ifdef WIN32
  #include <Base/Common/W32Compat.hh>
#else //Linux
  #include <sys/time.h>
#endif

namespace BIAS {

  class MyKalman : public ExtendedKalman {
    
  public:
    MyKalman(){};
    virtual ~MyKalman(){};
    
    virtual void MeasurementFunction(const Vector<double>& state,
                                     Vector<double>& measurePred)
    {
      measurePred = state;
    }
    
    virtual void StateTransition(const Vector<double>& oldState,
                                 const Vector<double>& control,
                                 Vector<double>& newState)
    {
      newState = oldState;
    }
    
  };
  
}

using namespace std;
using namespace BIAS;

int main(int argc, char ** argv )
{
  if (argc < 2) {
    cout << "Give number of iterations!" << endl;
    exit(0);
  }

  //'real' state to estimate
  double mean1 = 20.0;
  double mean2 = 30.5;

  MyKalman filter;

  int maxIter = atoi(argv[1]);

  Vector<double> state(2);
  state[0] = 0.0;
  state[1] = 0.0;
  Matrix<double> zero(2,2);
  zero.SetZero();
  Matrix<double> cov(2,2);
  cov.SetZero();
  cov[0][0] = 4.0;
  cov[1][1] = 16.0;
  Matrix<double> id(2,2);
  id.SetIdentity();

  filter.SetInitial(state, 100.0*cov);
  filter.SetMeasureDerive(id, id);
  filter.SetStateDerive(id, zero);
  filter.SetMeasureCov(cov);
  filter.SetProcessCov(zero);

  timeval t;
  gettimeofday(&t,NULL);
  long time = t.tv_usec;
  Random random(time);

  double sum1 = 0, sum2 = 0;

  for (int i=0; i<maxIter; i++) {
    state[0] = 
      random.GetNormalDistributed(mean1, 4.0);
    sum1 += state[0];
    state[1] = 
      random.GetNormalDistributed(mean2, 16.0);
    sum2 += state[1];
    filter.Predict(state);
    filter.Update(state);
    filter.GetState(state);
  }

  cout << "should be    1: " << mean1 << "       2: " << mean2 
       << endl << flush;
  cout << "kalman state 1: " << state[0] << "       2: " << state[1] 
       << endl << flush;
  cout << "avarage:     1: " << sum1/double(maxIter) << "       2: " 
       << sum2/double(maxIter) << endl << flush;

  return 0;
}
