#include "Base/Common/W32Compat.hh"
#include "ShowCamWxFrame.hh"
#include "ShowCamWxThreads.hh"

using namespace BIAS;
using namespace std;

#define PARAM_FILE_NAME "ShowCamWx.param"
#define BUFFER_SIZE 50

BEGIN_EVENT_TABLE(ShowCamWxFrame, wxFrame)
  EVT_MENU      (ID_Quit,               ShowCamWxFrame::OnQuit)
  EVT_MENU      (ID_Grab,               ShowCamWxFrame::OnGrab)
  EVT_MENU      (ID_GrabContinuous,     ShowCamWxFrame::OnGrabContinuous) 
  EVT_MENU      (ID_Show1,              ShowCamWxFrame::OnShowImage1) 
  EVT_MENU      (ID_Show2,              ShowCamWxFrame::OnShowImage2) 
  EVT_MENU      (ID_Show3,              ShowCamWxFrame::OnShowImage3) 
  EVT_MENU      (ID_Show4,              ShowCamWxFrame::OnShowImage4) 
  EVT_MENU      (ID_Show5,              ShowCamWxFrame::OnShowImage5) 
  EVT_MENU      (ID_Show6,              ShowCamWxFrame::OnShowImage6) 
  EVT_MENU      (ID_Show7,              ShowCamWxFrame::OnShowImage7) 
  EVT_MENU      (ID_Show8,              ShowCamWxFrame::OnShowImage8)
  EVT_MENU      (ID_Show9,              ShowCamWxFrame::OnShowImage9)   
  EVT_MENU      (ID_Controller,         ShowCamWxFrame::OnShowController)
  EVT_MENU      (ID_SaveParameters,     ShowCamWxFrame::OnSaveParameters)
  EVT_MENU      (ID_ShowParameters,     ShowCamWxFrame::OnShowParameters)
  EVT_TIMER     (ID_Timer,              ShowCamWxFrame::OnTimer)
  
  EVT_KEY_DOWN  (ShowCamWxFrame::OnKey) 
  END_EVENT_TABLE()


/****************************************************
   @brief


*****************************************************/
ShowCamWxFrame::
ShowCamWxFrame(const wxString& title,
                               const wxPoint& pos, const wxSize& size)
  : wxFrame(NULL, -1, title, pos, size) ,
    ParamGUI ()   // disable param desctructor warning
    
{
  // set default value for class variables
  bSingleSaveDirCreated_ = false; 
  dSingleDirIndex_ = 0;
  dDirIndex_=0;
  paramFrame_ =NULL;
  OpenedDevices_    = false;
  InitCameras_ = false;
  InitParams_  = false;
  bSaveImages_ = false;
  bWriteBuffered_ = false;
  EmptyDevice_ = false;
  FrameCounter_ = 0;
  GrabCounter_  = 0;
  LastWidth_    = 0;
  LastHeight_   = 0;
  NumCamSel_    = 0;
  Win32DCAMNumber_ = 0;
  ShowXB3_ = 0;
#ifdef BIAS_HAVE_PanTilt
  panTiltControl_=NULL;
  panTiltControlDialog_=NULL;
#endif //BIAS_HAVE_PanTilt

  // set timer delay to 5 ms
  Delay_ = 5;

  // init images for cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++)
  {
    // init Images
    CamImg_.push_back(new Camera<unsigned char>);
    CamImgMiddle_.push_back(new Camera<unsigned char>);
    CamImgRight_.push_back(new Camera<unsigned char>);
    XB3AllInOne_.push_back(new Camera<unsigned char>);
    
    // init Thread variables
    Mutexes_.push_back(new wxMutex);
    Conditions_.push_back(new wxCondition(*Mutexes_[i]));
    Mutexes_[i]->Lock();
    
    SaveMutexes_.push_back(new wxMutex);
    SaveConditions_.push_back(new wxCondition(*SaveMutexes_[i]));
    SaveMutexes_[i]->Lock();

    ThreadInited_.push_back(false);
 
    grabSingleThread_.push_back(NULL);
    saveThreads_.push_back(NULL);
    saveThreadsPMDDepth_.push_back(NULL);
    saveThreadsPMDAmp_.push_back(NULL);
    
    // Ringbuffer for writing images
    BufferAccess_.push_back(new wxMutex);
    WriteIndex_.push_back(0);
    ReadIndex_.push_back(BUFFER_SIZE-1);

    ImageBuffer_.push_back(new vector<Camera<unsigned char>*>);
    ImageBufferMiddle_.push_back(new vector<Camera<unsigned char>*>);
    ImageBufferRight_.push_back(new vector<Camera<unsigned char>*>);
    ImageBufferPMDDepth_.push_back(new vector<Camera<float>*>);
    ImageBufferPMDAmp_.push_back(new vector<Camera<float>*>);

    for(int j = 0; j < BUFFER_SIZE;  j++){
      ImageBuffer_[i]->push_back(NULL);
      ImageBufferMiddle_[i]->push_back(NULL);
      ImageBufferRight_[i]->push_back(NULL);
      ImageBufferPMDDepth_[i]->push_back(NULL);
      ImageBufferPMDAmp_[i]->push_back(NULL);
    }
    

    CamImg2D_.push_back(new Camera<unsigned char>);
    CamImgPMDDepth_.push_back(new Camera<float>);
    CamImgPMDAmp_.push_back(new Camera<float>);
    ColorModels_.push_back(ImageBase::CM_invalid);
    VideoControllers_.push_back(NULL);
#ifdef BIAS_HAVE_DCAM
    VideoControllersXB3_.push_back(NULL);
#endif

#ifdef BIAS_HAVE_PMD_PMDZess       
    VideoControllersUSBPMD_.push_back(NULL);
#endif //BIAS_HAVE_PMD_PMDZess
#ifdef BIAS_HAVE_SWISSRANGER
    VideoControllersSwissRanger_.push_back(NULL);
#endif // BIAS_HAVE_SWISSRANGER

  }

  // create status bar and add status text
  CreateStatusBar(3);
  wxString text;
  text.Printf(wxT("size: %4dx%4d"),0, 0);
  SetStatusText(text, 1);
  
  // init image canvas
  IC_ = new ImageCanvas(this,GetStatusBar(), 0,-1,wxDefaultPosition,                          
                        wxSize(640,480));
  
  // add image canvas to main window
  wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
  vbox->Add(IC_,1,  wxEXPAND |wxALL |wxFIXED_MINSIZE , 5);
  //vbox->SetMinSize(640,480);
  SetSizer(vbox);
  SetAutoLayout(true);
  Layout();
  
}

ShowCamWxFrame::
~ShowCamWxFrame()
{
  // close threads
  for(int i = 0; i < NUMBER_CAMERAS; i++){
    if(grabSingleThread_[i] != NULL){
      delete grabSingleThread_[i];
    }  
    if(saveThreads_[i] != NULL){
      delete saveThreads_[i];
    }
  }
}
/****************************************************
   @brief


*****************************************************/

void ShowCamWxFrame::
InitParams(int &argc, char *argv[])
{
  // initialize camera names for params
  vector<string> cameras;
  cameras.push_back("NONE");
  cameras.push_back("DCAM");
  cameras.push_back("PMD");
  cameras.push_back("PMDUSB");
  cameras.push_back("PMDSHM");  
  cameras.push_back("UEYE");
  cameras.push_back("SHM");
  cameras.push_back("NET");
  cameras.push_back("V4L");
  cameras.push_back("SR");
  cameras.push_back("XB3");

  // initialize color models for cameras
  vector<string> colorModels;
  colorModels.push_back("CM_Default");  
  colorModels.push_back("CM_Grey");        
  colorModels.push_back("CM_RGB");        
  colorModels.push_back("CM_BGR");        
  colorModels.push_back("CM_YUYV422");   
  colorModels.push_back("CM_UYVY422");   
  colorModels.push_back("CM_YUV420P");     
  colorModels.push_back("CM_YUV444");     
  colorModels.push_back("CM_YUV411");     
  colorModels.push_back("CM_HSV");        
  colorModels.push_back("CM_HSI_OBS");  
  colorModels.push_back("CM_DV");         
  colorModels.push_back("CM_RGBA");      
  colorModels.push_back("CM_GreyA");      
  colorModels.push_back("CM_Bayer_RGGB");  
  colorModels.push_back("CM_Bayer_GBRG");  
  colorModels.push_back("CM_Bayer_GRBG");  
  colorModels.push_back("CM_Bayer_BGGR");  
  colorModels.push_back("CM_HSL");
  colorModels.push_back("CM_hsL");     
  colorModels.push_back("CM_SymTensor2x2");
  colorModels.push_back("CM_BGRA");        
  colorModels.push_back("CM_RGBE");         

	

  for (int i = 0; i < NUMBER_CAMERAS; i++)
  {
    // add nine different cameras to params and set group name
    stringstream number;
    number << i + 1;
    ParamCameras_[i] = AddParamEnum("CameraType" + number.str(),
                                    "Type of camera", cameras, 0, NULL, 0, i);
    ParamColorModel_[i] = AddParamEnum("CameraColorModel" + number.str(),
                                       "Color Model for camera", colorModels,
                                       0, NULL, 0, i);     
    ParamFPS_[i]    = AddParamDouble("CameraFPS" + number.str(),
                                     "fps possibilities: 1.875 3.75 7.5 15.0 30.0 60.0 120.0 240.0",
                                     0.0, 0.0, 240.0, 0, i);
    ParamWidth_[i]  = AddParamInt("CameraWidth" + number.str(),
                                  "Width of camera", 0, 0, 10000, 0, i);
    ParamHeight_[i] = AddParamInt("CameraHeight" + number.str(),
                                  "Height of camera", 0, 0, 10000, 0, i);
    //ParamSetTop_[i]  = AddParamInt(); 
    //ParamSetLeft_[i] = AddParamInt();
    ParamDown_[i]   = AddParamInt("CameraDownsampling" + number.str(),
                                  "Downsampling of camera", 0, 0, 10, 0, i);
    ParamPort_[i]   = AddParamInt("CameraPort" + number.str(),
                                  "Port of NET camera", 0, 0, 100000, 0, i);
    ParamIP_[i]     = AddParamString("CameraIP" + number.str(),
                                     "IP of NET camera", "", 0, i);
    ParamBitFile_[i]= AddParamString("CameraBitFile" + number.str(),
                                     "Bit file for usb pmd cameras", "", 0, i);
    ParamDevice_[i] = AddParamString("CameraDevice" + number.str(),
                                     "Device/Interface name of camera",
                                     "", 0, i);    
    // every camera gets own group number
    stringstream text;
    text<<number.str()<<". Camera";
    SetGroupName(i, text.str()); 
  }
#ifdef BIAS_HAVE_PanTilt
  bUsePanTiltUnit_ = AddParamBool("UsePanTiltUnit","Use the Pan-Tilt-Unit?",false,
                                  'p', NUMBER_CAMERAS);
  stringstream text;
  text<<"General";
  SetGroupName(NUMBER_CAMERAS, text.str()); 
#endif
  // check if param file exists, else create one
  ifstream ifs(PARAM_FILE_NAME);
  if (!ifs)  
    WriteParameter(PARAM_FILE_NAME);
  else
  {
    // read param from file
    ReadParameter(PARAM_FILE_NAME); 
    // update parameters from file with arguments from command line
    UpdateParameter(argc, argv, PARAM_FILE_NAME);
    ifs.close();
  }

  // update parameters from file with arguments from command line
 //  if (argc > 1)
//     UpdateParameter(argc, argv, PARAM_FILE_NAME);

  // get color models for cameras
  ParseColorModels();
  
  InitParams_ = true;
}

#ifdef BIAS_HAVE_PanTilt
/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
InitPanTiltUnit(){
  if(*bUsePanTiltUnit_){
    panTiltControl_= new DPPanTiltControl();
    bool bRet = false;
#ifdef WIN32	  
    bRet = panTiltControl_->InitPanTiltUnit("COM1");
#else
    bRet = panTiltControl_->InitPanTiltUnit("/dev/ttyS0");
#endif
    panTiltControlDialog_ = new DPPanTiltControlDialog(this, 7651,
                                  wxT("Pan-Tilt-Unit Control"));
    panTiltControlDialog_->SetDPControl(panTiltControl_);
    panTiltControlDialog_->Show(true);
  }
}
#endif //BIAS_HAVE_PanTilt

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
InitCameras(){
 
  if (!InitParams_){
      BIASERR("Initialize params first");
      BIASABORT;
  }

  bool foundCamera = false; 
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    if (*ParamCameras_[i] != 0)        
      foundCamera = true;        
  }
  if (!foundCamera) { 
    wxMessageDialog errorDialog(this,
                                wxT("No Camera specified in param file!"),
                                wxT("ShowCamWx - Error"),
                                wxICON_ERROR|wxOK);
    errorDialog.ShowModal();   
  }

  //error string
  stringstream errorText; 
  bool bCameraError = false;
  // create cameras according to param file
  for (int i = 0; i < NUMBER_CAMERAS; i++) {   
    switch(*ParamCameras_[i]){
      case 0: // None camera
        Cameras_.push_back(NULL);
        break;
      case 1: // DCAM
        
#ifdef BIAS_HAVE_DCAM
        Cameras_.push_back(new VideoSource_DCAM);      
        // Cameras_[i]->SetSize(960,768,2);        
        Cameras_[i]->SetDebugLevel(D_DCAM);
#else        
        errorText << "DCAM Camera in param file, please enable support in BIAS\n";
        bCameraError = true;
        Cameras_.push_back(NULL);
        BIASERR("DCAM Camera in param file, please enable support in BIAS");     
#endif
      
        break;
      case 2: // PMD
        
#ifdef BIAS_HAVE_PMD_PMDTec     
        Cameras_.push_back(new VideoSource_DcamPMD);         
#else   
        bCameraError = true;
        errorText <<"PMDTec Camera in param file, please enable support in BIAS\n";     
       Cameras_.push_back(NULL);
        BIASERR("PMDTec Camera in param file, please enable support in BIAS");
#endif
      
        break;
      case 3: // PMDUSB
      
#ifdef BIAS_HAVE_PMD_PMDZess
        Cameras_.push_back(new VideoSource_usbPMD);  
        if (*ParamBitFile_[i] != "")
          dynamic_cast<VideoSource_usbPMD*>(Cameras_[i])->SetBitFile(*ParamBitFile_[i]);
        else
          {
            BIASERR("Specify bit file for usb pmd camera");  
            bCameraError = true;
            errorText << "Specify bit file for ZESS PMD camera!\n";
          }
#else
        bCameraError = true;
       Cameras_.push_back(NULL);
        errorText << "Zess PMD Camera in param file, please enable support in BIAS\n";
        BIASERR("Zess PMD Camera in param file, please enable support in BIAS");    
#endif
      
        break;
      case 4: // PMDSHM
      
#ifdef BIAS_HAVE_TIFF 
        Cameras_.push_back(new VideoSource_ShmPMD);               
#else      
        bCameraError = true;
        Cameras_.push_back(NULL);
        errorText << "SHM PMD Camera in param file, please enable support in BIAS\n";            
        BIASERR("SHM PMD Camera in param file, please enable support in BIAS");  
#endif
      
        break;      
      case 5: // UEYE
      
#ifdef BIAS_HAVE_UEYE
        Cameras_.push_back(new VideoSource_uEye);
        dynamic_cast<VideoSource_uEye*>(Cameras_[i])->SetLeftTop(56,0);
        dynamic_cast<VideoSource_uEye*>(Cameras_[i])->SetShutter(0.01f);
        dynamic_cast<VideoSource_uEye*>(Cameras_[i])->SetGain(0);     
#else        
        bCameraError = true;
      Cameras_.push_back(NULL);
        errorText << "UEYE Camera in param file, please enable support in BIAS\n";        
        BIASERR("UEYE Camera in param file, please enable support in BIAS");   
#endif
      
        break;
      case 6: // SHM
        Cameras_.push_back(new VideoSource_Shm);  
        break;
      case 7: // NET 
        Cameras_.push_back(new VideoSource_Net);
        break;
      case 8: // V4L

        // TODO: test support for v4l devices
        BIASERR("V4L support not yet implemented!");
      
#ifdef BIAS_HAVE_V4L      
        Cameras_.push_back(new VideoSource_V4L);
#else
        bCameraError = true;
        Cameras_.push_back(NULL);
        errorText << "V4L Camera in param file, please enable support in BIAS\n";       
        BIASERR("V4L Camera in param file, please enable support in BIAS");
#endif
      
        break;      
      case 9: // SR
#ifdef BIAS_HAVE_SWISSRANGER
        Cameras_.push_back(new VideoSource_SwissRanger);
#else
        bCameraError = true;
        Cameras_.push_back(NULL);
        errorText << "SwissRanger Camera in param file, please enable support in BIAS\n";        
        BIASERR("SwissRanger in param file, please enable support in BIAS");
#endif
        break;
      case 10: // XB3
//# ifndef WIN32
#ifdef BIAS_HAVE_DCAM
        Cameras_.push_back(new VideoSource_DCAM_XB3);         
        Cameras_[i]->SetDebugLevel(D_DCAM|D_DCAM_F7); 
        
        Cameras_[i]->SetColorModel(ImageBase::CM_PGR_XB3_F7M3_GBRG);
        ColorModels_[i] = ImageBase::CM_PGR_XB3_F7M3_GBRG;
             
        if( (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])) != NULL){
          dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->UsePort(0);
          dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->SetFormat7(3,4080,3);
          dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->SetLeftTop(0,0);
          dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->SetSize(1280,960,3);       
        }
#else
        bCameraError = true;
        Cameras_.push_back(NULL);
        errorText << "DCAM (Bumblebee) Camera in param file, please enable support in BIAS\n";        
        BIASERR("DCAM (Bumblebee) in param file, please enable support in BIAS");
#endif
//# else //WIN32
//        BIASERR("Bumblebee not supported within WIN32 right now!");
//        BIASABORT;
//# endif //WIN32
        break;
      default: // None
        Cameras_.push_back(NULL);      
        break;      
      }      
  }
  if(bCameraError){
    wxMessageDialog errorDialog1(this,
                               AsciiToWx(errorText.str().c_str()),
                               wxT("ShowCamWx - Error"),
                               wxICON_ERROR|wxOK);
    errorDialog1.ShowModal(); 
  }

  // set size of cameras if size and type are defined
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    if (*ParamHeight_[i] != 0 && *ParamWidth_[i] != 0 && Cameras_[i] != NULL)
      {
        Cameras_[i]->SetSize(*ParamWidth_[i], *ParamHeight_[i]); 
        //  cout<<"Setting size to :"<<*ParamWidth_[i]<<", "<< *ParamHeight_[i]<<endl;
      }
    if (*ParamFPS_[i] != 0.0 && Cameras_[i] != NULL)
      {
        //  cout<<"Setting "<<i<<". camera fps to "<<*ParamFPS_[i]<<endl;
        Cameras_[i]->SetFPS((float)*ParamFPS_[i]);
        }
  }  
  InitCameras_ = true;
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OpenDevices()
{
  if (!InitCameras_) {
    BIASERR("Initialize params first");
    return;
  }
#ifndef WIN32
#ifdef BIAS_HAVE_DCAM
  stringstream cameras;                            
  for(int k=0;k<10;k++)
    { 
      cout<<"Scanning bus:"<<k<<endl;
      ScanBus(k,cameras);
    }
  ofstream fout( "scanbus.info" );
  fout<<cameras.str()<<endl;
  fout.close();
#endif
#endif
  
  for (int i = 0; i < NUMBER_CAMERAS; i++)
    {
      switch(*ParamCameras_[i])
        {
      
#ifdef BIAS_HAVE_DCAM 
        case 10: // XB3    
        case 1: // DCAM
          if (*ParamDevice_[i] != "")  { 
            Cameras_[i]->AddDebugLevel(D_DCAM);
            if (Cameras_[i]->OpenDevice(atoi((*ParamDevice_[i]).c_str())) < 0)
              { 
                Cameras_[i]->CloseDevice();
                delete Cameras_[i];
                Cameras_[i] = NULL;  
                cout<<"Card 0 failed, trying card 1:"<<endl;  
                Cameras_[i] = new VideoSource_DCAM(1);
                Cameras_[i]->AddDebugLevel(D_DCAM);
             
                if (Cameras_[i]->OpenDevice(atoi((*ParamDevice_[i]).c_str())) < 0){
                  Cameras_[i]->CloseDevice();
                  delete Cameras_[i];
                  Cameras_[i] = NULL;  
                  stringstream text;
                  text << "DCAM Camera number " << i+1 <<" "<<(*ParamDevice_[i]).c_str() 
                       <<" on Card 1 could not be openend";
                  wxMessageDialog errorDialog(this,
                                              AsciiToWx(text.str().c_str()),
                                              wxT("ShowCamWx - Error"),
                                              wxICON_ERROR|wxOK);
                  errorDialog.ShowModal();  
                }
                else{
                  cout<<"camera number " << i+1 << " "<<atoi((*ParamDevice_[i]).c_str())
                      <<" successfully opened on card 1"<<endl;
                }
              
              } 
            else
              cout<<"Camera number " << i+1 << " successfully opened"<<endl;
          }
          else  {  
#ifdef WIN32
            // this does not work if not all cameras are dcam!
            if (Cameras_[i]->OpenDevice(Win32DCAMNumber_++) < 0)
              {  
                Cameras_[i]->CloseDevice(); 
                delete Cameras_[i];
                Cameras_[i] = NULL;    
                stringstream text;
                text << "Camera number " << i << " could not be openend";
                wxMessageDialog errorDialog(this,
                                            AsciiToWx(text.str().c_str()),
                                            wxT("ShowCamWx - Error"),
                                            wxICON_ERROR|wxOK);
                errorDialog.ShowModal();                                   
              }
#else
            // check if there are multiple cameras with no device name
                                    
            if (EmptyDevice_) {                
              stringstream text;
              text << "Found more than one camera with no device name!\n"
                   <<"Available cameras:\n"<<cameras.str().c_str();
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();               
            }
            // going here the first time sets on empty device
            EmptyDevice_ = true;
            if (Cameras_[i]->OpenDevice() < 0) { 
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Test Camera number " << i+1 << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();                  
            }
#endif // WIN32
          }   
          break;
#endif // BIAS_HAVE_DCAM
      
#ifdef BIAS_HAVE_PMD_PMDTec  
        case 2: // PMD
          ((VideoSource_DcamPMD*)Cameras_[i])->SetIntegrationTime(20000);
          if (Cameras_[i]->OpenDevice()<0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Camera PMDTec " << i << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();               
            }
#endif
#ifdef BIAS_HAVE_PMD_PMDZess 
        case 3: // PMDUSB
          if (Cameras_[i]->OpenDevice()<0)
            {   
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Camera PMD ZESS " << i << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();  
             
            }              
          break;
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
        case 4: // PMDSHM
          if ((Cameras_[i])->OpenDevice((*ParamDevice_[i]).c_str()) < 0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Camera PMD Shared Memory " << i << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
            }
          break;
#endif // BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_UEYE      
        case 5: // UEYE
#endif // BIAS_HAVE_UEYE
      
        case 6: // SHM
      
#ifdef BIAS_HAVE_V4L            
        case 8: // V4L
#endif // BIAS_HAVE_V4L
          if (*ParamDevice_[i] != "")
            {    
              if (Cameras_[i]->OpenDevice((*ParamDevice_[i]).c_str()) < 0)
                {
                  delete Cameras_[i];
                  Cameras_[i] = NULL;
                  stringstream text;
                  text << "Camera " << i << " could not be openend";
                  wxMessageDialog errorDialog(this,
                                              AsciiToWx(text.str().c_str()),
                                              wxT("ShowCamWx - Error"),
                                              wxICON_ERROR|wxOK);
                  errorDialog.ShowModal();
                }
            }
          else
            {
              if (Cameras_[i]->OpenDevice() < 0)
                {
                  delete Cameras_[i];
                  Cameras_[i] = NULL;
                  stringstream text;
                  text << "Camera " << i << " could not be openend";
                  wxMessageDialog errorDialog(this,
                                              AsciiToWx(text.str().c_str()),
                                              wxT("ShowCamWx - Error"),
                                              wxICON_ERROR|wxOK);
                  errorDialog.ShowModal();
                }
            }
          break;

        case 7: // NET
          // open on default port specified by class
          if (*ParamPort_[i] == 0)
            {
              if (((VideoSource_Net*)Cameras_[i])->OpenDevice(*ParamIP_[i]) < 0)
                {
                  delete Cameras_[i];
                  Cameras_[i] = NULL;
                  stringstream text;
                  text << "Net Camera " << i << " could not be openend";
                  wxMessageDialog errorDialog(this,
                                              AsciiToWx(text.str().c_str()),
                                              wxT("ShowCamWx - Error"),
                                              wxICON_ERROR|wxOK);
                  errorDialog.ShowModal();
                }        
            }
          // open on defined port by param file
          else
            {
              if (((VideoSource_Net*)Cameras_[i])->OpenDevice(*ParamIP_[i],
                                                              *ParamPort_[i]) < 0)
                {
                  delete Cameras_[i];
                  Cameras_[i] = NULL;
                  stringstream text;
                  text << "Net Camera on:"<<*ParamIP_[i]<<" Port:"<<*ParamPort_[i]
                       <<" could not be openend";
                  wxMessageDialog errorDialog(this,
                                              AsciiToWx(text.str().c_str()),
                                              wxT("ShowCamWx - Error"),
                                              wxICON_ERROR|wxOK);
                  errorDialog.ShowModal();
                }
            }
          break;

#ifdef BIAS_HAVE_SWISSRANGER
        case 9: // SwissRanger
          if (Cameras_[i]->OpenDevice()<0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Swissranger PMD-Camera could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
            }    
          break;
#endif

        default: // NONE
          break;
        }

      // init image for camera camera if opened
      if (Cameras_[i] != NULL)
        {
          // try to set color model from param file for camera
          if (ColorModels_[i] != ImageBase::CM_invalid)
            Cameras_[i]->SetColorModel(ColorModels_[i]);

          // initialize image for camera and grab
          Cameras_[i]->InitImage(*CamImg_[i]);
#ifdef BIAS_HAVE_DCAM
          if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]) != NULL)
          {
            CamImg_[i]->Release();
            CamImg_[i]->Init(1280, 960, 1);
            CamImg_[i]->SetColorModel(ImageBase::CM_Bayer_GBRG);
            CamImgMiddle_[i]->Init(1280, 960, 1);
            CamImgMiddle_[i]->SetColorModel(ImageBase::CM_Bayer_GBRG);
            CamImgRight_[i]->Init(1280, 960, 1);
            CamImgRight_[i]->SetColorModel(ImageBase::CM_Bayer_GBRG);
            //Cameras_[i]->InitImage(*CamImgMiddle_[i]);
            //Cameras_[i]->InitImage(*CamImgRight_[i]);
            Cameras_[i]->InitImage(*XB3AllInOne_[i]);
            //Cameras_[i]->InitImage(CamImgCombined_);
            //CamImgCombined_.Init(1024,255,1);
            //CamImgCombined_.SetColorModel(ImageBase::CM_Bayer_GBRG);
          }
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
          // init 2D image for usb pmd cameras
          if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[i]) != NULL)
            {
              // dynamic_cast<VideoSource_usbPMD*>(Cameras_[i])->Init2DImage(*CamImg2D_[i]);  
              //CamImgCombined is 3 times image in horizontal way
              CamImgCombined_.Init((CamImg_[NumCamSel_]->GetWidth()+1)*3,
                                   CamImg_[NumCamSel_]->GetHeight()+1);
            }
#endif // BIAS_HAVE_PMD_PMDZess

          Cameras_[i]->PreGrab();
          Cameras_[i]->SetDebugLevel(1);     
        }
    }

  /************************************************/
  /******* now start the threads for grabbing *****/
  
  for (int i = 0; i < NUMBER_CAMERAS; i++) {    
    //check if camera is used
    if(Cameras_[i] != NULL){
    
      bool startGrabThread = false;
      if (dynamic_cast<VideoSource_Shm*>(Cameras_[i]) != NULL)  {
        startGrabThread = true;      
      }
    
#ifdef BIAS_HAVE_DCAM    
      else if (dynamic_cast<VideoSource_DCAM*>(Cameras_[i]) != NULL) {
        startGrabThread = true;
      }
#endif // BIAS_HAVE_DCAM

#ifdef BIAS_HAVE_PMD_PMDTec    
      else if (dynamic_cast<VideoSource_DcamPMD*>(Cameras_[i]) != NULL) {
        startGrabThread = true;     
        if(Cameras_[i]!=NULL){
          dynamic_cast<VideoSource_DcamPMD*>(Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
          dynamic_cast<VideoSource_DcamPMD*>(Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);      
        }
      }
      else if (dynamic_cast<VideoSource_ShmPMD*>(Cameras_[i]) != NULL) {
        startGrabThread = true; 
        if(Cameras_[i]!=NULL){           
          dynamic_cast<VideoSource_ShmPMD*>(Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
          dynamic_cast<VideoSource_ShmPMD*>(Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]); 
        }
      }
#endif // BIAS_HAVE_PMD_PMDTec

#ifdef BIAS_HAVE_PMD_PMDZess    
      else if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[i]) != NULL) {
        startGrabThread = true;    
        InitUSBPMD_ = false;
      }
#endif // BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_UEYE    
      else if (dynamic_cast<VideoSource_uEye*>(Cameras_[i]) != NULL) {
        startGrabThread = true;        
      }
#endif // BIAS_HAVE_UEYE
    
      else if (dynamic_cast<VideoSource_Net*>(Cameras_[i]) != NULL)  {
        startGrabThread = true;        
      }

#ifdef BIAS_HAVE_V4L    
      else if (dynamic_cast<VideoSource_V4L*>(Cameras_[i]) != NULL) {
        BIASERR("V4L support not implemented!");      
      }    
#endif // BIAS_HAVE_V4L

#ifdef BIAS_HAVE_SWISSRANGER
      else if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]) != NULL) {         
        startGrabThread = true;        
      }    
#endif // BIAS_HAVE_SWISSRANGER
       
      int ret = 0;
      if(Cameras_[i] != NULL)
        ret = CreateGrabThreadForCamera_(i);
        
    }// end if Cameras_[i] != NULL   
  }
  //after opening the devices start the timer
  OpenedDevices_ = true;

  // init and start timer
  Timer_.SetOwner(this, ID_Timer);  
  cout<<"Setting timer to "<<Delay_<<" ms"<<endl;
  StopWatch_.Start();  
  Timer_.Start(Delay_,true);
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
CreateMenu()
{
  // create file menu
  wxMenu *menuFile = new wxMenu;
  menuFile->Append(ID_Grab, wxT("Grab &single image") ); 
  menuFile->AppendCheckItem(ID_GrabContinuous, wxT("Grab &continuously (Start/Stop)") );  
  menuFile->Append(ID_Quit, wxT("E&xit") );

  // create menu for showing different cameras
  wxMenu *menuShow = new wxMenu;
  menuShow->Append(ID_Show1, wxT("Camera 1") );
  menuShow->Append(ID_Show2, wxT("Camera 2") );
  menuShow->Append(ID_Show3, wxT("Camera 3") );
  menuShow->Append(ID_Show4, wxT("Camera 4") );
  menuShow->Append(ID_Show5, wxT("Camera 5") );
  menuShow->Append(ID_Show6, wxT("Camera 6") );
  menuShow->Append(ID_Show7, wxT("Camera 7") );
  menuShow->Append(ID_Show8, wxT("Camera 8") );
  menuShow->Append(ID_Show9, wxT("Camera 9") );
  menuShow->AppendSeparator();
  menuShow->Append(ID_Controller, wxT("Controller") );  
  
  wxMenu *menuParameters = new wxMenu;
  menuParameters->Append(ID_SaveParameters, wxT("Save Parameters") );
  // menuParameters->Append(ID_ShowParameters, wxT("Show Parameters") );
  
  // add menues to menu bar
  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  menuBar->Append(menuParameters, wxT("&Parameters"));
  menuBar->Append(menuShow, wxT("&Show"));
 
  SetMenuBar(menuBar);

  // deactivate uninitialized cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++)
  {
    if (Cameras_[i] == NULL) 
    {
      menuShow->Enable(ID_Show1 + i, false);
    }
  }
  
  // fit all components to size of window
  Fit();
}

#ifdef BIAS_HAVE_PanTilt

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
GetProjectionFromPanTilt_(BIAS::Projection &projHeader){

  double pan=0.0, tilt=0.0;
  panTiltControl_->GetCurrentPositionDegreesThreaded(pan, tilt);
  std::cout<<"Current Position:"<<pan<<","<<tilt<<std::endl;

  // rotation matrix resulting from panning
  BIAS::RMatrix pan_matrix = BIAS::RMatrix(BIAS::Vector3<double>(0,1,0), pan);
  // rotation matrix resulting from tilting
  BIAS::RMatrix tilt_matrix = BIAS::RMatrix(BIAS::Vector3<double>(1,0,0), tilt);

  // new modelling of ptu pose:
  // first, pan ptu
  BIAS::Matrix4x4<double> pan_ptu = BIAS::Matrix4x4<double>(pan_matrix, BIAS::Vector3<double>(0,0,0));

  // tilt ptu
  BIAS::Matrix4x4<double> tilt_ptu = BIAS::Matrix4x4<double>(tilt_matrix, BIAS::Vector3<double>(0,0,0));

  // identity matrix
  BIAS::Matrix3x3<double> id33 = BIAS::Matrix3x3<double>(1,0,0, 0,1,0, 0,0,1);
  // translation from rotation joint to camera center
  BIAS::Vector3<double> translation(0.0, 0.0, 0.0);
  BIAS::Matrix4x4<double> trans_to_camera_center = BIAS::Matrix4x4<double>(id33, translation);

  // calculate complete ptu transformation matrix
  BIAS::Matrix4x4<double> ptu_result = tilt_ptu * pan_ptu * trans_to_camera_center;

  // do some hackish access to the translation part of resulting matrix
  BIAS::Vector4<double> translation_rotated4 = ptu_result.GetCol(3);
  BIAS::Vector3<double> translation_rotated3;
  double *translation_rotated4_ptr = translation_rotated4.GetData();
  for (int i=0;i<3;i++) 
    translation_rotated3.GetData()[i] = translation_rotated4_ptr[i];

  // store transformation in dynamic data pool entry
  Pose p;
  Quaternion<double> panq;
  pan_matrix.GetQuaternion(panq);
  Quaternion<double> tiltq;
  tilt_matrix.GetQuaternion(tiltq);
  Quaternion<double> temp = panq * tiltq;
  p.SetC(translation_rotated3);
  p.SetQ(temp);
  KMatrix K;
  projHeader.CreatePerspective(p,K,640,480);
  cout<<"Projection:"<<projHeader<<endl;
}
#endif


/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
StopSavingImagesToDisk()
{ 
  bWriteBuffered_=false;
  for (int i = 0; i < NUMBER_CAMERAS; i++){          
    if(saveThreads_[i]!=NULL && Cameras_[i] != NULL){ 
      //      cout<<"Pausing camera :"<<i<<endl;
      saveThreads_[i]->Pause();
	  cout << "SaveThread for cam: " << i << " paused" << endl;
    }
	cout << "for: " << i << endl;
  }
  cout << "leave StopSavingThreads" << endl;
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
StartSavingImagesToDisk()
{

  int ret =-1;
 
  while(ret != 0){
    stringstream dir;
    dir <<"Take"<<BIAS::LeadingZeroString(dDirIndex_,5)<<"/";
    currentSaveDir_ = dir.str().c_str();
    
#ifdef WIN32
    ret = _mkdir(dir.str().c_str());
#else //WIN32
    ret = mkdir(dir.str().c_str(), 0755);
#endif //WIN32

    dDirIndex_++;
  }

  //create a dummy Projection to save Pan Tilt angles in Image headers
  BIAS::Projection projHeader;
  bWriteBuffered_ = true;
#ifdef BIAS_HAVE_PanTilt
  if(*bUsePanTiltUnit_){    
    GetProjectionFromPanTilt_(projHeader);
  } 
#endif //BIAS_HAVE_PanTilt

  for (int i = 0; i < NUMBER_CAMERAS; i++){          
    // check if camera is opened
    if(Cameras_[i] != NULL){
  
#ifdef BIAS_HAVE_PanTilt     
      if(*bUsePanTiltUnit_){
        (*CamImg_[i]).SetProj(projHeader);
        (*CamImg_[i]).UpdateMetaData();
      }
#endif //BIAS_HAVE_PanTilt          

      if(saveThreads_[i] == NULL){      
        //Here create ImageSaveThreadForCamera
        CreateImageSaveThreadForCamera_(i);
        saveThreads_[i]->SetSaveDirectory(currentSaveDir_);
      }
      else{
        // signal thread to save a single image
        saveThreads_[i]->SetSaveDirectory(currentSaveDir_);
        saveThreads_[i]->Resume();
      } 
    }//end else
        
  }//end for all cameras
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
ParseColorModels()
{
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    switch(*ParamColorModel_[i])  {
    case 0:  ColorModels_[i] = ImageBase::CM_invalid; break;
    case 1:  ColorModels_[i] = ImageBase::CM_Grey; break;
    case 2:  ColorModels_[i] = ImageBase::CM_RGB; break;
    case 3:  ColorModels_[i] = ImageBase::CM_BGR; break;
    case 4:  ColorModels_[i] = ImageBase::CM_YUYV422; break;
    case 5:  ColorModels_[i] = ImageBase::CM_UYVY422; break;
    case 6:  ColorModels_[i] = ImageBase::CM_YUV420P; break;
    case 7:  ColorModels_[i] = ImageBase::CM_YUV444; break;
    case 8:  ColorModels_[i] = ImageBase::CM_YUV411; break;
    case 9:  ColorModels_[i] = ImageBase::CM_HSV; break;
    case 10: ColorModels_[i] = ImageBase::CM_HSI_OBS; break;
    case 11: ColorModels_[i] = ImageBase::CM_DV; break;
    case 12: ColorModels_[i] = ImageBase::CM_RGBA; break;
    case 13: ColorModels_[i] = ImageBase::CM_GreyA; break;
    case 14: ColorModels_[i] = ImageBase::CM_Bayer_RGGB; break;
    case 15: ColorModels_[i] = ImageBase::CM_Bayer_GBRG; break;
    case 16: ColorModels_[i] = ImageBase::CM_Bayer_GRBG; break;
    case 17: ColorModels_[i] = ImageBase::CM_Bayer_BGGR; break;
    case 18: ColorModels_[i] = ImageBase::CM_HSL; break;
    case 19: ColorModels_[i] = ImageBase::CM_hsL; break;
    case 20: ColorModels_[i] = ImageBase::CM_SymTensor2x2; break;
    case 21: ColorModels_[i] = ImageBase::CM_BGRA; break;
    case 22: ColorModels_[i] = ImageBase::CM_RGBE; break;
    default: ColorModels_[i] = ImageBase::CM_invalid; break;
    }
  }
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
InitCombinedImage()
{
  // initialize combined image from pmd usb camera images
  CamImgCombined_.SetROICorners(0, 0, 
                                CamImg_[NumCamSel_]->GetWidth(),
                                CamImg_[NumCamSel_]->GetHeight());
  CamImgCombined_.Paste2ROI(*CamImg_[NumCamSel_]);
  CamImgCombined_.SetROICorners(CamImg_[NumCamSel_]->GetWidth(),
                                0,
                                CamImg_[NumCamSel_]->GetWidth() * 2,
                                CamImg_[NumCamSel_]->GetHeight());


  // add Amp image from pmd usb camera
  BIAS::Camera<float> tmpImg;
  BIAS::Camera<unsigned char> tmpImgUC;  
  tmpImg = *CamImgPMDAmp_[NumCamSel_];

  if (InitUSBPMD_)
    tmpImg.ScaleShiftBetween(0,255);

  BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
  CamImgCombined_.Paste2ROI(tmpImgUC);
  CamImgCombined_.SetROICorners(CamImg_[NumCamSel_]->GetWidth() * 2,
                                0,
                                CamImg_[NumCamSel_]->GetWidth() * 3,
                                CamImg_[NumCamSel_]->GetHeight());

  // add Depth image from pmd usb camera
  tmpImg = *CamImgPMDDepth_[NumCamSel_];

  if (InitUSBPMD_)
    tmpImg.ScaleShiftBetween(0,255);

  BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
  CamImgCombined_.Paste2ROI(tmpImgUC);
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnSaveParameters(wxCommandEvent& event){
  
  wxString fileName;
  wxFileDialog dialog(this,wxT("Save Parameters"),wxT(""),wxT(""),
                      wxT("MIP Parameter files (*.param)|*.param"),
                      wxSAVE|wxOVERWRITE_PROMPT);
  int ret = dialog.ShowModal();
  if(ret == wxID_OK){
    fileName = dialog.GetPath();
    WriteParameter(WxToAscii(fileName));
  } 
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnQuit(wxCommandEvent& event)
{
  // stop timer
  Timer_.Stop();

  // shut down all opened cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++)  {
    if (Cameras_[i] != NULL) {
      Cameras_[i]->PostGrab();
      Cameras_[i]->CloseDevice();
    }
  }

#ifdef BIAS_HAVE_PanTilt
  if(*bUsePanTiltUnit_){
    if(panTiltControlDialog_->IsShown())
      panTiltControlDialog_->Hide();
    delete panTiltControlDialog_;

    if(panTiltControl_ != NULL){  
      panTiltControl_->SetPosition(0,0);
      panTiltControl_->ClosePanTiltUnit();
      delete panTiltControl_;
    }
  }
#endif //BIAS_HAVE_PanTilt

  Close(true);
}
 
/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnGrab(wxCommandEvent& event)
{

  if(!bSingleSaveDirCreated_){
    int ret =-1;

    while(ret != 0){
      stringstream dir;
      dir <<"SingleTake"<<BIAS::LeadingZeroString(dSingleDirIndex_,5)<<"/";
      currentSingleSaveDir_ = dir.str().c_str();
    
#ifdef WIN32
      ret = _mkdir(dir.str().c_str());
#else //WIN32
      ret = mkdir(dir.str().c_str(), 0755);
#endif //WIN32

      dSingleDirIndex_++;
    }
    bSingleSaveDirCreated_ = true;
  }
 
  for (int i = 0; i < NUMBER_CAMERAS; i++){          
    if(Cameras_[i] != NULL){
      SaveAllImagesUnthreaded_(i);
    }
  }
  GrabCounter_++;
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnGrabContinuous(wxCommandEvent& event)
{
  // reset counter and enable/disable continuous saving of images
  FrameCounter_ = 0;    
  bSaveImages_ = !bSaveImages_;
  if (bSaveImages_)
    StartSavingImagesToDisk();
  else
    StopSavingImagesToDisk();
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage1(wxCommandEvent& event)
{
  NumCamSel_ = 0;  
 
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage2(wxCommandEvent& event)
{
  NumCamSel_ = 1;  

}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage3(wxCommandEvent& event)
{
  NumCamSel_ = 2;  
 
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage4(wxCommandEvent& event)
{
  NumCamSel_ = 3;  
 
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage5(wxCommandEvent& event)
{
  NumCamSel_ = 4;  
 
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage6(wxCommandEvent& event)
{
  NumCamSel_ = 5;  
  
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage7(wxCommandEvent& event)
{
  NumCamSel_ = 6;  
  cout<<"Selected Camera: "<<NumCamSel_<<endl;  
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage8(wxCommandEvent& event)
{
  NumCamSel_ = 7;  
  cout<<"Selected Camera: "<<NumCamSel_<<endl;  
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowImage9(wxCommandEvent& event)
{
  NumCamSel_ = 8;  
  cout<<"Selected Camera: "<<NumCamSel_<<endl;  
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnShowController(wxCommandEvent& event)
{
  // check if camera is initialized
  if (Cameras_[NumCamSel_] != NULL) {

#ifdef BIAS_HAVE_PMD_PMDZess
    // check if it is a usb pmd camera
    if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]) != NULL) {
      // initialize controller if it was not before 
      if(VideoControllersUSBPMD_[NumCamSel_] == NULL) {
        VideoControllersUSBPMD_[NumCamSel_] = 
          new VideoSource_Controller_usbPMD(
                                            dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]),
                                            this,
                                            0,3000,1002,8000,100,2000);
      }
      // show controller
      VideoControllersUSBPMD_[NumCamSel_]->Show(true);  
    } else 
#endif // BIAS_HAVE_PMD_PMDZess
#ifdef BIAS_HAVE_SWISSRANGER
      if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[NumCamSel_]) != NULL) {
        // initialize controller if it was not before 
        if(VideoControllersSwissRanger_[NumCamSel_] == NULL) {
          VideoControllersSwissRanger_[NumCamSel_] = 
            new VideoSource_Controller_SwissRanger(
                                   dynamic_cast<VideoSource_SwissRanger*>(Cameras_[NumCamSel_]),
                                   this);
        }
        // show controller
        VideoControllersSwissRanger_[NumCamSel_]->Show(true);  
      } else 
#endif // BIAS_HAVE_SWISSRANGER
#ifdef BIAS_HAVE_DCAM
      if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[NumCamSel_]) != NULL) {
        // initialize controller if it was not before 
        if(VideoControllersXB3_[NumCamSel_] == NULL) {
          VideoControllersXB3_[NumCamSel_] = 
            new VideoSource_Controller_XB3(
                                   dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[NumCamSel_]),
                                   this,"VideoController for XB3",ShowXB3_);
        }
        // show controller
        VideoControllersXB3_[NumCamSel_]->Show(true);  
      } else 
#endif
        // initialize controller if it was not before
        if (VideoControllers_[NumCamSel_] == NULL)        
          {       
            VideoControllers_[NumCamSel_] = 
              new VideoSource_Controller(
                                         Cameras_[NumCamSel_],
                                         this,
                                         "VideoController",1.0,99.0);        
            // show controller
            VideoControllers_[NumCamSel_]->Show(true);
          } else {
          // show controller
          VideoControllers_[NumCamSel_]->Show(true);
        }

  }

  // camera was not initalized
  else
    {
      wxMessageDialog errorDialog(this,
                                  wxT("No correct camera was selected"),
                                  wxT("ShowCamWx - Error"),
                                  wxICON_ERROR|wxOK);
      errorDialog.ShowModal();    
    }      
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
OnTimer(wxTimerEvent& event)
{     
  // update frames per second and restart timer and counters
  Timer_.Stop();
  StopWatch_.Stop();
  double fps = 1000000.0 / StopWatch_.GetRealTime();
  StopWatch_.Reset();
  StopWatch_.Start();
  FrameCounter_++;

  wxString text;
  text.Printf(wxT("%05d %2.1f fps "),FrameCounter_, fps);
  SetStatusText(text, 2);
  
  //check if we opened all devices and started the grabthreads
  if(!OpenedDevices_){
    BIASERR("Devices not open!");
    return;
  } 
  // get images from camera 
  // wait for Signal from each Thread
  for ( int i = 0; i < NUMBER_CAMERAS; i++) { 
    if(Cameras_[i] != NULL && ThreadInited_[i] == true){
      // signal thread to continue    
      Conditions_[i]->Wait(); 
      //std::cout<<"Waiting camera :"<<i<<std::endl;
    }
    else if(Cameras_[i] != NULL){
      GrabSingleImageUnthreadedFromCamera_(i);
    } //end Cameras_[i] != NULL           
  }

  SelectAndShowCameraImage_();

  Timer_.Start(Delay_,true);   
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
SelectAndShowCameraImage_(){
  // show camera image in main window if a camera was selected
  if (Cameras_[NumCamSel_] != NULL)
    {
      unsigned int w;
      unsigned int h;

#ifdef BIAS_HAVE_PMD_PMDZess    
      if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]) != NULL)
        {
          // get width and height from combined image
          w = CamImgCombined_.GetWidth(); 
          h = CamImgCombined_.GetHeight();
        }
      else
        {
#endif // BIAS_HAVE_PMD_PMDZess

            // get width and height from image
            w = CamImg_[NumCamSel_]->GetWidth();
            h = CamImg_[NumCamSel_]->GetHeight();
            //  }

          if(w > 850 || h >600){        
            w= 850; h=600;       
          }
          
#ifdef BIAS_HAVE_PMD_PMDZess    
        }
#endif // BIAS_HAVE_PMD_PMDZess
      wxString text;
      text.Printf(wxT("size: %4dx%4d"),CamImg_[NumCamSel_]->GetWidth(), 
                  CamImg_[NumCamSel_]->GetHeight());
      SetStatusText(text, 1);

      // set new height or width if changed
      if (w != LastWidth_ || h != LastHeight_)
        {
          LastWidth_  = w;
          LastHeight_ = h;
          IC_->SetSize(w+50,h+80);
          SetMinSize(wxSize(w+50,h+80));
          Fit();        
        }

      if (CamImg_[NumCamSel_] != NULL)
        {

#ifdef BIAS_HAVE_PMD_PMDZess    
          if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]) != NULL)
            {
              InitCombinedImage();
              ShowImage_ = CamImgCombined_;
            }
          else
            
#endif // BIAS_HAVE_PMD_ZESS              
// show grabbed image on main frame
            if(*ParamCameras_[NumCamSel_] == 10){
              
              switch(ShowXB3_){
              case 1: ShowImage_ = *CamImg_[NumCamSel_]; break;
              case 2: ShowImage_ = *CamImgMiddle_[NumCamSel_]; break;
              case 3: ShowImage_ = *CamImgRight_[NumCamSel_]; break;
              case 4: ShowImage_ = *XB3AllInOne_[NumCamSel_]; break;
              case 0: ShowImage_ = *XB3AllInOne_[NumCamSel_];
                BIAS::ImageBase tmp = ShowImage_;
                ImageConvert::DeinterleaveHorizontal(tmp, ShowImage_); 
                break;
              }
              
              
            }else{         
              ShowImage_ = *CamImg_[NumCamSel_];
            }
          
          
          // downsampling of actual camera image if set by param file
          int downSample = *ParamDown_[NumCamSel_];
          if (downSample != 0)
            {
              Camera<unsigned char> rescaledImage;        
              Rescale<unsigned char, unsigned char> rescale;
              rescale.Downsample(ShowImage_, rescaledImage,
                                 (int)(ShowImage_.GetWidth() / downSample),
                                 (int)(ShowImage_.GetHeight() / downSample));
              ShowImage_ = rescaledImage;
            }

          unsigned width = ShowImage_.GetWidth();
          unsigned height = ShowImage_.GetHeight();
     
          if(width > 1024 || height >768){

            /////////////////////////////////////
            // get size available for image canvas
            if(ShowXB3_ == 0){          
              int widthClient, heightClient;
              GetClientSize(&widthClient, &heightClient);
              
              // compute scale that keeps the aspect ratio and the image is fully visible
              float xscale = (float)widthClient / width;
              float yscale = (float)heightClient / height;
              float scale = xscale < yscale ? xscale : yscale;
              
              IC_->SetScaledImageSize((int)(width * scale), (int)(height * scale));
              IC_->SetShowScaled(true);
              /////////////////////////////////////
              
            }else{
              IC_->SetScaledImageSize ( 960,  768);
              IC_->SetShowScaled(true);
            }
          }
          else
            IC_->SetShowScaled(false);
          // show actual camera image on main frame
          IC_->Show(ShowImage_, "Camera Image");
        }
    }
}


/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::OnKey(wxKeyEvent& event)
{
  // no key events implemented yet
}


/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
OnShowParameters(wxCommandEvent& event){
  ShowParamGUI();
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
ShowParamGUI(){
  if(paramFrame_ == NULL){
    paramFrame_ = new wxFrame(this,ID_Paramframe,wxT("Parameters"), 
                              wxPoint(500,200),wxSize(550,400));
    paramNotebook_ = GetNotebook(paramFrame_);
    wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
    vbox->Add(paramFrame_,1,  wxEXPAND |wxALL |wxFIXED_MINSIZE , 5);
    //paramFrame_->SetMinSize(wxSize(450,400));
    paramFrame_->SetSizer(vbox);
    paramFrame_->Show(true);
    paramFrame_->Move( wxPoint(500,200));
  }
}


/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveAllImagesUnthreaded_(int cameraIndex){
  //HACK!!!!
#ifdef BIAS_HAVE_PMD_PMDZess                                    
  if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[cameraIndex]) != NULL){
    SaveImagePMDZessUnthreaded_(cameraIndex);
  }
#endif // BIAS_HAVE_PMD_PMDZess
                
#ifdef BIAS_HAVE_SWISSRANGER
  if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[cameraIndex]) != NULL){
    SaveImagePMDSRUnthreaded_(cameraIndex);         
                
  }
#endif // BIAS_HAVE_SWISSRANGER
                  
                   
#ifdef BIAS_HAVE_DCAM
//#ifndef WIN32 
  if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[cameraIndex]) != NULL){
    SaveImageXB3Unthreaded_(cameraIndex);                   
  }
  else if (dynamic_cast<VideoSource_DCAM*>(Cameras_[cameraIndex]) != NULL){
    SaveImageUnthreaded_(cameraIndex);                        
  }
#endif //DCAM                  
    //  SaveImageUnthreaded_(cameraIndex);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImagePMDZessUnthreaded_(int cameraIndex){
  wxString text;
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-grey.mip"), cameraIndex, GrabCounter_);
  CamImg_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  
  if(CamImg2D_[cameraIndex] != NULL){
    text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-2D.mip"), cameraIndex, GrabCounter_);
    CamImg_[cameraIndex]->UpdateMetaData();
    ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImg2D_[cameraIndex]);
  }
  //  cout <<"Saved PMD Zess image: "<<WxToAscii(text)<<endl;                       
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
  CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-amp.mip"), cameraIndex, GrabCounter_);
  CamImgPMDAmp_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
}


/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImagePMDSRUnthreaded_(int cameraIndex){
  wxString text;
  //  CamImg_[cameraIndex]->UpdateMetaData();
  
  //   text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-grey.mip"), cameraIndex, GrabCounter_);
  //   ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
  CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
  
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d-amp.mip"), cameraIndex, GrabCounter_);
  CamImgPMDAmp_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImageUnthreaded_(int cameraIndex){
  wxString text;
  CamImg_[cameraIndex]->UpdateMetaData();
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-%05d.mip"), cameraIndex, GrabCounter_);
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImageXB3Unthreaded_(int cameraIndex){
  wxString text;
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-left-%05d.mip"), cameraIndex, GrabCounter_);
  CamImg_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-center-%05d.mip"), cameraIndex, GrabCounter_);
  CamImgMiddle_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImgMiddle_[cameraIndex]);
                    
  text.Printf(wxT("ShowCamWx-SINGLE-CAM%02d-right-%05d.mip"), cameraIndex, GrabCounter_);
  CamImgRight_[cameraIndex]->UpdateMetaData();
  ImageIO::Write(currentSingleSaveDir_ + WxToAscii(text), *CamImgRight_[cameraIndex]);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
GrabSingleImageUnthreadedFromCamera_(int cameraIndex){
  int i=0;
  i = cameraIndex;
#ifdef BIAS_HAVE_SWISSRANGER
  if(Cameras_[i] != NULL && dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]) != NULL){
    dynamic_cast<VideoSource_SwissRanger*>
      (Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
    dynamic_cast<VideoSource_SwissRanger*>
      (Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);
    Cameras_[i]->GrabSingle(*CamImg_[i]);
  }else
 
#endif // BIAS_HAVE_SWISSRANGER
//#ifndef WIN32
#ifdef BIAS_HAVE_DCAM
  if(dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]) != NULL){
    //cout << "Before Casting" << endl;
    (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]))->
      GrabSingle(*XB3AllInOne_[i], *CamImg_[i],*CamImgMiddle_[i],*CamImgRight_[i]);
  }else
//#endif
#endif //not WIN32
  {
    Cameras_[i]->GrabSingle(*CamImg_[i]);                    
  }
}


/****************************************************
   @brief


*****************************************************/
int ShowCamWxFrame::
CreateGrabThreadForCamera_(int cameraIndex){
  // create new thread
#ifdef BIAS_HAVE_SWISSRANGER
  if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[cameraIndex]) != NULL){
    std::cout<<"SR Cond i OK?:"<<Conditions_[cameraIndex]->IsOk()<<"CameraIndex: "<<cameraIndex<<endl;
    std::cout<<"SR Mutex i OK?:"<<Mutexes_[cameraIndex]->IsOk()<<"CameraIndex: "<<cameraIndex<<endl;
    grabSingleThread_[cameraIndex] = 
      new GrabSingleThreadSR(dynamic_cast<VideoSource_SwissRanger*>(Cameras_[cameraIndex]),
                             CamImg_[cameraIndex],
                             CamImgPMDDepth_[cameraIndex], 
                             CamImgPMDAmp_[cameraIndex],
                             Conditions_[cameraIndex],
                             BufferAccess_[cameraIndex],
                             *ImageBuffer_[cameraIndex],
                             *ImageBufferPMDDepth_[cameraIndex],
                             *ImageBufferPMDAmp_[cameraIndex],
                             bWriteBuffered_,
                             ReadIndex_[cameraIndex],
                             WriteIndex_[cameraIndex],
                             Mutexes_[cameraIndex]);          
  }else
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
    if(dynamic_cast<VideoSource_usbPMD*>(Cameras_[cameraIndex]) != NULL){
      std::cout<<"Zess Cond i :"<<Conditions_[cameraIndex]->IsOk()<<" "<<cameraIndex<<endl;
      std::cout<<"Zess Mutex i :"<<Mutexes_[cameraIndex]->IsOk()<<" "<<cameraIndex<<endl;
      grabSingleThread_[cameraIndex] = 
        new GrabSingleThreadPMDZess(dynamic_cast<VideoSource_usbPMD*>(Cameras_[cameraIndex]),
                                    CamImg2D_[cameraIndex],
                                    CamImg_[cameraIndex],
                                    CamImgPMDDepth_[cameraIndex], 
                                    CamImgPMDAmp_[cameraIndex],
                                    Conditions_[cameraIndex],
                                    BufferAccess_[cameraIndex],
                                    *ImageBuffer_[cameraIndex],
                                    *ImageBufferMiddle_[cameraIndex],
                                    *ImageBufferPMDDepth_[cameraIndex],
                                    *ImageBufferPMDAmp_[cameraIndex],
                                    bWriteBuffered_,
                                    ReadIndex_[cameraIndex],
                                    WriteIndex_[cameraIndex],
                                    Mutexes_[cameraIndex]);          
    }
    else
#endif

#ifdef BIAS_HAVE_DCAM
      //#ifndef WIN32
      
      if(dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[cameraIndex]) != NULL){            
        std::cout<<"XB3 Cond i OK?:"<<Conditions_[cameraIndex]->IsOk()<<"CameraIndex: "<<cameraIndex<<endl;
        std::cout<<"XB3 Mutex i OK?:"<<Mutexes_[cameraIndex]->IsOk()<<"CameraIndex: "<<cameraIndex<<endl;
        grabSingleThread_[cameraIndex] =
          new GrabSingleThreadXB3(dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[cameraIndex]), 
                                  CamImg_[cameraIndex],
                                  CamImgMiddle_[cameraIndex],
                                  CamImgRight_[cameraIndex],
                                  XB3AllInOne_[cameraIndex],
                                  Conditions_[cameraIndex],
                                  BufferAccess_[cameraIndex],
                                  *ImageBuffer_[cameraIndex],
                                  *ImageBufferMiddle_[cameraIndex],
                                  *ImageBufferRight_[cameraIndex],
                                  bWriteBuffered_,
                                  ReadIndex_[cameraIndex],
                                  WriteIndex_[cameraIndex],
                                  Mutexes_[cameraIndex]);
      }else
        //#endif
#endif

        {
          std::cout<<"Cond i OK?:"<<Conditions_[cameraIndex]->IsOk()<<"CameraIndex: "<<cameraIndex<<endl;
          std::cout<<"Mutex i OK?:"<<Mutexes_[cameraIndex]->IsOk()<<"CameraIndex: "<<cameraIndex<<endl;
          grabSingleThread_[cameraIndex] = 
            new GrabSingleThread(Cameras_[cameraIndex], 
                                 CamImg_[cameraIndex],
                                 Conditions_[cameraIndex],
                                 BufferAccess_[cameraIndex],
                                 *ImageBuffer_[cameraIndex],
                                 bWriteBuffered_,
                                 ReadIndex_[cameraIndex],
                                 WriteIndex_[cameraIndex],
                                 Mutexes_[cameraIndex]);
        }
        

  // try to create and start thread
  if(grabSingleThread_[cameraIndex]!= NULL && !grabSingleThread_[cameraIndex]->IsAlive()){
    if(grabSingleThread_[cameraIndex]->Create()!=wxTHREAD_NO_ERROR) {
      BIASERR("Cannot create GrabSingleThread, saving unthreaded");   
    }     
    else{
      std::cout<<"Created GrabSingleThread: "<<cameraIndex<<endl;        
      grabSingleThread_[cameraIndex]->Run();         
      ThreadInited_[cameraIndex] = true;
      // Thread ID ausgeben
      cout << "Created Thread with ID: " << grabSingleThread_[cameraIndex]->GetId() 
           << " for CAM: " << cameraIndex << endl;
    }
  }
  else if(grabSingleThread_[cameraIndex]== NULL){
    BIASERR("Thread:"<<cameraIndex<<" is NULL!");
  }
  else if(grabSingleThread_[cameraIndex]->IsAlive()){
    BIASERR("Thread:"<<cameraIndex<<" is already Alive!");
  } 
    

  return 0;
}



/****************************************************
   @brief


*****************************************************/
int ShowCamWxFrame::
CreateImageSaveThreadForCamera_(int cameraIndex){
  int i = 0;
  i = cameraIndex;
  wxString text;
  text.Printf(wxT("ShowCamWx-CAM%02d"), i);
#ifdef BIAS_HAVE_PMD_PMDZess
  // grab all images from usb pmd camera
  if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[i]) != NULL){
   
    // #ifdef BIAS_HAVE_PanTilt
    //     if(*bUsePanTiltUnit_){
    //       (*CamImgPMDDepth_[i]).SetProj(projHeader);
    //       (*CamImgPMDAmp_[i]).SetProj(projHeader);
    //       (*CamImgPMDAmp_[i]).UpdateMetaData();
    //       (*CamImgPMDDepth_[i]).UpdateMetaData();
    //     }
    // #endif
    // create new thread
    saveThreads_[i] = new ImageSaveThread(text,
                                          *CamImg2D_[i],
                                          *CamImg_[i],
                                          *CamImgPMDDepth_[i],
                                          *CamImgPMDAmp_[i],
                                          *ParamDown_[i],
                                          SaveConditions_[i],
                                          BufferAccess_[i],
                                          *ImageBuffer_[i],
                                          *ImageBufferMiddle_[i],
                                          *ImageBufferPMDDepth_[i],
                                          *ImageBufferPMDAmp_[i],
                                          bWriteBuffered_,
                                          ReadIndex_[i],
                                          WriteIndex_[i]);          
  }    
#endif // BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_SWISSRANGER
  // grab all images from usb pmd camera
  if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]) != NULL){
    // #ifdef BIAS_HAVE_PanTilt
    //       if(*bUsePanTiltUnit_){
    //         (*CamImgPMDDepth_[i]).SetProj(projHeader);
    //         (*CamImgPMDAmp_[i]).SetProj(projHeader);
    //         (*CamImgPMDAmp_[i]).UpdateMetaData();
    //         (*CamImgPMDDepth_[i]).UpdateMetaData();
    //       }
    // #endif
                                             
    // create new thread
    saveThreads_[i] = new ImageSaveThread(text,
                                          *CamImg_[i],
                                          *CamImgPMDDepth_[i],
                                          *CamImgPMDAmp_[i],
                                          *ParamDown_[i],
                                          SaveConditions_[i],
                                          BufferAccess_[i],
                                          *ImageBuffer_[i], 
                                          *ImageBufferPMDDepth_[i],
                                          *ImageBufferPMDAmp_[i],
                                          bWriteBuffered_,
                                          ReadIndex_[i],
                                          WriteIndex_[i]);
                      
  } // dynamic_cast<VideoSource_SwissRanger*>   
#endif // BIAS_HAVE_SWISSRANGER

#ifdef BIAS_HAVE_DCAM
  if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]) != NULL){
    // create new thread  
    saveThreads_[i] = new ImageSaveThread(text,
                                          *CamImg_[i],
                                          *CamImgMiddle_[i],
                                          *CamImgRight_[i],
                                          *ParamDown_[i],
                                          SaveConditions_[i],
                                          BufferAccess_[i],
                                          *ImageBuffer_[i],
                                          *ImageBufferMiddle_[i],
                                          *ImageBufferRight_[i],
                                          bWriteBuffered_,
                                          ReadIndex_[i],
                                          WriteIndex_[i]);
                          
  } // dynamic_cast<VideoSource_DCAM_XB3*>
  
  else if (dynamic_cast<VideoSource_DCAM*>(Cameras_[i]) != NULL)
    {
      saveThreads_[i] = new ImageSaveThread(text,
                                            *CamImg_[i],
                                            *ParamDown_[i],
                                            SaveConditions_[i],
                                            BufferAccess_[i],
                                            *ImageBuffer_[i],
                                            bWriteBuffered_,
                                            ReadIndex_[i],
                                            WriteIndex_[i]);
    }
#endif //BIAS_HAVE_DCAM 
#ifdef BIAS_HAVE_UEYE
  if (dynamic_cast<VideoSource_uEye*>(Cameras_[i]) != NULL){
    saveThreads_[i] = new ImageSaveThread(text,
                                          *CamImg_[i],
                                          *ParamDown_[i],
                                          SaveConditions_[i],
                                          BufferAccess_[i],
                                          *ImageBuffer_[i],
                                          bWriteBuffered_,
                                          ReadIndex_[i],
                                          WriteIndex_[i]);
  }
#endif


  // init the thread or capture single image
  if(saveThreads_[i] != NULL && saveThreads_[i]->Create() != wxTHREAD_NO_ERROR){
    BIASERR("Cannot create ImageSaveThread, saving unthreaded");  
  }//saveThreads_[i] != NULL && saveThreads_[i]->Create()            
  else{
    cout << "Run Thread" << endl;
    saveThreads_[i]->SetPriority(100);
    saveThreads_[i]->Run();
  }
  return 0;
}
