#ifdef WIN32
#include <Base\Common\W32Compat.hh>
#endif
#include "ShowCamWxThreads.hh"
#include <Filter/Rescale.hh>
#include <wx/wx.h>
using namespace BIAS;
using namespace std;


#define BUFFER_CAPACITY_MIN  0.7
#define BUFFER_WRITE_SLEEP 10


/*********************  ImageSaveThread  **********************************
 **************************************************************************/

ImageSaveThread::
ImageSaveThread(const wxString& filename,
                const BIAS::Camera<unsigned char>& img,
                const int& downsample,
                wxCondition *condition,
                wxMutex* mutex,
                std::vector<Camera<unsigned char>*>& imageBuffer,
                bool& writeToBuffer,
                unsigned int& readIndex,
                unsigned int& writeIndex)
  : wxThread(wxTHREAD_DETACHED)  
{
  InitBoolean_();
  DownSample_ = downsample;
  Img_ = img; 
  Filename_ = filename;
  BufferAccess_ =  mutex;
  ImageBuffer_ = &imageBuffer;
  WriteToBuffer_ = &writeToBuffer;
  ReadIndex_ = &readIndex;
  WriteIndex_ = &writeIndex;
  ImageCounter_ = 0;
  Condition_ = condition; 
  saveDirectory_ = "";
}   

    // constructor for triclops cam
ImageSaveThread::
ImageSaveThread(const wxString& filename,
                const BIAS::Camera<unsigned char>& img,
                const BIAS::Camera<unsigned char>& imgCenter,
                const BIAS::Camera<unsigned char>& imgRight,
                const int& downsample,
                wxCondition *condition,
                wxMutex* mutex,
                std::vector<Camera<unsigned char>*> &imageBuffer,
                std::vector<Camera<unsigned char>*> &imageBufferCenter,
                std::vector<Camera<unsigned char>*> &imageBufferRight,
                bool& writeToBuffer,
                unsigned int& readIndex,
                unsigned int& writeIndex)
  : wxThread(wxTHREAD_DETACHED)  
{
  InitBoolean_();
  DownSample_ = downsample;
  Img_ = img;
  ImgCenter_  = imgCenter;
  ImgRight_ = imgRight;
  isTriclops_ = true;
  Condition_ = condition;
  Filename_ = filename; 
  BufferAccess_ =  mutex;
  ImageBuffer_ = &imageBuffer;
  ImageBufferCenter_ = &imageBufferCenter;
  ImageBufferRight_ = &imageBufferRight;
  WriteToBuffer_ = &writeToBuffer;
  ReadIndex_ = &readIndex;
  WriteIndex_ = &writeIndex;
  ImageCounter_ = 0; 
  saveDirectory_ = "";
}
    // constructor for swissranger
ImageSaveThread::
ImageSaveThread(const wxString& filename,
                const BIAS::Camera<unsigned char>& img,
                const BIAS::Camera<float>& imgDepth,
                const BIAS::Camera<float>& imgAmp,
                const int& downsample,
                wxCondition *condition,
                wxMutex* mutex,
                std::vector<Camera<unsigned char>*> &imageBuffer,
                std::vector<Camera<float>*> &imageBufferPmdDepth,
                std::vector<Camera<float>*> &imageBufferPmdAmp,
                bool& writeToBuffer,
                unsigned int& readIndex,
                unsigned int& writeIndex)
  : wxThread(wxTHREAD_DETACHED)  
{
  InitBoolean_();
  DownSample_ = downsample;
  Img_ = img;
  ImgDepth_  = imgDepth;
  ImgAmp_ = imgAmp;
  Condition_ = condition;
  Filename_ = filename; 
  BufferAccess_ =  mutex;
  ImageBuffer_ = &imageBuffer;
  ImageBufferPMDDepth_ = &imageBufferPmdDepth;
  ImageBufferPMDAmp_ = &imageBufferPmdAmp;
  WriteToBuffer_ = &writeToBuffer;
  ReadIndex_ = &readIndex;
  WriteIndex_ = &writeIndex;
  ImageCounter_ = 0;
  isSwissRanger_ = true;
  saveDirectory_ = "";        
}

// constructor for pmdzess
ImageSaveThread::
ImageSaveThread(const wxString& filename,
                const BIAS::Camera<unsigned char>& img2D,
                const BIAS::Camera<unsigned char>& img,
                const BIAS::Camera<float>& imgDepth,
                const BIAS::Camera<float>& imgAmp,
                const int& downsample,
                wxCondition *condition,
                wxMutex* mutex,
                std::vector<Camera<unsigned char>*> &imageBuffer2d,
                std::vector<Camera<unsigned char>*> &imageBuffer,
                std::vector<Camera<float>*> &imageBufferPmdDepth,
                std::vector<Camera<float>*> &imageBufferPmdAmp,
                bool& writeToBuffer,
                unsigned int& readIndex,
                unsigned int& writeIndex)
  : wxThread(wxTHREAD_DETACHED)  
{
  InitBoolean_();
  DownSample_ = downsample;
  Img_ = img;
  Img2D_ = img2D;
  ImgDepth_  = imgDepth;
  ImgAmp_ = imgAmp;
  isPMDZess_ = true;
  Condition_ = condition;
  Filename_ = filename; 
  BufferAccess_ =  mutex;
  ImageBuffer_ = &imageBuffer;
  ImageBufferCenter_ = &imageBuffer2d;
  ImageBufferPMDDepth_ = &imageBufferPmdDepth;
  ImageBufferPMDAmp_ = &imageBufferPmdAmp;
  WriteToBuffer_ = &writeToBuffer;
  ReadIndex_ = &readIndex;
  WriteIndex_ = &writeIndex;
  ImageCounter_ = 0; 
  isSwissRanger_ = false; 
  saveDirectory_ = "";      
}

// constructor for single float images
ImageSaveThread::
ImageSaveThread(const wxString& filename,
                const BIAS::Camera<float>& img,
                const int& downsample,
                wxCondition *condition,
                wxMutex* mutex,
                std::vector<Camera<float>*> &imageBuffer,
                bool& writeToBuffer,
                unsigned int& readIndex,
                unsigned int& writeIndex) 
  : wxThread(wxTHREAD_DETACHED)  
{
  InitBoolean_();
  DownSample_ = downsample;
  isFloat_ = true;
  ImgF_ = img;
  Filename_ = filename;
  Condition_ = condition;
  BufferAccess_ =  mutex;
  ImageBufferFloat_ = &imageBuffer;
  WriteToBuffer_ = &writeToBuffer;
  ReadIndex_ = &readIndex;
  WriteIndex_ = &writeIndex;
  ImageCounter_ = 0; 
  isSwissRanger_ = false;  
  saveDirectory_ = "";      
}   


void* ImageSaveThread::
Entry() {
  while(!TestDestroy()){
    
    if(*WriteToBuffer_){
      // save images from buffer    
      BufferAccess_->Lock();
      //    std::cout << "WriteIndex: " << *WriteIndex_ << ", ReadIndex: "
      //      << *ReadIndex_ << std::endl;
      if((*WriteIndex_ != ((*ReadIndex_ + 1)  % ImageBuffer_->size()))){
        // && ImageBuffer_[*ReadIndex_ + 1] != NULL){
        *ReadIndex_ = (*ReadIndex_ + 1) % ImageBuffer_->size();
  
        BufferAccess_->Unlock();

        // write Image     
        if(isTriclops_){
          // std::cout << "Save DCAM XB3 in Thread" << std::endl;
          SaveImage_(*(*ImageBuffer_)[*ReadIndex_],
                     *(*ImageBufferCenter_)[*ReadIndex_],
                     *(*ImageBufferRight_)[*ReadIndex_]);

        }else if(isSwissRanger_){
          SaveImage_(*(*ImageBuffer_)[*ReadIndex_],
                     *(*ImageBufferPMDDepth_)[*ReadIndex_],
                     *(*ImageBufferPMDAmp_)[*ReadIndex_]);
          
        }else if(isFloat_){
          SaveImage_(ImgF_);
        }else{         
          SaveImage_(*(*ImageBuffer_)[*ReadIndex_]);
        }

        ImageCounter_++;
      }
      else{
        // wait for images in buffer
        BufferAccess_->Unlock();      
        usleep(10);
      }       
    }else{
      // save single image
      if(isTriclops_){
        SaveImage_(Img_,
                   ImgCenter_,
                   ImgRight_);
      }else if(isSwissRanger_){
        SaveImage_(Img_,
                   ImgDepth_,
                   ImgAmp_);
      }else if(isFloat_){
        SaveImage_(ImgF_);
      }else{
        SaveImage_(Img_);
      }
      ImageCounter_++;
    }
    //  Condition_->Wait();
  }

  return NULL;
}

void ImageSaveThread::
SaveImage_(Camera <unsigned char>& im){
  wxString text;
  text.Printf(wxT("-%05d.mip"),ImageCounter_);
  FilenameOut_ = Filename_  + text;
      
  if(DownSample_ != 0){
    Camera<unsigned char> rescaledImage;  
    Rescale<unsigned char, unsigned char> rescale;
    rescale.Downsample(im, rescaledImage,
                       (int)(im.GetWidth() / DownSample_),
                       (int)(im.GetHeight() / DownSample_)); 
    rescaledImage.UpdateMetaData();
        
    BIAS::ImageIO::Write(saveDirectory_ + WxToAscii(FilenameOut_), rescaledImage, true); 
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }else{
    im.UpdateMetaData();  
    BIAS::ImageIO::Write(saveDirectory_ + WxToAscii(FilenameOut_), im, true);
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }
}

void ImageSaveThread::
SaveImage_(Camera <float>& imF){
  wxString text;
  text.Printf(wxT("-%05d.mip"),ImageCounter_);
  FilenameOut_ = Filename_  + text;
      
  if(DownSample_ != 0){
    Camera<float> rescaledImage;  
    Rescale<float,float> rescale;
    rescale.Downsample(imF, rescaledImage,
                       (int)(imF.GetWidth() / DownSample_),
                       (int)(imF.GetHeight() / DownSample_)); 
    rescaledImage.UpdateMetaData();
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImage, true); 
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }else{
    imF.UpdateMetaData();
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), imF, true);
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }
}
    

void ImageSaveThread::
SaveImage_(Camera <unsigned char>& im,
           Camera <float>& imDepth,
           Camera <float>& imAmp){
  wxString text;

  if (DownSample_ != 0){ 
    Camera<unsigned char> rescaledImage;  
    Rescale<unsigned char, unsigned char> rescale;
    rescale.Downsample(im, rescaledImage,
                       (int)(im.GetWidth() / DownSample_),
                       (int)(im.GetHeight() / DownSample_)); 
    rescaledImage.UpdateMetaData(); 
    text.Printf(wxT("-grey-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImage, true);
        
    Camera<float> rescaledImageF;  
    Rescale<float,float> rescaleF;
    rescaleF.Downsample(imDepth, rescaledImageF,
                        (int)(imDepth.GetWidth() / DownSample_),
                        (int)(imDepth.GetHeight() / DownSample_)); 
    text.Printf(wxT("-depth-%05d.mip"),ImageCounter_);
    rescaledImage.UpdateMetaData();
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImageF, true); 
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;

    rescaleF.Downsample(imAmp, rescaledImageF,
                        (int)(imAmp.GetWidth() / DownSample_),
                        (int)(imAmp.GetHeight() / DownSample_)); 
    text.Printf(wxT("-amp-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    rescaledImage.UpdateMetaData();
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImageF, true); 
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }else{
    im.UpdateMetaData();  
    text.Printf(wxT("-grey-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), im, true);

    imDepth.UpdateMetaData();
    text.Printf(wxT("-depth-%05d.mip"),ImageCounter_);
     FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), imDepth, true);
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;

    imAmp.UpdateMetaData();
    text.Printf(wxT("-amp-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), imAmp, true);
    //    std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }

}
    
void ImageSaveThread::
SaveImage_(Camera <unsigned char>& im,
           Camera <unsigned char>& imC,
           Camera <unsigned char>& imR){
  wxString text;
      
  if (DownSample_ != 0){
    // rescale image and store it to disk          
    Camera<unsigned char> rescaledImage;  
    Rescale<unsigned char, unsigned char> rescale;
    rescale.Downsample(im, rescaledImage,
                       (int)(im.GetWidth() / DownSample_),
                       (int)(im.GetHeight() / DownSample_)); 
    rescaledImage.UpdateMetaData(); 
    text.Printf(wxT("-left-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImage, true);
        
    rescale.Downsample(imC, rescaledImage,
                       (int)(imC.GetWidth() / DownSample_),
                       (int)(imC.GetHeight() / DownSample_)); 
    rescaledImage.UpdateMetaData(); 
    text.Printf(wxT("-center-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImage, true);
        
    rescale.Downsample(imR, rescaledImage,
                       (int)(imR.GetWidth() / DownSample_),
                       (int)(imR.GetHeight() / DownSample_)); 
    rescaledImage.UpdateMetaData(); 
    text.Printf(wxT("-right-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), rescaledImage, true);
            
  }else{
    // save image without rescaling 
        
    im.UpdateMetaData();  
    text.Printf(wxT("-left-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), im, true);
    // std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;

    imC.UpdateMetaData();  
    text.Printf(wxT("-center-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), imC, true);
    // std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;

    imR.UpdateMetaData();  
    text.Printf(wxT("-right-%05d.mip"),ImageCounter_);
    FilenameOut_ = Filename_  + text;
    BIAS::ImageIO::Write(saveDirectory_ +WxToAscii(FilenameOut_), imR, true);
    // std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
  }
  //  std::cout <<"Saved image: "<<WxToAscii(FilenameOut_)<<std::endl;
}



/************************* GrabSingleThread *******************************
 **************************************************************************/


GrabSingleThread::
GrabSingleThread(BIAS::VideoSource* cam,
                 BIAS::Camera<unsigned char>* img,
                 wxCondition *condition,
                 wxMutex* BufferAccessMutex,
                 std::vector<Camera<unsigned char>*> &imageBuffer,
                 bool& writeToBuffer,
                 unsigned int& readIndex,
                 unsigned int& writeIndex,
                 wxMutex *mutexCondition)
  : wxThreadReady(img,condition,BufferAccessMutex,imageBuffer,writeToBuffer,
                  readIndex,writeIndex,mutexCondition)  
{
  Cam_ = cam;
  std::cout<<"GrabSingleThread:"<<endl;
}   
 


void* GrabSingleThread::
Entry()
{ 
  //std::cout<<"Entry in Thread:"<<std::endl;
  while(!TestDestroy()){
    //  std::cout<<"NOT TestDestroy:"<<std::endl;
    if(Cam_ != NULL){
      timeval t1,t2;
      if(*WriteToBuffer_){
        //        std::cout<<"Writing to buffer:"<<std::endl;
        double bufferPercent = 0.0;
        if(*WriteIndex_ >= *ReadIndex_){
          bufferPercent = (double)((*WriteIndex_ + 1) -
                                   (*ReadIndex_ + 1) ) / (double)BufferSize_;
        }else{
          bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                   (*ReadIndex_ + 1)) / (double)BufferSize_;
        }

        while(*WriteToBuffer_ && bufferPercent >BUFFER_CAPACITY_MIN){
              
          if(*WriteIndex_ >= *ReadIndex_){
            bufferPercent = (double)((*WriteIndex_ + 1) -
                                     (*ReadIndex_ + 1) ) / (double)BufferSize_;
          }else{
            bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                     (*ReadIndex_ + 1)) / (double)BufferSize_;
          }            
          //  std::cout << "Pufferfreiraum: " << bufferPercent << std::endl;
          usleep(BUFFER_WRITE_SLEEP);
        }
      }
      gettimeofday(&t1,NULL);        
      Cam_->GrabSingle(*Img_);
      gettimeofday(&t2,NULL);
      long duration = ((t2.tv_sec-t1.tv_sec)*1000000) + (t2.tv_usec-t1.tv_usec);
      long sec=t1.tv_sec + duration/1000000;
      long usec=t1.tv_usec + duration%1000000;
      Img_->SetTime(sec, usec);
    }
      
    // write images to buffer
    if(*WriteToBuffer_){
      // std::cout<<"Writing to buffer:"<<std::endl;
      BufferAccess_->Lock();
          
      //std::cout << "WriteIndex: " << *WriteIndex_ << std::endl;
      if((*WriteIndex_ + 1) % ImageBuffer_->size() != *ReadIndex_ ){
        BufferAccess_->Unlock(); 
        if((*ImageBuffer_)[*WriteIndex_] == NULL){
          // initialize buffer
          //          std::cout << "Init Buffer Normal" << std::endl;
          (*ImageBuffer_)[*WriteIndex_] = new Camera<unsigned char>;
        }
        //std::cout << "Write Image to buffer at pos:  " << *WriteIndex_ << std::endl;
        (*ImageBuffer_)[*WriteIndex_] = Img_;
        BufferAccess_->Lock();
        *WriteIndex_ = (*WriteIndex_ + 1) % ImageBuffer_->size();
        BufferAccess_->Unlock();
      } else {
        BufferAccess_->Unlock();
      //   std::cout << "Camera grab thread overtakes writing Thread, Frames lost.\n"
//                   << "Thread: " << this->GetId() << " sleeps." 
//                   << std::endl;
        usleep(10);
      }
      // BufferAccess_->Unlock();
    }
   //  else
//       std::cout<<"NOT Writing to buffer:"<<endl;
            
       //lock the mutex 
    wxMutexLocker lock(*mutexCondition_);
    // signal the finish
    Condition_->Broadcast(); 
    //    std::cout<< "Thread: " << this->GetId() << " Broadcasts"<<endl;   
  }
  return NULL;
}
    


/************************* GrabSingleThreadSR *******************************
 **************************************************************************/



#ifdef BIAS_HAVE_SWISSRANGER
GrabSingleThreadSR:: 
GrabSingleThreadSR(BIAS::VideoSource_SwissRanger* cam,
                   BIAS::Camera<unsigned char>* img,
                   BIAS::Camera<float>* imgDepth,
                   BIAS::Camera<float>* imgAmp,                   
                   wxCondition *condition,
                   wxMutex* mutex,
                   std::vector<Camera<unsigned char>*> &imageBuffer,
                   std::vector<Camera<float>*> &imageBufferPmdDepth,
                   std::vector<Camera<float>*> &imageBufferPmdAmp,
                   bool& writeToBuffer,
                   unsigned int& readIndex,
                   unsigned int& writeIndex,
                   wxMutex *mutexCondition)
: wxThreadReady(img,condition,mutex,imageBuffer,writeToBuffer,readIndex,writeIndex,mutexCondition)
 
{ 
 
  ImgDepth_ = imgDepth;
  ImgAmp_ = imgAmp;
  Cam_ = cam;
  ImageBufferPMDDepth_ = &imageBufferPmdDepth; 
  ImageBufferPMDAmp_ = &imageBufferPmdAmp;
 
}   

void* GrabSingleThreadSR::
Entry()
{     
  while(!TestDestroy()){       
    if(Cam_ != NULL){
      timeval t1,t2;
      if(*WriteToBuffer_){             
        double bufferPercent = 0.0;
        if(*WriteIndex_ >= *ReadIndex_){
          bufferPercent = (double)((*WriteIndex_ + 1) - 
                                   (*ReadIndex_ + 1) ) / (double)BufferSize_;
        }else{
          bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                   (*ReadIndex_ + 1)) / (double)BufferSize_;
        }

        while(*WriteToBuffer_ && bufferPercent > BUFFER_CAPACITY_MIN){
              
          if(*WriteIndex_ >= *ReadIndex_){
            bufferPercent = (double)((*WriteIndex_ + 1) -
                                     (*ReadIndex_ + 1) ) / (double)BufferSize_;
          }else{
            bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                     (*ReadIndex_ + 1)) / (double)BufferSize_;
          }            
          //std::cout << "Pufferfreiraum: " << bufferPercent << std::endl;
          usleep(BUFFER_WRITE_SLEEP);
        }
      }
      gettimeofday(&t1,NULL);        
      Cam_->GrabSingleDepth(*ImgDepth_);
      Cam_->GrabSingleModCoeff(*ImgAmp_); 
      Cam_->GrabSingle(*Img_);
      gettimeofday(&t2,NULL);
      long duration = ((t2.tv_sec-t1.tv_sec)*1000000) + (t2.tv_usec-t1.tv_usec);
      long sec=t1.tv_sec + duration/1000000;
      long usec=t1.tv_usec + duration%1000000;
      Img_->SetTime(sec, usec);
      ImgDepth_->SetTime(sec, usec);
      ImgAmp_->SetTime(sec, usec);
    } 
    // write images to buffer
    if(*WriteToBuffer_){
              
      BufferAccess_->Lock();
          
      if((*WriteIndex_ + 1) % ImageBuffer_->size() != *ReadIndex_ ){
        BufferAccess_->Unlock(); 
        if((*ImageBuffer_)[*WriteIndex_] == NULL){
          // initialize buffer
          //          std::cout << "Init Buffer SR" << std::endl;
          (*ImageBuffer_)[*WriteIndex_] = new Camera<unsigned char>;
          (*ImageBufferPMDDepth_)[*WriteIndex_] = new Camera<float>;
          (*ImageBufferPMDAmp_)[*WriteIndex_] = new Camera<float>;
        }
        //std::cout << "Write Image to buffer at pos:  " << *WriteIndex_ << std::endl;
        (*ImageBuffer_)[*WriteIndex_] = Img_;
        (*ImageBufferPMDDepth_)[*WriteIndex_] = ImgDepth_;
        (*ImageBufferPMDAmp_)[*WriteIndex_] = ImgAmp_;
        BufferAccess_->Lock();
        *WriteIndex_ = (*WriteIndex_ + 1) % ImageBuffer_->size();
        BufferAccess_->Unlock();
      } else {
        BufferAccess_->Unlock();
        std::cout << "Puffer schreiben ueberholt den auslesenden Thread, Frames gehen verloren.\n"
                  << "Thread: " << this->GetId() << " wird kurz schlafen gelegt." 
                  << std::endl;
            
        usleep(10);
      }
      // BufferAccess_->Unlock();
    }
    wxMutexLocker lock(*mutexCondition_);  
    Condition_->Broadcast(); // signal thread finished catching image    
  }
  return NULL;
}
#endif



/************************* GrabSingleThreadXB3 ****************************
 **************************************************************************/

//#ifndef WIN32
#ifdef BIAS_HAVE_DCAM
GrabSingleThreadXB3::
GrabSingleThreadXB3(BIAS::VideoSource_DCAM_XB3* cam,
                    BIAS::Camera<unsigned char>* imgLeft,
                    BIAS::Camera<unsigned char>* imgMiddle,
                    BIAS::Camera<unsigned char>* imgRight,
                    BIAS::Camera<unsigned char>* allImages,
                    wxCondition* condition,
                    wxMutex* mutex,
                    std::vector<Camera<unsigned char>*> &imageBuffer,
                    std::vector<Camera<unsigned char>*> &imageBufferCenter,
                    std::vector<Camera<unsigned char>*> &imageBufferRight,
                    bool& writeToBuffer,
                    unsigned int& readIndex,
                    unsigned int& writeIndex,
                    wxMutex *mutexCondition)
  : wxThreadReady(imgLeft,condition,mutex,imageBuffer,writeToBuffer,readIndex,writeIndex,mutexCondition)  
{
 
  ImgLeft_ = imgLeft;
  ImgMiddle_ = imgMiddle;
  ImgRight_ = imgRight;
  AllImages_ = allImages;
  Cam_ = cam;
  ImageBufferCenter_ = &imageBufferCenter;
  ImageBufferRight_ = &imageBufferRight;
}   
 
void *GrabSingleThreadXB3::
Entry()
{        
  while(!TestDestroy()){             
    if(Cam_ != NULL){
      timeval t1,t2;
      if(*WriteToBuffer_){ 
        double bufferPercent = 0.0;
        if(*WriteIndex_ >= *ReadIndex_){
          bufferPercent = (double)((*WriteIndex_ + 1) -
                                   (*ReadIndex_ + 1) ) / (double)BufferSize_;
        }else{
          bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                   (*ReadIndex_ + 1)) / (double)BufferSize_;
        }

        while(*WriteToBuffer_ && bufferPercent > BUFFER_CAPACITY_MIN){
              
          if(*WriteIndex_ >= *ReadIndex_){
            bufferPercent = (double)((*WriteIndex_ + 1) - 
                                     (*ReadIndex_ + 1) ) / (double)BufferSize_;
          }else{
            bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                     (*ReadIndex_ + 1)) / (double)BufferSize_;
          }            
          // std::cout << "Pufferfreiraum: " << bufferPercent << std::endl;
          usleep(BUFFER_WRITE_SLEEP);
        }
      }
      gettimeofday(&t1,NULL);        
      Cam_->GrabSingle(*AllImages_,*ImgLeft_,*ImgMiddle_,*ImgRight_);
      gettimeofday(&t2,NULL);
      long duration = ((t2.tv_sec-t1.tv_sec)*1000000) + (t2.tv_usec-t1.tv_usec);
      long sec=t1.tv_sec + duration/1000000;
      long usec=t1.tv_usec + duration%1000000;
      ImgLeft_->SetTime(sec, usec);
      ImgMiddle_->SetTime(sec, usec);
      ImgRight_->SetTime(sec, usec);
    }
    // write images to buffer
    if(*WriteToBuffer_){
    
      BufferAccess_->Lock();
          
      if((*WriteIndex_ + 1) % ImageBuffer_->size() != *ReadIndex_ ){
        BufferAccess_->Unlock(); 
        if((*ImageBuffer_)[*WriteIndex_] == NULL){
          // initialize buffer
          //          std::cout << "Init Buffer XB3" << std::endl;
          (*ImageBuffer_)[*WriteIndex_] = new Camera<unsigned char>;
          (*ImageBufferCenter_)[*WriteIndex_] = new Camera<unsigned char>;
          (*ImageBufferRight_)[*WriteIndex_] = new Camera<unsigned char>;
        }
        //std::cout << "Write Image to buffer at pos:  " << *WriteIndex_ << std::endl;
        (*ImageBuffer_)[*WriteIndex_] = ImgLeft_;
        (*ImageBufferCenter_)[*WriteIndex_] = ImgMiddle_;
        (*ImageBufferRight_)[*WriteIndex_] = ImgRight_;
        BufferAccess_->Lock();
        *WriteIndex_ = (*WriteIndex_ + 1) % ImageBuffer_->size();
        BufferAccess_->Unlock();
      } else {
        BufferAccess_->Unlock();
//         std::cout << "Puffer schreiben ueberholt den auslesenden Thread, Frames gehen verloren.\n"
//                   << "Thread: " << this->GetId() << " wird kurz schlafen gelegt." 
//                   << std::endl;            
        usleep(10);
      }
      // BufferAccess_->Unlock();
    }

   
   
    wxMutexLocker lock(*mutexCondition_);
    Condition_->Broadcast();
    
  }
  return NULL;
}

#endif //DCAM
//#endif //NOT win32

/************************* GrabSingleThreadPMDZess ****************************
 **************************************************************************/


#ifdef BIAS_HAVE_PMD_PMDZess
GrabSingleThreadPMDZess::
GrabSingleThreadPMDZess(BIAS::VideoSource_usbPMD* cam,
                        BIAS::Camera<unsigned char> *img2D,
                        BIAS::Camera<unsigned char>* img,
                        BIAS::Camera<float>* imgDepth,
                        BIAS::Camera<float>* imgAmp,
                        wxCondition *condition,
                        wxMutex* BufferAccessMutex,
                        std::vector<Camera<unsigned char>*> &imageBuffer2d,
                        std::vector<Camera<unsigned char>*> &imageBuffer,
                        std::vector<Camera<float>*> &imageBufferPmdDepth,
                        std::vector<Camera<float>*> &imageBufferPmdAmp,
                        bool& writeToBuffer,
                        unsigned int& readIndex,
                        unsigned int& writeIndex,
                        wxMutex *mutexCondition)
  : wxThreadReady(img,condition,BufferAccessMutex,imageBuffer,writeToBuffer,readIndex,writeIndex,mutexCondition)
  
{
  ImgDepth_ = imgDepth;
  ImgAmp_ = imgAmp;
  Img2D_ = img2D;
  Cam_ = cam;
  ImageBuffer2D_ = &imageBuffer2d;
  ImageBufferPMDDepth_ = &imageBufferPmdDepth; 
  ImageBufferPMDAmp_ = &imageBufferPmdAmp;
}   

void *GrabSingleThreadPMDZess::
Entry()
{         
  while(!TestDestroy()){                 
    if(Cam_ != NULL){
      timeval t1,t2;
      if(*WriteToBuffer_){ 
        double bufferPercent = 0.0;
        if(*WriteIndex_ >= *ReadIndex_){
          bufferPercent = (double)((*WriteIndex_ + 1) -
                                   (*ReadIndex_ + 1) ) / (double)BufferSize_;
        }else{
          bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                   (*ReadIndex_ + 1)) / (double)BufferSize_;
        }

        while(*WriteToBuffer_ && bufferPercent > BUFFER_CAPACITY_MIN){
              
          if(*WriteIndex_ >= *ReadIndex_){
            bufferPercent = (double)((*WriteIndex_ + 1) - 
                                     (*ReadIndex_ + 1) ) / (double)BufferSize_;
          }else{
            bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                     (*ReadIndex_ + 1)) / (double)BufferSize_;
          }            
          std::cout << "Pufferfreiraum: " << bufferPercent << std::endl;
          usleep(BUFFER_WRITE_SLEEP);
        }
      }
      gettimeofday(&t1,NULL);   
      Cam_->GrabSingle(*Img_); //grabs the image and stores depth an dmod image in variables
      Cam_->GrabSingleDepth(*ImgDepth_);
      Cam_->GrabSingleModCoeff(*ImgAmp_); 
      //  std::cout<<"Grab Thread PMDZess:"<<std::endl;
      if(Img2D_ != NULL && Cam_->GrabSingle2D(*Img2D_) == -1){
        BIASERR("No 2D Image");      
      }     
      gettimeofday(&t2,NULL);
      long duration = ((t2.tv_sec-t1.tv_sec)*1000000) + (t2.tv_usec-t1.tv_usec);
      long sec=t1.tv_sec + duration/1000000;
      long usec=t1.tv_usec + duration%1000000;
      Img_->SetTime(sec, usec);
      ImgDepth_->SetTime(sec, usec);
      ImgAmp_->SetTime(sec, usec);
      Img2D_->SetTime(sec,usec);
    }
    if(*WriteToBuffer_){
                
      BufferAccess_->Lock();
          
      if((*WriteIndex_ + 1) % ImageBuffer_->size() != *ReadIndex_ ){
        BufferAccess_->Unlock(); 
        if((*ImageBuffer_)[*WriteIndex_] == NULL){
          // initialize buffer
          //          std::cout << "Init Buffer PMDZess" << std::endl;
          (*ImageBuffer_)[*WriteIndex_] = new Camera<unsigned char>;
          (*ImageBuffer2D_)[*WriteIndex_] = new Camera<unsigned char>;
          (*ImageBufferPMDDepth_)[*WriteIndex_] = new Camera<float>;
          (*ImageBufferPMDAmp_)[*WriteIndex_] = new Camera<float>;
        }
        //std::cout << "Write Image to buffer at pos:  " << *WriteIndex_ << std::endl;
        (*ImageBuffer_)[*WriteIndex_] = Img_;
        (*ImageBuffer2D_)[*WriteIndex_] = Img2D_;
        (*ImageBufferPMDAmp_)[*WriteIndex_] = ImgAmp_;
        (*ImageBufferPMDDepth_)[*WriteIndex_] = ImgDepth_;

        BufferAccess_->Lock();
        *WriteIndex_ = (*WriteIndex_ + 1) % ImageBuffer_->size();
        BufferAccess_->Unlock();
      } else {
        BufferAccess_->Unlock();
        std::cout << "Puffer schreiben ueberholt den auslesenden Thread, "
                  <<"Frames gehen verloren.\n"
                  << "Thread: " << this->GetId() << " wird kurz schlafen gelegt." 
                  << std::endl;
            
        usleep(10);
      }
      // BufferAccess_->Unlock();
    }
    
    wxMutexLocker lock(*mutexCondition_);
    Condition_->Broadcast();
    //    std::cout<< "Thread: " << this->GetId() << " Broadcasts"<<endl;   
  }
  return NULL;
}
    

#endif
