// must be first:
//#include <Base/Common/LeakChecking.h> 

#include <Base/Image/ImageIO.hh>
#include "Base/Image/ImageConvert.hh"
#include "Utils/Param.hh"
#include <stdio.h>
#include <iostream>

using namespace BIAS;
using namespace std;



/** @class biasaddchannel
    @ingroup g_tools
    @brief tool for combining images channel-wise
    @author evers */

int main(int argc, char *argv[]){
  // Dart Testing (JW)
  if (argc>=2) if (strcmp(argv[1],"-darttest")==0){
    std::cout<<"started dart test: "<<argv[0]<<std::endl; 
    return 0;
  }

    
  Param param;
  bool *planar = param.AddParamBool("planar","use planar instead of interleaved",false, 'p');
  int next = param.ParseCommandLine(argc, argv);
  if (next<0) exit(1);
  param.DisableDestructorWarning();
  
  if (argc<3){
    param.Usage();
    return -1;
  }

  ImageBase im1, im2, diff;
  if (ImageIO::Load(argv[next], im1)!=0){
    BIASERR("error loading "<<argv[next]);
    return -1;
  }
  if (ImageIO::Load(argv[next+1], im2)!=0){
    BIASERR("error loading "<<argv[next+1]);
    return -1;
  }

  if (im1.GetWidth() != im2.GetWidth() ||
      im1.GetHeight() != im2.GetHeight()){
    BIASERR("image sizes do not match");
    return -2;
  }



  string newfilename;
  string fn1,fn2;
  fn1 = argv[next];
  fn2 = argv[next+1];
  newfilename = argv[next+2];

  Image<float> fim1, fim2, fdiff;
  Image<unsigned char> ucim1, ucim2, ucdiff;


  switch (im1.GetStorageType()){
  case ImageBase::ST_unsignedchar:
    ucim1=im1;
    ucim2=im2;
    fim1.SetInterleaved(!(*planar));
    ucim1.AppendChannel(ucim1);
    ucim1.SetColorModel(ImageBase::CM_Grey);
    if (ImageIO::Write(newfilename, ucim2)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
    break;
  case ImageBase::ST_float:
    fim1=im1;
    fim2=im2;
    fim1.SetInterleaved(!(*planar));
    ucim1.SetColorModel(ImageBase::CM_Grey);
    fim1.AppendChannel(fim2);
 
 
    if (ImageIO::Write(newfilename, fim1)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
   
    break;
  default:
    cerr << "unknown storage type\n";
    break;
  }

  return 0;
}
