#include <Base/Debug/Error.hh>
#include <Base/Image/ImageIO.hh>
#include <Utils/Param.hh>
#include <iostream>

using namespace BIAS;
using namespace std;

/** @class biasdiv
    @ingroup g_tools
    @brief tool for computing pixelwise division of two images 
    @author woelk (vision^n) 09/2007 */
int main(int argc, char ** argv)
{
  Param param;
  param.AddParamBool("help","",false, 'h');
  param.AddParamBool("ignoreNegative", "do not use negative values", 
                     false,'n');
  param.DisableDestructorWarning();

  int next = param.ParseCommandLine(argc, argv);
  if (argc<4 || *param.GetParamBool("help")){
    param.Usage();
    if (argc>0) cerr << argv[0] <<" <image 1> <image 2> <outfile> \n";
    cerr << "   divides image 1 by image 2 (pixelwise) and writes the result "
         <<"to outfile"<<endl;    
    return 0; // for Dart testing
  }

  ImageBase im1, im2, result;
  if (ImageIO::Load(argv[next], im1)!=0){
    BIASERR("error loading "<<argv[next]);
    return -1;
  }
  if (ImageIO::Load(argv[next+1], im2)!=0){
    BIASERR("error loading "<<argv[next+1]);
    return -1;
  }
  if (!im1.SamePixelAndChannelCount(im2)) {
    BIASERR("image size,depth,or ST does not match");
    exit(-2);
  }
  BIASASSERT(im1.GetStorageType()==im2.GetStorageType());
  BIASASSERT(im1.GetWidth()==im2.GetWidth());
  BIASASSERT(im1.GetHeight()==im2.GetHeight());

  switch (im1.GetStorageType()){
  case ImageBase::ST_float: 
    {
      Image<float> fim1(im1), fim2(im2), div;
      div = fim1 / fim2;
      
      if (*param.GetParamBool("ignoreNegative")){
        cout << "ignoring negative\n";
        float *i1=fim1.GetImageData();
        float *i2=fim2.GetImageData();
        float *d=div.GetImageData();
        float *end = i1 + fim1.GetPixelCount() * fim1.GetChannelCount();
        // ignore negative pixel
        do {
          if ( (*i1)<=0.0 || (*i2) <=0.0) *d = 0.0;
          i1++; i2++; d++;
        } while (i1<end);
      
        if (ImageIO::Save(string(argv[next+2]), div)!=0){
          BIASERR("error writing "<<argv[next+2]);
        }
      }
    }
    break;
  default:
    cerr << "storage type not supported \n";
    break;
  }
  return 0;
}
