#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <FeatureDetector/CheckerboardDetector.hh>
#include <iostream>
#include <fstream>

#ifndef  WIN32
#include <unistd.h>
#else  //WIN32
#include "Base/Common/getopt_W32.h"
#endif //WIN32

using namespace std;
using namespace BIAS;


/** @class biasextractcheckerboard
@author streckel 08/2006
@ingroup g_tools
@brief The tool tries to find checkerboards in the input images
finds corners also in fisheye distorted images andwrites these 
to the Matlab file 'Filename.m'.
*/


void usage()
{
  cout <<"Usage: biasextractcheckerboard -f Filename.m -x xnum -y ynum -d image1 image2 ..."<<endl
    <<endl;
  cout <<" The tool tries to find checkerboards in the input images" << endl
    <<" finds corners also in fisheye distorted images and " << endl
    <<" writes these to the Matlab file 'Filename.m'." << endl
    <<" xnum and ynum are the number of the checkerboards inner corners." 
    << endl
    <<" -d writes debug images." << endl
    <<" Do not use .mip images since Matlab is unable to read them." << endl
    <<endl<<endl;
}


int main(int argc, char *argv[])
{

  string MatlabFilename = "ExtractedCorners.m";
  int x_corners = 7;
  int y_corners = 4;
  bool writedebug = true;

  int c = 1;  
  while(c >0) {
    c = char(getopt(argc,argv,"f:x:y:hd"));
    switch (c) {
    case 'h':
    case ':':
    case '?':
      usage();
      break;
    case 'x':
      x_corners = atoi(optarg);
      break;
    case 'y':
      y_corners = atoi(optarg);
      break;
    case 'f':
      MatlabFilename = optarg;
      break;
    case 'd':
      writedebug=true;
      break;
    default:
      if (c>0){
        usage();
        exit(1);
      }
    }
  }

  if (argc-optind < 1) {
    cout <<"==> To few arguments."<<endl;
    usage();
    exit(1);
  }

  BIAS::Image<unsigned char> img;
  vector<string> filenames, bad_filenames;
  vector<vector<pair<float,float> > > all_coords;
  for (int i=optind; i<argc; i++) {

    if (ImageIO::Load(argv[i], img)!=0) {
      BIASERR("Could not load "<<argv[i]); BIASBREAK;
      return -1;
    }
    std::vector< std::pair<float,float> > coords;
    int result = CheckerboardDetector::Compute(img, x_corners, y_corners, 
      coords, writedebug);
    if (result==0) {
      cout << "Sucess:  " << argv[i]<< endl;
      all_coords.push_back(coords);
      filenames.push_back(argv[i]);
    } else {
      cout << "Failure: " << argv[i]<< endl;
      bad_filenames.push_back(argv[i]);
    }

  }
  ofstream ofs(MatlabFilename.c_str());

  ofs << "numpoints = [" << x_corners << " " << y_corners << "];" << endl;

  ofs << "imagenames = {";
  for (unsigned int i=0; i<filenames.size(); i++) {
    ofs << "'" << filenames[i] << "' ";
  }
  ofs << "};" << endl;

  if (bad_filenames.size()>0) {
    ofs << "bad_imagenames = {";
    for (unsigned int i=0; i<bad_filenames.size(); i++) {
      ofs << "'" << bad_filenames[i] << "' ";
    }
    ofs << "};" << endl;
  }    

  ofs << "xpos = [";
  for (unsigned int i=0; i<all_coords.size(); i++) {
    for (unsigned int j=0; j<all_coords[i].size(); j++) {
      ofs << (all_coords[i][j]).first << " ";      
    }
    ofs << ";" << endl;
  }
  ofs << "];" << endl;

  ofs << "ypos = [";
  for (unsigned int i=0; i<all_coords.size(); i++) {
    for (unsigned int j=0; j<all_coords[i].size(); j++) {
      ofs << (all_coords[i][j]).second << " ";      
    }
    ofs << ";" << endl;
  }
  ofs << "];" << endl;

  ofs.close();

  return 0;
}

