
#include <Base/Image/ImageIO.hh>
#include <Image/Histogram.hh>
#include <Gui/GuiGTK.hh>

using namespace BIAS;
using namespace std;

Histogram hist;
bool loga = false;
bool ignore_negative = true;
Image<float> imf;

void Draw()
{
  hist.ZeroHist(0);
  if (ignore_negative){
    vector<float> v;
    float *id = imf.GetImageData();
    float *end = id + imf.GetPixelCount();
    while (id<end){
      if (*id>=0.0) v.push_back(*id);
      id++;
    }
    hist.AddHist(v);
  } else {
    hist.AddHist(imf);
  }
  hist.FillImageWithConstValue(255);
  if (loga) hist.DrawLog();
  else hist.Draw();
}


void Help()
{
  cout << "+/-  : increase/decrease bin count\n"
       << "m    : toggle drawing mode\n"
       << "l    : toggle logarithmic scale for y axis\n"
       << "n    : toggle clipping of negative values\n"
       << "w    : write histogram to ascii file \"hist.txt\"\n"
       << "q    : quit\n"
       << "h    : display this help\n";
}

int main(int argc, char *argv[])
{
  const int num_bins = 200;
  const int num_hist = 1;
  hist.InitHist(num_bins, num_hist);
  hist.SetColor(0,0,0,0);
  hist.AddDebugLevel("D_HIST_BIN");
  ImageBase im;
  
  GuiGTK gui;

  if (argc!=2){
    cerr << argv[0] << " <image>\n";
    return -1;
  }
  
  if (ImageIO::Load(argv[1], im)!=0){
    cerr << "error loading image "<<argv[1]<<endl;
    return -2;
  }

  imf = Image<float>(im);
  if (imf.IsEmpty()){
    cerr << "only for float images"<<endl;
    return -3;
  }
  hist.AddHist(imf);
  Draw();

  gui.ShowImage(hist);
  int bin_count = hist.GetBinCount();
  int bin_step = 5;
  int mode = 0;
  bool quit = false;
  char key = 0;
  while (!quit){
    key = gui.WaitForKeyEvent();
    switch (key){
    case '-':
      bin_count-=bin_step;
      if (bin_count<1) bin_count=1;
      cout << "bin count: "<<bin_count<<endl;
      hist.SetBinCount(bin_count);
      Draw();
      break;
    case '+':
      bin_count+=bin_step;
      if (bin_count<1) bin_count=1;
      cout << "bin count: "<<bin_count<<endl;
      hist.SetBinCount(bin_count);
      Draw();
      break;
    case 'w':
      hist.WriteASCII("hist.txt");
      break;
    case 'l':
      loga = !loga;
      Draw();
      cout << "logarithmic scale of y axis = "<<boolalpha<<loga<<endl;
      break;
    case 'm':
      mode++;
      if (mode>2) mode=0;
      hist.SetDrawMode((HistMode)mode);
      Draw();
      break;
    case 'n':
      ignore_negative = !ignore_negative;
      Draw();
      break;
    case 'h':
      Help();
      break;
    case 27: 
    case 'q': quit = true; break;
    default:
      cout << (int)key<<endl;
      break;
    }
    gui.ShowImage(hist);
  }

  return 0;
}
