#include <Utils/GenSynthMatches.hh>
#include <Gui/GuiEnhanced.hh>
#include <Base/Image/ImageIO.hh>

#define SPEED_STEP 10000

using namespace BIAS;
using namespace std;

void usage(char *name)
{
  cout << name << " <binary match data> \n"
       << "   visualize matches from binary files\n";
}

void help(char *name)
{
  usage(name);
  cout << ",/.         decrease/increase speed of automatic slideshow\n"
       << "s           toggle automatic slideshow on/off\n"
       << "q/esc       quit\n"
       << "space       visualize next match data\n"
       << "backspace   visualize previous match data\n"
       << "w           write visualization image to disk (\"true.mip\", "
       <<           "\"noisy.mip\")\n"
       << "i           print match data to stdout\n"
       << "t           toggle display of ground truth/noisy matches\n"
       << "h           show this help\n";
}

/** @class biasshowsm
    @ingroup g_tools
    @brief tool for showing binary written GensSynthMatches
    @author wopelk 08/2004 */
int main(int argc, char *argv[])
{
  Image<unsigned char> noisyim, trueim, zoomimage;
  Param para;
  GenSynthMatches gsm(para);
  MatchDataBase mdb(para);
  bool gt=false, showtrue=true;
  GuiEnhanced gui;
  GuiGTK zoomgui;
  int numimages, index=1, lastindex=0;
  string title;
  string::size_type pos;
  const unsigned zoomfactor=5;
  const unsigned zoomsize=25;
  bool finished=false, zoomed=false, KeyEvent=false, Pressed=false, 
    slideshow=false;
  char key=0;
  int Xpos=0, Ypos=0, Button=0;
  long int speed = 10001;
   
  if (argc<2){
    usage(argv[0]);
    return -1;
  }
  while (!finished){
    if (index!=lastindex){
      if (zoomed){
        zoomgui.DestroyWindow();
        zoomed=false;
      }
      if (gsm.Read(argv[index])!=0){
        if (mdb.Read(argv[index])==0){
          cout << "without ground truth information\n";
          gt=false;
          gsm=mdb;
        } else {
          cerr << "error reading match data "<<argv[index]<<endl;
          usage(argv[0]);
          return -2;
        }
      } else { // gsm
        cout << "with ground truth information\n";
        gt=true;
      }

      numimages=gsm.GetNumImages();
      gsm.DrawNoisy(noisyim, 0, numimages-1);
      if (gt)
        gsm.DrawTrue(trueim, 0, numimages-1);
      lastindex=index;
    }
    title=argv[index];
    pos = title.rfind('/'); 
    if (pos!=std::string::npos) title = title.substr(pos+1, title.size()-1 );
    
    if (gt && showtrue) {
      gui.ShowImage(trueim);
      title ="ground truth: "+title;
    } else {
      gui.ShowImage(noisyim);
      title ="noisy: "+title;
    }
    gui.SetTitle(title);
    
    if (slideshow){
      key=-1;
      key=gui.CheckForKeyEvent();
      if (key==-1){
        key=' ';
        usleep(speed);
      }
      KeyEvent=true;
    } else {
      gui.WaitForEvent(KeyEvent, key, Pressed, Button, Xpos, Ypos);
    }
    if (KeyEvent){
      switch (key){
      case ',':
        if (speed - SPEED_STEP > 0)
          speed -= SPEED_STEP;
        cerr << "speed = "<<speed<<endl;
        break;
      case '.':
        speed += SPEED_STEP;
        cerr << "speed = "<<speed<<endl;
        break;
      case 's':
        slideshow=!slideshow;
        cerr << "slideshow "<<boolalpha << slideshow<<endl;
        break;
      case 'q':
      case 27 : // esc
        finished=true;
        break;
      case ' ':
        index++;
        if (index==argc) index=1;
        break;
      case '8': // backspace
        index--;
        if (index<=1) index=argc-1;
        break;
      case 'i':
        cout << argv[index] << ":"<<endl;
        cout << gsm << endl;
        break;
      case  'w':
        if (gt && showtrue){ 
          ImageIO::Write("true.mip", trueim);
          cout << "wrote \"true.mip\"\n";
        } else {
          ImageIO::Write("noisy.mip", trueim);
          cout << "wrote \"noisy.mip\"\n";
        }
        break;
      case 't':
        showtrue=!showtrue;
        if (showtrue)
          cout << "showing ground truth matches\n";
        else 
          cout << "showing noisy matches\n";
        break;
      case 'h':
        help(argv[0]);
        break;
      default: 
        cerr << "key "<<(int)key<<endl;
        break;
      }
    } else { // mouse event
      if (Button==3 && Pressed){ // right mouse button pressed
        gui.GetZoomImage(Xpos, Ypos, zoomfactor, zoomsize, zoomimage);
        zoomed=true;
        zoomgui.SetFactorOffset(Xpos-zoomsize, Ypos-zoomsize, zoomfactor);
        zoomgui.ShowImage(zoomimage);
      }
    }
  }
  return 0;
}
