/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "VideoSource_DCAM_XB3.hh"

#ifdef WIN32

#include <wtypes.h>
#include <1394Camera.h>
#include <Base/Debug/TimeMeasure.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>

#else

#include <libdc1394/dc1394_control.h>
#include "sys/time.h"
#include "unistd.h"
#include <sys/stat.h>

#endif // WIN32

#define WIDTH 1280
#define HEIGHT 960

using namespace BIAS;
using namespace std;

int j = 0;


VideoSource_DCAM_XB3::VideoSource_DCAM_XB3(int Card): VideoSource_DCAM(Card)
{
}

int VideoSource_DCAM_XB3::GrabSingle(Camera <unsigned char> &allImages,
                                     Camera <unsigned char> &imageLeft,
                                     Camera <unsigned char> &imageMiddle,
                                     Camera <unsigned char> &imageRight)
{
  // fetch complete image from cam

  VideoSource_DCAM::GrabSingle(allImages);

  // create 3 images out of source image
  unsigned char* grabedPixels = allImages.GetImageData();
  unsigned char* sinkPixels = NULL;

  for(unsigned int j=0; j<3; j++) {
      if(j == 0){
        sinkPixels = imageLeft.GetImageData();
      }
      if(j == 1){
        sinkPixels = imageMiddle.GetImageData();
      }
      if(j == 2){
        sinkPixels = imageRight.GetImageData();
      }
      for(unsigned int h=0; h<HEIGHT; h++) {
        for(unsigned int w=0; w<WIDTH; w++) {
          sinkPixels[h*WIDTH + w] = grabedPixels[h*WIDTH*3 + w*3 + j]; 
        }
      }
    }
  return 0;
}
