#include "VideoSource_DSHOW.hh"
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <stdio.h>


using namespace BIAS;
using namespace std;




VideoSource_DSHOW::VideoSource_DSHOW(bool bShowDialogs): VideoSource()
{
  dWidth_                 = 640;
	dHeight_                = 480;
	dDepth_                 = 1;
	dDsBufferSize_          = 0;
  pDsBuffer_              = NULL;
	pMediaControl_          = NULL;
	pMediaEvent_            = NULL;
	pGraphBuilder_          = NULL;
  pCaptureGraphBuilder2_  = NULL;
	pGrabber_               = NULL;
	pNullRenderer_          = NULL;
  bShowDialogs_           = bShowDialogs;
  pImageBuffer_           = NULL;
  sCaptureDeviceName_     = "";
  BIASColorModel_         = BIAS::ImageBase::CM_BGR;
  MediaSubTypeColorModel_ = MEDIASUBTYPE_None;
  FGCB                    = NULL; //callback pointer
  SetSizeAndColorModel(dWidth_,dHeight_,dDepth_,BIASColorModel_);
}

VideoSource_DSHOW::VideoSource_DSHOW(int width, int height,int depth,bool bShowDialogs): VideoSource()
{
  dWidth_                 = width;
	dHeight_                = height;
	dDepth_                 = depth;
	dDsBufferSize_          = 0;
  pDsBuffer_              = NULL;
	pMediaControl_          = NULL;
	pMediaEvent_            = NULL;
	pGraphBuilder_          = NULL;
  pCaptureGraphBuilder2_  = NULL;
	pGrabber_               = NULL;
	pNullRenderer_          = NULL;
  bShowDialogs_           = bShowDialogs;
  pImageBuffer_           = NULL;
  sCaptureDeviceName_     = "";
  BIASColorModel_         = BIAS::ImageBase::CM_BGR;
  MediaSubTypeColorModel_ = MEDIASUBTYPE_None;
  FGCB                    = NULL; //callback pointer
  SetSizeAndColorModel(dWidth_,dHeight_,dDepth_,BIASColorModel_);
}


VideoSource_DSHOW::~VideoSource_DSHOW()
{
  if(pImageBuffer_!=NULL)
    delete [] pImageBuffer_;
  if(pDsBuffer_!=NULL)
    delete [] pDsBuffer_;
  if(FGCB != NULL)
    delete FGCB;
}

int VideoSource_DSHOW::InitImage(BIAS::ImageBase &Image){
  if (!Image.IsEmpty())
    BIASERR("VideoSource::InitImage() should be called with uninitialized image");
  
  Image.Init(GetWidth(), GetHeight(),GetColorChannels());
  Image.SetColorModel(BIASColorModel_);
  
  return 0;
}


int VideoSource_DSHOW::OpenDevice(const char * device)
{
  sCaptureDeviceName_ = device;
  InitVideoCapture_();
  return 0;
}

int VideoSource_DSHOW::OpenDevice(unsigned int width, unsigned int height, unsigned int channels,
    const char * device,BIAS::ImageBase::EColorModel model/* = BIAS::ImageBase::CM_RGB*/)
{
  sCaptureDeviceName_ = device;
  SetSize(width,height,channels);
  SetColorModel(model);
  InitVideoCapture_();
  return 0;
}
 
int VideoSource_DSHOW::CloseDevice()
{
  pGrabber_.Release(); 
  pNullRenderer_.Release();
  pSrcFilter_.Release();
  pMediaControl_->Stop();
  pMediaControl_.Release();
  pMediaEvent_.Release();
  pGraphBuilder_.Release();
  pCaptureGraphBuilder2_.Release();
  return 0;
}


int VideoSource_DSHOW::GrabSingle(BIAS::Camera <unsigned char> &image)
{
  image.CopyIn_NoInit(FGCB->camera_.GetImageData());
  return 0;
}


void VideoSource_DSHOW::DeleteMediaType_(AM_MEDIA_TYPE *pmt)
{
    // allow NULL pointers for coding simplicity

    if (pmt == NULL) {
        return;
    }

    if (pmt->cbFormat != 0) {
        CoTaskMemFree((PVOID)pmt->pbFormat);

        // Strictly unnecessary but tidier
        pmt->cbFormat = 0;
        pmt->pbFormat = NULL;
    }
    if (pmt->pUnk != NULL) {
        pmt->pUnk->Release();
        pmt->pUnk = NULL;
    }

    CoTaskMemFree((PVOID)pmt);
}


int VideoSource_DSHOW::PreGrab()
{
  return 0;
}

int VideoSource_DSHOW::PostGrab()
{
	return 0;
}

bool VideoSource_DSHOW::DisplayPinProperties_( CComPtr<IPin> pSrcPin )
{  
	CComPtr<ISpecifyPropertyPages> pPages;

	HRESULT hr = pSrcPin->QueryInterface(IID_ISpecifyPropertyPages, (void**)&pPages);
	if (SUCCEEDED(hr))
	{
		PIN_INFO PinInfo;
		pSrcPin->QueryPinInfo(&PinInfo);

		CAUUID caGUID;
		pPages->GetPages(&caGUID);

		OleCreatePropertyFrame(
			NULL,
			0,
			0,
			L"Property Sheet",
			1,
			(IUnknown **)&(pSrcPin),
			caGUID.cElems,
			caGUID.pElems,
			0,
			0,
			NULL);

		CoTaskMemFree(caGUID.pElems);
		PinInfo.pFilter->Release();
	}
	return false;
}

bool VideoSource_DSHOW::DisplayFilterProperties( CComPtr<IBaseFilter> pFilter)
{
	CComPtr<ISpecifyPropertyPages> pProp;
	HRESULT hr = pFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
	if (SUCCEEDED(hr)) 
	{
	    // Get the filter's name and IUnknown pointer.
	    FILTER_INFO FilterInfo;
	    pFilter->QueryFilterInfo(&FilterInfo); 

	    // Show the page. 
	    CAUUID caGUID;
	    pProp->GetPages(&caGUID);
	    OleCreatePropertyFrame(
		    NULL,                   // Parent window
			0, 0,                   // (Reserved)
			FilterInfo.achName,     // Caption for the dialog box
			1,                      // Number of objects (just the filter)
			(IUnknown **)&pFilter,  // Array of object pointers. 
	        caGUID.cElems,          // Number of property pages
		    caGUID.pElems,          // Array of property page CLSIDs
			0,                      // Locale identifier
			0, NULL);               // Reserved

        // Clean up.
	    if(FilterInfo.pGraph != NULL) 
            FilterInfo.pGraph->Release(); 
        CoTaskMemFree(caGUID.pElems);
        return( true );
	}
    else
        return( false );
}

void VideoSource_DSHOW::DisplaySourcePinProperties()
{
  if(pOutPin_)
  {
      DisplayPinProperties_(pOutPin_);
  }
}


bool VideoSource_DSHOW::DisplayCameraProperties()
{
  CComPtr<IBaseFilter> pFilter = pSrcFilter_;
	CComPtr<ISpecifyPropertyPages> pProp;
	HRESULT hr = pFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
	if (SUCCEEDED(hr)) 
	{
	    // Get the filter's name and IUnknown pointer.
	    FILTER_INFO FilterInfo;
	    pFilter->QueryFilterInfo(&FilterInfo); 

	    // Show the page. 
	    CAUUID caGUID;
	    pProp->GetPages(&caGUID);
	    OleCreatePropertyFrame(
		    NULL,                   // Parent window
			0, 0,                   // (Reserved)
			FilterInfo.achName,     // Caption for the dialog box
			1,                      // Number of objects (just the filter)
			(IUnknown **)&pFilter,  // Array of object pointers. 
	        caGUID.cElems,          // Number of property pages
		    caGUID.pElems,          // Array of property page CLSIDs
			0,                      // Locale identifier
			0, NULL);               // Reserved

        // Clean up.
	    if(FilterInfo.pGraph != NULL) 
            FilterInfo.pGraph->Release(); 
        CoTaskMemFree(caGUID.pElems);
        return( true );
	}
    else
        return( false );
}

///////////////////////////////////////////////////////////////
void VideoSource_DSHOW::DetermineColorModel_()
{
  if(BIASColorModel_ != BIAS::ImageBase::CM_invalid)
  {
    if(BIASColorModel_ == BIAS::ImageBase::CM_RGB)
      MediaSubTypeColorModel_ = MEDIASUBTYPE_RGB24;
    if(BIASColorModel_ == BIAS::ImageBase::CM_RGBA)
      MediaSubTypeColorModel_ = MEDIASUBTYPE_RGB32;
    if(BIASColorModel_ == BIAS::ImageBase::CM_Grey)
      MediaSubTypeColorModel_ = MEDIASUBTYPE_RGB1;
    if(BIASColorModel_ == BIAS::ImageBase::CM_UYVY422)
      MediaSubTypeColorModel_ = MEDIASUBTYPE_UYVY;
    if(BIASColorModel_ == BIAS::ImageBase::CM_YUV420P)
      MediaSubTypeColorModel_ = MEDIASUBTYPE_Y41P;
  }
  else
    MediaSubTypeColorModel_ = MEDIASUBTYPE_None;
}

void VideoSource_DSHOW::DetermineBIASColorModel_()
{
  if(MediaSubTypeColorModel_ != MEDIASUBTYPE_None)
  {
    if(MediaSubTypeColorModel_ == MEDIASUBTYPE_RGB24 || MediaSubTypeColorModel_== MEDIASUBTYPE_RGB32)
      BIASColorModel_ = BIAS::ImageBase::CM_BGR;
    else if(MediaSubTypeColorModel_ == MEDIASUBTYPE_RGB1)
      BIASColorModel_ = BIAS::ImageBase::CM_Grey;   
    else if(MediaSubTypeColorModel_ == MEDIASUBTYPE_UYVY)
      BIASColorModel_ = BIAS::ImageBase::CM_UYVY422;
    else if(MediaSubTypeColorModel_ == MEDIASUBTYPE_Y41P)
      BIASColorModel_ = BIAS::ImageBase::CM_YUV420P;
    else if(MediaSubTypeColorModel_ == MEDIASUBTYPE_IYUV)
      BIASColorModel_ = BIAS::ImageBase::CM_YUV411;
    else if(MediaSubTypeColorModel_ ==MEDIASUBTYPE_Y411)
      BIASColorModel_ = BIAS::ImageBase::CM_YUV411;
    else if(MediaSubTypeColorModel_ ==MEDIASUBTYPE_Y211)
      BIASColorModel_ = BIAS::ImageBase::CM_YUV420P;
    else
      BIASColorModel_ = BIAS::ImageBase::CM_YUV420P;
  }
  else
    BIASColorModel_ = BIAS::ImageBase::CM_RGB;
}

HRESULT VideoSource_DSHOW::InitVideoCapture_()
{
    HRESULT hr;
    // initialize the COM library.    
	  CoInitialize(0);

    // Get DirectShow interfaces
    hr = GetInterfaces_();
    if (FAILED(hr))
    {
        BIASERR( "Failed to get video interfaces! ");
        return hr;
    }

    // Use the system device enumerator and class enumerator to find
    // a video capture/preview device, such as a desktop USB video camera.
    hr = FindCaptureDevice_(&pSrcFilter_);
    if (FAILED(hr))
    {
        // Don't display a message because FindCaptureDevice will handle it
        return hr;
     }
    

    // Add Capture filter to our graph.
    hr = pGraphBuilder_->AddFilter(pSrcFilter_, L"Video Capture");
    if (FAILED(hr))
    {
        BIASERR("Couldn't add the capture filter to the graph! \n"<<
            "If you have a working video capture device, please make sure\n"<<
            "that it is connected and is not being used by another application.");
        pSrcFilter_.Release();
        return hr;
    }

     // get the output pin of the Capture device filter
	  if(pSrcFilter_->FindPin(L"Capture",&pOutPin_)!=S_OK)
   	  if (pSrcFilter_->FindPin(L"0",&pOutPin_)!=S_OK)
	   	  if (pSrcFilter_->FindPin(L"Out",&pOutPin_)!=S_OK)
	    	  if (pSrcFilter_->FindPin(L"1",&pOutPin_)!=S_OK)
		    	  throw exception("Could not find Source Outpin");

    if( pOutPin_ )
    {
        cout<< "Found VideoSource Outpin" << endl;
        if(bShowDialogs_)
          DisplayPinProperties_( pOutPin_ );
    }
 
    //DisplayCameraProperties();
    
    //here determine size and set
    IAMStreamConfig * pConfig=0;
    hr = pCaptureGraphBuilder2_->FindInterface(&PIN_CATEGORY_CAPTURE,
                                      &MEDIATYPE_Video,
                                      pSrcFilter_, IID_IAMStreamConfig, (void **)&pConfig);

    if(hr != NOERROR)
    {
        hr = pCaptureGraphBuilder2_->FindInterface(&PIN_CATEGORY_CAPTURE,
                                          &MEDIATYPE_Interleaved, pSrcFilter_,
                                          IID_IAMStreamConfig, (void **)&pConfig);
    }

    // define two media types to detect correct one
	  AM_MEDIA_TYPE *pSourceVideoFormat = NULL, * pNewVideoFormat = NULL;
    if((hr = pConfig->GetFormat(&pSourceVideoFormat)) == S_OK)
    {
        // Examine the media type for any information you need. 
       	if(pSourceVideoFormat->majortype == MEDIATYPE_Video)
        {
    	    MediaSubTypeColorModel_ = pSourceVideoFormat->subtype;
          if(MediaSubTypeColorModel_ == MEDIASUBTYPE_RGB32){
            MediaSubTypeColorModel_ = MEDIASUBTYPE_RGB24;
            pSourceVideoFormat->subtype = MediaSubTypeColorModel_;
          }         
        }
        else{  
          //get mediatype from videosource
          BIASERR("The major format type is NOT video");
        }
        DetermineBIASColorModel_(); //determine corresponding BIAS ColorModel      
      
        pSourceVideoFormat->formattype = FORMAT_VideoInfo; 
        VIDEOINFOHEADER *vih = (VIDEOINFOHEADER *)pSourceVideoFormat->pbFormat;
        vih->bmiHeader.biWidth = GetWidth();
        vih->bmiHeader.biHeight = GetHeight();
        vih->bmiHeader.biBitCount = (WORD)GetColorChannels()*8; 
		    hr = pConfig->SetFormat( pSourceVideoFormat );
        if(FAILED(hr))
          cout<<"Error Videoformat is inconsistent"<<endl;
    }
    DeleteMediaType_(pSourceVideoFormat);

    pConfig->Release();

    IEnumMediaTypes * pMedia=0;
    vector<AM_MEDIA_TYPE *> vpfnt;

    //outpin is outpin from pSrcFilter_
	  hr = pOutPin_->EnumMediaTypes( &pMedia );
	  if( (hr = pOutPin_->EnumMediaTypes( &pMedia )) == S_OK)
	  {
		  while(pMedia->Next(1, &pSourceVideoFormat, 0) == S_OK)
		  {
        if( pSourceVideoFormat->formattype == FORMAT_VideoInfo )
			  {
          VIDEOINFOHEADER *vih = (VIDEOINFOHEADER *)pSourceVideoFormat->pbFormat;
          int dCurWidth = vih->bmiHeader.biWidth;
          int dCurHeight = vih->bmiHeader.biHeight;
         
          if( dCurWidth == GetWidth() && dCurHeight == GetHeight() )// && sfcc == "IYUV")
				  {
            // this is the correct new Video Format
            pNewVideoFormat = pSourceVideoFormat;
          }
        }      
        else if( pSourceVideoFormat->formattype == FORMAT_None )
            cout<<"Format type: FORMAT_None"<<endl;
        else if( pSourceVideoFormat->formattype == FORMAT_DvInfo )
             cout<<"Format type: FORMAT_DvInfo"<<endl;
        else if( pSourceVideoFormat->formattype == FORMAT_MPEGVideo )
          cout<<"Format type: FORMAT_MPEGVideo"<<endl;
        else if( pSourceVideoFormat->formattype == FORMAT_MPEG2Video )
             cout<<"Format type: FORMAT_MPEG2Video"<<endl;
        else if( pSourceVideoFormat->formattype == FORMAT_VideoInfo2 )
              cout<<"Format type: FORMAT_VideoInfo2"<<endl;
      }
		  pMedia->Release();
    }

    //finally set the new videoFormat
		if( pNewVideoFormat != 0 )
		{
			pConfig->SetFormat( pNewVideoFormat );
			int dWidth = ((VIDEOINFOHEADER *)pNewVideoFormat->pbFormat)->bmiHeader.biWidth;
			int dHeight = ((VIDEOINFOHEADER *)pNewVideoFormat->pbFormat)->bmiHeader.biHeight;
      double dDepth = ((VIDEOINFOHEADER *)pNewVideoFormat->pbFormat)->bmiHeader.biBitCount;
      if(dDepth == 12)
        dDepth = 2;
      else
        dDepth /= 8;
      SetSizeAndColorModel(dWidth,dHeight,(int)dDepth,BIASColorModel_);	
			DeleteMediaType_( pNewVideoFormat );
    }
    
    // add sample grabber 
    CComQIPtr< IBaseFilter, &IID_IBaseFilter > pGrabberBase( pGrabber_ );
	  if ( (hr =  pGraphBuilder_->AddFilter(pGrabberBase,L"SampleGrabber")) != S_OK)
    {
          BIASERR("Couldn't add the grab filter.");
	  }
    //set mediatype to sample grabber
    hr = pConfig->GetFormat( &pNewVideoFormat );
    if( (hr = pGrabber_->SetMediaType(pNewVideoFormat)) != S_OK )
    {
		  BIASERR("Setting Media Type failed!");
	  }   
    DeleteMediaType_( pNewVideoFormat ); 
   

         // get the SampleGrabber Input Pin
	  if (pGrabberBase->FindPin(L"In",&pInPin_) != S_OK)
		  cout << "Could not find SampleGrabber Input Pin" << endl;

	  // connect SmartTee and SampleGrabber
	  if ((hr = pGraphBuilder_->Connect(pOutPin_,pInPin_))!=S_OK)
		  cout << "Could not not connect Camera Source and SampleGrabber Pins" << endl;
    else
      cout<< "Succesfully connected Camera Source and SampleGrabber Pins" <<endl;

    // add the NullRenderer filter
	  hr = pGraphBuilder_->AddFilter(pNullRenderer_,L"NullRenderer");
	  if (FAILED(hr))
      {
          BIASERR("Couldn't add the grab filter.");
	  }

   // get the SampleGrabber Output Pin
	if (pGrabberBase->FindPin(L"Out",&pSampleOutPin_)!=S_OK)
		cout<<"Pin out of SampleGrabber not found"<<endl;

	// get the NullRenderer Input Pin
	if (pNullRenderer_->FindPin(L"In",&pInPin_)!=S_OK)
		cout<< "Could not find NullRenderer Input Pin" <<endl;
	
	// connect SampleGrabber and NullRenderer
	if (pGraphBuilder_->Connect(pSampleOutPin_,pInPin_)!= S_OK)
		cout<< "Could not not connect SampleGrabber and NullRenderer Pins" <<endl;
  else
  	cout<<"Succesfully connected SampleGrabber and NullRenderer Pins" << endl;
  
	pGrabber_->SetBufferSamples(FALSE);
	pGrabber_->SetOneShot(FALSE);
  if(FGCB == NULL)
    FGCB = new MySampleGrabberCB(GetWidth(),GetHeight(),GetColorChannels());
  pGrabber_->SetCallback(FGCB,0);
  
  EnumFilters_(pGraphBuilder_);
  
  // Start video data source
	hr = pMediaControl_->Run();    
	if (FAILED(hr))
    {
        BIASERR("Couldn't run the graph!  ");
        return hr;
    }   
 
  //SaveGraphFile_(pGraphBuilder_,L"C:\\MyGraf.grf");
  return S_OK;
}

HRESULT VideoSource_DSHOW::GetInterfaces_(void)
{
    HRESULT hr;

    // Create the filter graph
    hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER,
                           IID_IGraphBuilder, (void **) &pGraphBuilder_);
    if (FAILED(hr))
        return hr;

    // Create the capture graph builder
    hr = CoCreateInstance(CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC,
                           IID_ICaptureGraphBuilder2, (void **) &pCaptureGraphBuilder2_);
    if (FAILED(hr))
        return hr;
    // Attach the filter graph to the capture graph
    hr = pCaptureGraphBuilder2_->SetFiltergraph(pGraphBuilder_);
    if (FAILED(hr))
    {
        BIASERR("Failed to set capture filter graph!");
        return hr;
    }
   
    // Obtain interfaces for media control and Video Window
    hr = pGraphBuilder_->QueryInterface(IID_IMediaControl,(LPVOID *) &pMediaControl_);
    if (FAILED(hr))
        return hr;

    hr = pGraphBuilder_->QueryInterface(IID_IMediaEvent, (LPVOID *) &pMediaEvent_);
    if (FAILED(hr))
        return hr;

	  // Create NullRenderer Instance
	  hr = pNullRenderer_.CoCreateInstance(CLSID_NullRenderer);
	  if (FAILED(hr))
          return hr;

	  // Create SampleGrabber Instance 
	  // ISampleGrabber can Queried on this Filter to Grab Samples from the running Graph
	  hr = pGrabber_.CoCreateInstance(CLSID_SampleGrabber);
	  if(FAILED(hr))
	  {
		  MessageBox( GetDesktopWindow(), "Could not create the SampleGrabber filter.", "ImageSourceDShow", MB_OK | MB_ICONERROR ); 
          exit( -1 );
    }

    return hr;
}

HRESULT VideoSource_DSHOW::FindCaptureDevice_(IBaseFilter ** ppSrcFilter)
{
    HRESULT hr;
    IBaseFilter * pSrc = NULL;
    CComPtr <IMoniker> pMoniker =NULL;
    ULONG cFetched=0;
    
    if (!ppSrcFilter)
        return E_POINTER;
   
    // Create the system device enumerator
    CComPtr <ICreateDevEnum> pDevEnum =NULL;

    hr = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC,
                           IID_ICreateDevEnum, (void **) &pDevEnum);
    if (FAILED(hr))
    {
        BIASERR("Couldn't create system enumerator!");
        return hr;
    }

    // Create an enumerator for the video capture devices
    IEnumMoniker* pClassEnum = 0;

    hr = pDevEnum->CreateClassEnumerator (CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
    if (FAILED(hr))
    {
        BIASERR("Couldn't create class enumerator! Now DirectShow Hardware?");
        return hr;
    }

    //////////////////// new
    pClassEnum->Reset();
    IMoniker *pM =0;
    while(hr = pClassEnum->Next(1, &pM, &cFetched), hr==S_OK )
    {
		IPropertyBag *pBag;
		hr = pM->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pBag);
		if(SUCCEEDED(hr)) 
		{
			VARIANT var;
			var.vt = VT_BSTR;
      hr = pBag->Read(L"FriendlyName", &var, NULL);
			if (hr == NOERROR) 
			{
				string cDevName = (char*)((_bstr_t)(var.bstrVal));
        
        // if the string is empty, take first device
        // if device is equal to defined string take it as well
        if(sCaptureDeviceName_ == "" ||
          sCaptureDeviceName_.compare(cDevName)==0)
        {
				  cout<< "Found CaptureDevice: " << cDevName.c_str() <<endl;
				  hr = pM->BindToObject(0, 0, IID_IBaseFilter, (void**)&pSrc);
          if(!SUCCEEDED(hr))  
            cout<< "Failed to bind to object!" <<endl;
          else
            cout<< "Succesfully bound to object!" <<endl;
          break;
        }
        else
            cout<<"The defined device is not found taking next.."<<endl;    
				
				SysFreeString(var.bstrVal);
			}
			pBag->Release();
		}
		pM->Release();
    }
    *ppSrcFilter = pSrc;

    return hr;
}

HRESULT VideoSource_DSHOW::EnumFilters_(IFilterGraph *pGraph) 
{
    IEnumFilters *pEnum = NULL;
    IBaseFilter *pFilter;
    ULONG cFetched;

    HRESULT hr = pGraph->EnumFilters(&pEnum);
    if (FAILED(hr)) return hr;

    while(pEnum->Next(1, &pFilter, &cFetched) == S_OK)
    {
        FILTER_INFO FilterInfo;
        hr = pFilter->QueryFilterInfo(&FilterInfo);
        if (FAILED(hr))
        {
            MessageBox(NULL, TEXT("Could not get the filter info"),
                TEXT("Error"), MB_OK | MB_ICONERROR);
            continue;  // Maybe the next one will work.
        }

        // The FILTER_INFO structure holds a pointer to the Filter Graph
        // Manager, with a reference count that must be released.
        if (FilterInfo.pGraph != NULL)
        {
            FilterInfo.pGraph->Release();
        }
        if( bShowDialogs_ )
            DisplayFilterProperties( pFilter );

      /*  IPin * pPin = GetPin( pFilter, PINDIR_INPUT );
        pPin->Release();*/

        pFilter->Release();
    }

    pEnum->Release();
    return hr;
}

// Pass it a file name in wszPath, and it will save the filter graph
// to that file.
HRESULT VideoSource_DSHOW::SaveGraphFile_(IGraphBuilder *pGraph, WCHAR *wszPath) 
{
    const WCHAR wszStreamName[] = L"ActiveMovieGraph"; 
    HRESULT hr;
    IStorage *pStorage = NULL;

    // First, create a document file that will hold the GRF file
    hr = StgCreateDocfile(
         wszPath,
         STGM_CREATE | STGM_TRANSACTED | STGM_READWRITE | 
            STGM_SHARE_EXCLUSIVE,
         0, &pStorage);
    if(FAILED(hr)) 
    {
        return hr;
    }

    // Next, create a stream to store.
    IStream *pStream;
    hr = pStorage->CreateStream(
         wszStreamName,
         STGM_WRITE | STGM_CREATE | STGM_SHARE_EXCLUSIVE,
         0, 0, &pStream);
    if (FAILED(hr)) 
    {
        pStorage->Release();    
        return hr;
    }

    // The IpersistStream::Save method converts a stream
    // into a persistent object.
    IPersistStream *pPersist = NULL;
    pGraph->QueryInterface(IID_IPersistStream, 
                           reinterpret_cast<void**>(&pPersist));
    hr = pPersist->Save(pStream, TRUE);
    pStream->Release();
    pPersist->Release();
    if (SUCCEEDED(hr)) 
    {
        hr = pStorage->Commit(STGC_DEFAULT);
    }
    pStorage->Release();
    return hr;
}


////////////////////////////////////////////////////////////////////////   

//constructor
MySampleGrabberCB::MySampleGrabberCB(int width, int height, int depth)
{
  if(!(camera_.GetImageData()==NULL))
    camera_.Release();
  camera_.Init(width,height,depth);
}

/****************************************************************/

MySampleGrabberCB::~MySampleGrabberCB() {
  if(!(camera_.GetImageData()==NULL))
    camera_.Release();
}
/****************************************************************/
STDMETHODIMP MySampleGrabberCB::BufferCB(double SampleTime, BYTE *pBuffer, long BufferLen)
{
  unsigned char *pImageData = camera_.GetImageData();
  memcpy(pImageData,pBuffer,BufferLen);
  return S_OK;
}

/****************************************************************/
STDMETHODIMP MySampleGrabberCB::SampleCB(double SampleTime, IMediaSample *pSample)
{
  unsigned char *pImageData = camera_.GetImageData();
  BYTE *pSampleData;
  pSample->GetPointer(&pSampleData);
  long size = pSample->GetSize();
  
  memcpy(pImageData,pSampleData,size);
 
	return S_OK;
}

/****************************************************************/

STDMETHODIMP MySampleGrabberCB::QueryInterface(REFIID riid, void ** ppv) {

	if( riid == IID_ISampleGrabberCB || riid == IID_IUnknown ) {
        *ppv = (void *) static_cast<ISampleGrabberCB*> ( this );
        return NOERROR;
    }    
    return E_NOINTERFACE;
}



