/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIAS_VIDEOSOURCE_V4L_HH__
#define __BIAS_VIDEOSOURCE_V4L_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_V4L
#  error Please recompile MIP with BUILD_V4L to use VideoSource V4L
#endif 
#ifndef BIAS_HAVE_PTHREADS
#  error BIAS_HAVE_PTHREADS not defined. But BIAS::ImageCanvas needs pthreads (on Unix and Win32). Please enable USE_PTHREADS in CMake configure step.
#endif

#include <VideoSource/VideoSource_Base.hh>

#include <sys/time.h>
#include <linux/videodev.h>
#include <pthread.h>




namespace BIAS{
/** \class VideoSource_V4L 
    \ingroup VideoG
    \author evers, v4l2 by grest
    \brief This class extends VideoSource for the use of Video4Linux 
    supported devices like framegrabber, USB cameras.
*/
  class VideoSource_V4L : public VideoSource{
  
 
  public: 
    VideoSource_V4L();
    virtual ~VideoSource_V4L();
  
    int ShowCapabilities();
    int ShowCapabilities(const char *device);
  
    virtual int OpenDevice();

    virtual int OpenDevice(const char *device);
    virtual int CloseDevice();
  
    /** blocks until a new frame arrives from the cam,
        if dont want blocking set waitForNew() to false
    */
    virtual int GrabSingle(BIAS::Camera <unsigned char> &image);
                       
    /** If you want as many images as possbile, and dont care about equal ones
        call this with "false". Default is that GrabSingle blocks until
        a new frame arrives from the camera.
        @author grest, Oct. 2005
     */
    inline void SetWaitForNew(bool waitForNew) { waitForNew_=waitForNew; }

    virtual int StartGrabContinous();

    virtual int StopGrabContinous();

  protected:
    int fd;

    struct v4l2_requestbuffers reqBuf;
    struct v4l2_capability   capability2;
    struct v4l2_format       format2_;
    struct v4l2_buffer buffer_;
    unsigned int numBuffers_;
    int bufferLastCaptured_;

    int waitForNew_;
    std::vector<int> bufLengths; // remember for unmap
    std::vector<char*> bufStarts; // remember for unmap

  
    bool GrabbingContinous_;
    pthread_t ContinousThread_;
    pthread_cond_t ThreadCondition_;
    pthread_mutex_t ActiveFrameMutex_;

    struct timeval GrabTime_;

    static void *GrabContThread_(void * arg);

    bool newFrame_; 

  protected:
    // for doublebuffering this swich is used to select the frame
    int ActiveFrame ;
  
  
  };

} // namespace MIP


#endif
